/**
 * @file
 * Renders job listings from Lever in a div with a lever ID.
 */

(function() {
  'use strict';

  var DEBUG = false;
  var TOKEN = 'teachable';
  var ENDPOINT = 'https://api.lever.co/v0/postings/' + TOKEN + '?group=department&mode=json';

  /**
   * If debug is set to true, this logs warnings to the console.
   *
   * @param {String} msg
   */
  function warn(msg) {
    if (DEBUG) {
      console.warn(msg);
    }
  }

  /**
   * @class Lever
   * @param {HTMLElement} $el
   */
  function Lever($el) {
    this.$el = $el;
    this.init();
  }

  Lever.prototype = {
    /**
     * Fetches data from Lever and kicks off the sorting and rendering.
     */
    init: function() {
      $.ajax({
        url: ENDPOINT,
        success: $.proxy(function(data) {
          this.render(data);
        }, this),
        error: function() {
          $('#lever-fail').show();
        }
      });
      $('.job-listings .spinner').hide();
    },

    /**
     * Takes in the response from Lever then renders the job board.
     *
     * @param {Object} data
     */
    render: function(data) {
      var template = '';
      data.forEach($.proxy(function(department) {
        var jobs = department.postings;
        var title = department.title;
        if (title && jobs.length > 0) {
          template += this.createPostingMarkup(title, jobs);
        }
      }, this));
      this.$el.innerHTML = '<div class="modules">' + template;
    },

    /**
     * Create the HTML for each posting.
     *
     * @param {String} title
     * @param {Array} jobs
     * @returns {String}
     */
    createPostingMarkup: function(title, jobs) {
      var id = title.replace(/\s+/g, '-').replace(/[^a-zA-Z-]/g, '').toLowerCase();
      var className = title.replace(/\s+/g, '-').replace(/[^a-zA-Z-]/g, '').toLowerCase();
      return (
        '<div class="' + className + ' module no-team-default" style="text-align: left;">' +
        this.getHeaderMarkup(title, className) +
        this.getJobsMarkup(jobs, className) +
        '</div>'
      );
    },

    /**
     * Each department needs its own header. This creates that markup.
     *
     * @param   {String} name
     * @returns {String}
     */
    getHeaderMarkup: function(name, className) {
      return '<div class="job-department-color department-label title ' + className + ' ">' + name + '</div>';
    },

    /**
     * Takes in a list of jobs, returns markup.
     *
     * @param   {Array} jobs
     * @returns {String}
     */
    getJobsMarkup: function(jobs, className) {
      return jobs.map(function(job) {
        var categories = job.categories;
        return '<a ' +
          'href="/job_posts/' + job.id + '"' +
          'class="item job-department-color job-post-link ' + className + '"' +
        '>' +
          '<div class="item-title">' + job.text + '</div>' +
          '<div class="left-text">' +
            '<span class="arrow"> &gt; </span>' +
          '</div>' +
        '</a>';
      }).join('');
    }
  };

  // Go!
  $(function() {
    var $lever = document.getElementById('lever');

    if (!$lever) {
      warn('Lever $el not found.');
      return;
    }

    new Lever($lever);
  });
})();
