class Admin::Api::BlogPostsController < AdminController

  expire_page :@blog_post, only: [:update, :destroy]

  def index
    @blog_posts = BlogPost.all.includes(:author)
    render 'index.json.jbuilder'
  end

  def create
    @blog_post = BlogPost.new(blog_post_params)
    @blog_post.url = '/' + @blog_post.title.parameterize
    @blog_post.canonical_url = 'https://teachable.com/blog' + @blog_post.url
    if @blog_post.save
      render 'create.json.jbuilder'
    else
      render json: @blog_post.errors.full_messages, status: 422
    end
  end

  def show
    @blog_post = BlogPost.includes(:author, :cover_photo, :categories, :mobile_cover_photo).find(params[:id])
    gather_additional_data_for_show
    render 'show.json.jbuilder'
  end

  def update
    @blog_post = BlogPost.find(params[:id])
    ActiveRecord::Base.transaction do
      updated_post = @blog_post.update(blog_post_params) # update blog before categories for cache clearing
      category_params = params[:categories] || []
      update_blog_post_categories!(category_params)
      if updated_post
        @blog_post = BlogPost.includes(:author, :cover_photo, :categories).find(params[:id])
        gather_additional_data_for_show
        Rails.cache.clear # clear the cache so updates appear properly
        render 'show.json.jbuilder'
      else
        render json: @blog_post.errors.full_messages, status: 422
      end
    end
  end

  def destroy
    @blog_post = BlogPost.find(params[:id])
    @blog_post.destroy
    render json: @blog_post
  end

  private

  def gather_additional_data_for_show
    @all_categories = ContentCategory.general.map(&:name_and_value)
    user_authors = User.all.map do |user|
      {
        displayText: user.name,
        value: user.id,
        polymorphicTypeName: 'authorType',
        polymorphicTypeValue: 'User'
      }
    end
    guest_authors = Objekt.guest_authors.map do |guest_author|
      {
        displayText: guest_author.field_values_hash["Name"],
        value: guest_author.id,
        polymorphicTypeName: 'authorType',
        polymorphicTypeValue: 'Objekt'
      }
    end
    @authors = user_authors.concat(guest_authors)
  end

  def create_blog_post_categories!(category_ids)
    category_ids.each do |id|
      BlogPostCategory.create!(blog_post_id: params[:id], blog_category_id: id)
    end
  end

  def update_blog_post_categories!(received_category_ids)
    current_category_ids = @blog_post.categories.map(&:id)
    received_category_ids.map!(&:to_i) unless received_category_ids && received_category_ids.all? { |id| id.class == Integer }
    new_category_ids = received_category_ids - current_category_ids
    discarded_category_ids = current_category_ids - received_category_ids
    new_category_ids.each do |id|
      BlogPostCategory.create!(blog_post_id: @blog_post.id, blog_category_id: id)
    end
    discarded_categories = BlogPostCategory.where(blog_post_id: @blog_post.id, blog_category_id: discarded_category_ids)
    discarded_categories.each { |discarded_category| discarded_category.destroy }
  end

  def blog_post_params
    params.require(:blog_post).permit(
      :id, :title, :title_text, :url, :description, :canonical_url, :author_id, :active,
      :cover_photo_id, :mobile_cover_photo_id, :cover_photo_alt_text, :index_page, :date, :editor_content_state,
      :html_content, :author_type, categories: []
    )
  end
end
