# frozen_string_literal: true

class Admin::Api::TrackingLinksController < AdminController

  def index
    @tracking_links = TrackingLink.includes(:creator).latest_versions
    render "index.json.jbuilder"
  end

  def new
    @plans = Plan.where.not(name: "All Plans").order(:order)
    render "new.json.jbuilder"
  end

  def copy
    @tracking_link = TrackingLink.find(params[:id])
    @plans = Plan.where.not(name: "All Plans").order(:order)
    render "copy.json.jbuilder"
  end

  def create
    @tracking_link = TrackingLink.new(tracking_link_params)
    @tracking_link.creator = @current_user
    set_audit_attributes(@tracking_link)
    if @tracking_link.save
      render "show.json.jbuilder", locals: {
        tracking_link: @tracking_link,
      }, status: 201
    else
      render json: @tracking_link.errors.full_messages, status: 422
    end
  end

  def show
    @tracking_link = TrackingLink.includes(:creator).latest_versions.find(params[:id])
    @plans = Plan.where.not(name: "All Plans").order(:order)
    render "show.json.jbuilder"
  end

  def update
    @tracking_link = TrackingLink.find(params[:id])
    @tracking_link = TrackingLinks::Updater.call(tracking_link: @tracking_link, update_attributes: tracking_link_params, updater: @current_user)
    if @tracking_link.errors.none?
      render "show.json.jbuilder"
    else
      render json: @tracking_link.errors.full_messages, status: 422
    end
  end

  def destroy
    tracking_link = TrackingLink.find(params[:id])
    tracking_link.destroy!
    render json: tracking_link
  end

  def export
    job = Job.create(name: "export tracking links", text: "Exporting Tracking Links", total_value: TrackingLink.count)
    ExportTrackingLinks.perform_async(job.id)
    render json: { job: job.serializable_hash.deep_transform_keys! { |k| k.camelize(:lower) } }, status: 200
  end

  def test
    url = params.require(:url)

    # maybe use head instead?
    begin
      res = HTTParty.get(url,
        headers: {
          'X-CMS-AUTH': ENV.fetch("CLOUDFLARE_BYPASS_AUTH", "secret")
        }
      )
    rescue
      return render json: {
        status: "ERROR",
        redirect_url: url
      }
    end

    if (200..299) === res.code
      status = if url == res.request.last_uri.to_s then "OK" else "REDIRECT" end
    else
      status = "ERROR"
    end

    render json: {
      status: status,
      redirect_url: res.request.last_uri
    }
  end

  private

  # This makes me think we should move this to a service
  def set_audit_attributes(tracking_link)
    return unless tracking_link.status_changed?
    if tracking_link.published?
      tracking_link.published_at = Time.now
      tracking_link.publisher = @current_user
    end
    # if this request is deactivating the link record the time and user
    if tracking_link.hidden?
      tracking_link.hidden_at = Time.now
      tracking_link.hider = @current_user
    end
  end

  def tracking_link_params
    permitted_params = params.require(:tracking_link).permit(
      :destination_url,
      :slug,
      :description,
      :status,
      :forward_all_query_params,
      forward_query_params_whitelist: [],
      metadata: {},
      additional_query_params: {}
    )
    defaults = { metadata: {}, forward_query_params_whitelist: [], additional_query_params: {} }
    defaults.merge(permitted_params)
  end

  def ever_published_tracking_link_params
    params.require(:tracking_link).permit(:description, :status)
  end

end
