# frozen_string_literal: true

class Admin::Api::WebinarsController < AdminController

  include UpdateIterableMetadata

  after_action :update_iterable_webinar, only: [:create, :update]

  def index
    @webinar_times = Webinar.all_times
    render 'index.json.jbuilder'
  end

  def create
    # use the current datetime for testing to ensure that it gets its own
    # firebase ref. Should be fine until the year 2038
    if APP_ENV == "test"
      @webinar = Webinar.new({:id => DateTime.now.to_i}.merge(webinar_params))
    else
      @webinar = Webinar.new(webinar_params)
    end
    if @webinar.save
      @webinar.create_in_firebase
      render 'create.json.jbuilder'
    else
      render json: @webinar.errors.full_messages, status: 422
    end
  end

  def copy
    @old_webinar = Webinar.find(params[:id])
    @webinar = @old_webinar.dup
    @webinar.name = webinar_params[:name]
    @webinar.url = webinar_params[:url]
    if @webinar.save
      @webinar.create_in_firebase
      gather_data_for_show_json
      render 'show.json.jbuilder'
    else
      render json: @webinar.errors.full_messages, status: 422
    end
  end

  def show
    @webinar = Webinar.find_by_id(params[:id])
    gather_data_for_show_json
    render 'show.json.jbuilder'
  end

  def update
    @webinar = Webinar.find(params[:id])
    params = webinar_params
    params.delete(:time) if @webinar.recurring
    if @webinar.update(params)
      gather_data_for_show_json
      render 'show.json.jbuilder'
    else
      render json: @webinar.errors.full_messages, status: 422
    end
  end

  def destroy
    @webinar = Webinar.find(params[:id])
    @webinar.destroy
    @webinar.delete_from_firebase
    render json: @webinar
  end

  def upload_replay
    unless params[:youtube_id].strip.empty?
      @webinar = Webinar.find_by_id(params[:webinar_id])
      firebase = Firebase::Client.new(Webinar.get_firebase_url, Webinar.firebase_auth_json)
      firebase.update('', {
        "#{@webinar.id}/controls/replayVideoStatus" => 'scheduled'
      })
      slack = Slack::Web::Client.new
      slack.chat_postMessage(channel: ENV['WEBINAR_NOTIFICATIONS_CHANNEL'], text: "Replay upload has been scheduled for \"#{@webinar.title}\"", as_user: true)
      ProcessReplay.perform_in(1.hour, params[:webinar_id], params[:youtube_id])
      render json: { message: 'upload replay job scheduled' }, status: 200
    else
      render json: { error: 'missing youtube_id param' }, status: 422
    end
  end

  private

  def gather_data_for_show_json
    @proper_time = @webinar.proper_time
    @bundles = Template.find_by_name('Webinar Bonus Bundle').objects
    @downloads = @webinar.webinar_downloads
    @other_downloads = Objekt.where(template_id: Template.find_by_name('Webinar Download').id) - @downloads.map(&:objekt)
    @hosts = @webinar.webinar_hosts
    @other_hosts = Objekt.where(template_id: Template.find_by_name('Webinar Host/Inviter').id) - @hosts.map(&:objekt)
    @offers = @webinar.webinar_offers.order(:start_time)
  end

  def webinar_params
    params[:webinar].permit(
      :name, :title, :url, :time, :time2, :host_id, :inviter_id, :header, :subheader, :bullet_points, :thank_you_video_id,
      :upgrade_timeframe, :upgrade_options, :basic_replay_bundle, :basic_play_bundle, :pro_replay_bundle, :pro_play_bundle,
      :replay_video_id, :custom_modal_image, :redirect, :live_start_date, :live_end_date, :custom_utm_campaign, :custom_utm_content,
      :coupon_code, :custom_mt_campaign, :faq, :sidebar_options, :custom_cta_url, :custom_cta_text, :platform, :automation_interval,
      :webinar_type, :automated_host_header, :automated_host_description, :automated_host_image_url, :automated_hero_image_url, :automated_post_event_redirect,
      :description, :display_thank_you_free_account_cta, :reg_page_warning, :expected_duration, :automated_cta_header
    )
  end

end
