class Banner < ApplicationRecord

  enum inclusion_method: { whitelist: 1, blacklist: 2 }

  validates :name, :banner_key, presence: true

  has_many :banner_fields, dependent: :destroy
  has_many :banner_pages, dependent: :destroy
  has_many :banner_webinars, dependent: :destroy
  has_many :pages, through: :banner_pages
  has_many :webinars, through: :banner_webinars

  after_create :create_banner_fields

  scope :active, -> { where(active: true) }

  def self.keys
    YAML.load_file(Rails.root.join("config", "banners.yml")).keys
  end

  def banner_config
    YAML.load_file(Rails.root.join("config", "banners.yml"))[banner_key]
  end

  def create_banner_fields
    fields_to_create = banner_config["fields"]
    fields_to_create.each do |key, _value|
      BannerField.create!(banner_id: id, banner_field_key: key)
    end
  end

  def publish
    update!(active: true)
    clear_cache!
  end

  def unpublish
    update!(active: false)
    clear_cache!
  end

  def clear_cache!
    Rails.cache.clear
  end

  def field_values_hash(args = {})
    fields = banner_config["fields"]
    fields.map.with_object({}) do |(field_key, field_data), obj|
      field = banner_fields.find_by(banner_field_key: field_key)
      field_value = field.present? ? field.value : default_field_data(field_data)
      if field_data["type"] == "DateTime"
        obj[field_key] = args[:extend_datetime] ? (Time.zone.parse(field_value) || "") : field_value
      else
        obj[field_key] = field_value
      end
    end
  end

  def self.banner_from_path(args)
    current_path = args[:path]
    if current_path.ends_with?("/preview") && current_path.split("/").include?("pages")
      page_id = current_path.split("/")[-2]
      url = Page.find(page_id).url
    else
      url = current_path
    end
    return false if ["/create-account", "/sign_in", "/session"].include?(url)
    active_banners = Banner.active
    if active_banners.present?
      active_banners.each do |banner|
        if args[:is_webinar_page]
          banner_urls = []
          banner.webinars.each do |webinar|
            banner_urls << "/webinars/#{webinar.url}/register"
            banner_urls << "/webinars/#{webinar.url}/thank-you"
            banner_urls << "/webinars/#{webinar.url}/play"
            banner_urls << "/webinars/#{webinar.url}/replay"
          end
        else
          banner_urls = banner.pages.pluck(:url)
        end
        if banner.inclusion_method == "whitelist" && banner_urls.include?(url)
          return banner
        elsif banner.inclusion_method == "blacklist" && !banner_urls.include?(url)
          return banner
        end
      end
    end
    return nil
  end

  def default_field_data(field_config)
    if field_config["default_value"].present?
      field_config["default_value"]
    else
      case field_config["type"]
      when "Boolean"
        false
      else
        ""
      end
    end
  end

end
