class BlogPost < ApplicationRecord

  include StandardPageValidations
  include Metadatable

  validates :title, :author_id, presence: true
  validates :title, uniqueness: true
  validates_datetime :date, allow_blank: true

  has_many :blog_post_categories, dependent: :destroy
  has_many :categories, through: :blog_post_categories

  belongs_to :author, polymorphic: true
  belongs_to(
    :cover_photo,
    class_name: "Upload",
    primary_key: :id,
    foreign_key: :cover_photo_id,
    dependent: :destroy
  )

  belongs_to(
    :mobile_cover_photo,
    class_name: "Upload",
    primary_key: :id,
    foreign_key: :mobile_cover_photo_id,
    dependent: :destroy
  )

  scope :published, -> { where(active: true) }

  before_save do
    @previous_url = changes.dig("url", 0)
    @previous_active = changes.dig("active", 0)
    @category_ids_before_update = self.category_ids
  end

  before_destroy :store_ids, prepend: true

  @@default_author_image_url = ActionController::Base.helpers.image_path("admin/default-user-image.jpg")

  def store_ids
    @destroyed_category_ids = self.category_ids
  end

  def publish
    self.update(active: true, index_page: true)
  end

  def unpublish
    self.update(active: false)
  end

  def author_info
    author = self.author
    author_info_obj = { id: self.author_id.to_s }
    if self.author_type == "User"
      author_info_obj.merge({
        name: author.name,
        bio: author.bio,
        image_id: author.profile_pic.try!(:id),
        image_url: author.image_url
      })
    else
      field_values_hash = author.field_values_hash
      author_info_obj.merge({
        name: field_values_hash["Name"],
        bio: field_values_hash["Bio"],
        image_id: field_values_hash["Image"],
        image_url: Upload.find_by(id: field_values_hash["Image"]).try!(:url) || @@default_author_image_url
      })
    end
  end

  def full_url
    "https://teachable.com/blog#{url}"
  end

  def cover_photo_url
    cover_photo = self.cover_photo
    cover_photo ? ApplicationController.helpers.filestack_url(cover_photo) : ''
  end

  def mobile_cover_photo_url
    mobile_cover_photo = self.mobile_cover_photo
    mobile_cover_photo ? ApplicationController.helpers.filestack_url(mobile_cover_photo) : ''
  end

  def content_preview
    Nokogiri::HTML(self.html_content).text[0..200]
  end

  def formatted_date(type = 'admin')
    date = self.date
    if type == 'admin'
      date ? date.strftime("%-m/%-d/%y") : ''
    elsif type == 'public'
      date ? date.strftime("%b %d, %Y") : ''
    end
  end

  def formatted_cover_photo
    cover_photo = self.cover_photo
    cover_photo ? cover_photo.id_and_url : { id: '', url: '' }
  end

  def category_ids
    self.categories.map(&:id)
  end

  def get_draftjs_entities_by_type(entity_type)
    content_state = JSON.parse(self.editor_content_state)
    content_state
      .fetch("entityMap")
      .values
      .select { |entity| entity.fetch("type") == entity_type }
  end

  def cached_urls
    if self.active || @previous_active
      all_category_ids = @destroyed_category_ids || cached_category_ids

      blog_posts = BlogPost.joins(:blog_post_categories)
      .where.not(id: self.id)
      .where(active: true, blog_post_categories: { blog_category_id: all_category_ids })
      .group(:id)
      .pluck(:url)

      blog_posts << (@previous_url || url)
      return blog_posts.map { |url| "/blog#{url}" }.concat(['/blog', '/blog/resources'])
    end
    []
  end

  def cached_category_ids
    ((@category_ids_before_update || []) + BlogPost.find(id).category_ids).uniq
  end
end
