class ContentDownload < ApplicationRecord
  attr_reader :index_page
  include Metadatable

  belongs_to :upload
  belongs_to :content_category
  belongs_to :content_funnel_stage

  belongs_to :landing_page_background_image, class_name: "Upload"
  belongs_to :landing_page_large_image, class_name: "Upload"

  belongs_to :hero_form_background_image, class_name: "Upload"
  belongs_to :preview_card_image, class_name: "Upload"

  validates_presence_of :name, :content_category, :content_score, :content_funnel_stage
  validates_uniqueness_of :name
  validates_numericality_of :content_score

  scope :with_content_info, -> { includes(:upload, :content_category, :content_funnel_stage) }
  scope :with_landing_page_data, -> { includes(:upload, :landing_page_background_image, :landing_page_large_image) }
  scope :with_hero_form_data, -> { with_landing_page_data.includes(:hero_form_background_image) }
  scope :with_preview_card_data, -> { includes(:content_category, :preview_card_image) }
  scope :with_all_data, -> { with_content_info.with_landing_page_data.with_hero_form_data.with_preview_card_data }

  validates_presence_of :redemption_link,
    :title_text,
    :description,
    :landing_page_event_name,
    :landing_page_background_image,
    :landing_page_header,
    :landing_page_button_text,
    :landing_page_bullet_header,
    :landing_page_bullet_points,
    if: :active

  validates_format_of :landing_page_slug,
    with: /\A([a-z0-9]+-{0,1})*([a-z0-9]+)\z/,
    if: :active
  validates_uniqueness_of :landing_page_slug, if: :active

  enum content_type: {
    file: 0,
    external_link: 1
  }

  after_initialize :set_up_page_flags
  before_save do
    @previous_landing_page_slug = changes.dig("landing_page_slug", 0)
  end

  def cached_urls
    slug_to_clear = @previous_landing_page_slug || landing_page_slug
    urls = [
      "/get/#{slug_to_clear}",
      "/get/#{slug_to_clear}/thank-you",
    ]

    blog_objekt_fields = Objekt.blog_settings[0].objekt_fields
    is_featured_cd = blog_objekt_fields.any? { |of| of.field.field_type == "content_download" && of.value == "#{id}" }
    urls += ["/blog/resources", "/blog"] if is_featured_cd

    urls
  end

  def set_up_page_flags
    @index_page = true
  end

  def self.js_menu_items
    where(active: true).map { |content| { displayText: content.name, value: content.id } }
  end

  def redemption_link
    case content_type.to_sym
    when :file
      return upload ? upload.url : ''
    when :external_link
      return external_link
    end
  end

  def category_name
    content_category ? content_category.name : "None"
  end

  def funnel_stage
    content_funnel_stage.name
  end

  def landing_page_url
    "/get/#{landing_page_slug}"
  end

  def canonical_url
    "https://teachable.com/get/#{landing_page_slug}"
  end

  def landing_page_data
    {
      header: landing_page_header,
      background_image: image_url(landing_page_background_image),
      large_image: image_url(landing_page_large_image),
      button_text: landing_page_button_text,
      bullet_header: landing_page_bullet_header,
      event_name: landing_page_event_name,
      bullet_points: landing_page_bullet_points,
      redemption_link: redemption_link,
      slug: landing_page_slug
    }
  end

  def hero_form_data
    {
      header: hero_form_header,
      background_image: image_url(hero_form_background_image),
      button_text: hero_form_button_text
    }
  end

  def preview_card_data
    {
      image: preview_card_image.try!(:url) || ''
    }
  end

  private

  def image_url(image)
    return '' if image.nil?
    ApplicationController.helpers.filestack_url(image)
  end

end
