class Objekt < ApplicationRecord
  attr_accessor :images

  include Netlifiable

  validates :template_id, presence: true

  belongs_to :template

  has_many :fields, -> { joins(:field).order('fields.order') }, class_name: "ObjektField", foreign_key: :objekt_id, primary_key: :id, dependent: :destroy
  has_many :objekt_fields, -> { joins([:field]).order('fields.order') }, foreign_key: :objekt_id, primary_key: :id, dependent: :destroy
  has_many :template_fields, -> { preload(:template).order('fields.order') }, through: :template, source: :fields

  has_many :blog_posts, as: :author

  scope :with_images, -> {
    self.includes(:template_fields, :objekt_fields).each do |objekt|
      image_field_ids = objekt.template_fields.select { |f| f.field_type == "image" }.map(&:id)
      image_ids = objekt.objekt_fields.select { |f| image_field_ids.include?(f.field_id) }.map(&:value) - ['']
      images = Upload.where(id: image_ids)
      objekt.images = images
    end
  }

  scope :blog_settings, -> { preload(:objekt_fields, :template_fields).includes(:template).where(templates: { name: "Blog Settings" }) }
  scope :notifications, -> { includes(:fields, template: [:fields]).where(templates: { name: "Notification" }) }
  scope :guest_authors, -> { includes(:fields, template: [:fields]).where(templates: { name: "Guest Author" }) } do
    def find_by_name(author_name)
      self.select do |guest_author_obj|
        obj_field_id = guest_author_obj.template.fields.select { |field| field.name == "Name" }[0].id
        guest_author_obj.fields.any? do |obj_field|
          obj_field.field_id == obj_field_id && obj_field.value == author_name
        end
      end
    end
  end

  def object
    result = {id: id}
    fields = Field.where(template_id: template_id)
    objekt_fields = ObjektField.where("objekt_id = ? AND field_id IN (?)", id, fields.map { |field| field.id })
    objekt_fields.each do |objekt_field|
      result[objekt_field.field_id] = objekt_field.value
    end
    result
  end

  def fields_hash
    result = {}
    fields = Field.where(template_id: template_id).order(:order)
    objekt_fields = ObjektField.where("objekt_id = ? AND field_id IN (?)", id, fields.map { |field| field.id }).joins(:field).order('fields.order')
    objekt_fields.each_with_index do |objekt_field, index|
      result[fields[index].name] = objekt_field
    end
    result
  end

  def field_values_hash(args = {})
    defaults = {
      extend_file_urls: false,
      extend_image_urls: false,
      extend_objects: false,
      extend_page_partials: false,
      extend_webinars: false
    }
    args = defaults.merge(args)

    objekt_fields = self.objekt_fields
    template_fields = self.template_fields

    result = {}
    objekt_fields.each_with_index do |objekt_field, i|
      template_field = template_fields[i]
      if objekt_field.value.nil?
        value = nil
      elsif args[:extend_file_urls] && template_field['field_type'] == 'file'
        value = Upload.find(objekt_field.value).url
      elsif args[:extend_image_urls] && template_field['field_type'] == 'image'
        image = Upload.find(objekt_field.value)
        value = URI::encode("https://process.filestackapi.com/#{ENV['FILESTACK_API_KEY']}/auto_image/compress/quality=value:90/#{image.url.split('/')[-1]}")
      elsif args[:extend_objects] && template_field['field_type'] == 'object'
        value = objekt_field.instance.field_values_hash
      elsif args[:extend_webinars] && template_field['field_type'] == 'webinar'
        value = Webinar.find(objekt_field.value)
      elsif args[:extend_page_partials] && template_field['field_type'] == 'page-partial'
        value = objekt_field.nested_page_partial
      elsif template_field['field_type'] == 'content_download'
        value = ContentDownload.find(objekt_field.value)
      else
        value = objekt_field.value
      end
      result[template_field.name] = value
    end
    result
  end

  def id_and_first_field(id_key = :id, value_key = :first_field_value)
    result = { id: id }
    field_id = Field.where(template_id: template_id).order(:order).first.id
    first_field_value = ObjektField.find_by_objekt_id_and_field_id(id, field_id).value
    { id_key => id, value_key => first_field_value }
  end

  def cached_urls
    urls_to_clear = cached_cms_urls
    active_blog_posts = self.blog_posts.select { |post| post.active }
    active_blog_posts.each { |post| urls_to_clear << "/blog#{post.url}" }
    urls_to_clear << '/blog' if active_blog_posts.length > 0
    urls_to_clear.concat(['/blog', '/blog/resources']) if "Blog Settings" == template.name
    urls_to_clear
  end

  # clear all pages to increase speed
  def cached_cms_urls
    blog_objekt = blog_posts.length > 0 || "Blog Settings" == template.name
    return [] if blog_objekt
    Page.all_cached_urls
  end

  def page_partial_fields
    PagePartialField.joins(:partial_field).where(value: id, partial_fields: {field_type: 'object'})
  end
end
