class PagePartial < ApplicationRecord

  validates :partial_id, presence: true

  belongs_to :page
  belongs_to :partial
  belongs_to :page_partial_field

  has_many :partial_fields, -> { order('partial_fields.order') }, through: :partial
  has_many :page_partial_fields, -> { joins(:partial_field).order('partial_fields.order') }, dependent: :destroy

  def partial_name
    self.partial.name
  end

  def self.proper_number_of_fields
    page_partial_fields = 0
    Partial.all.includes(:partial_fields, :page_partials).each do |partial|
      instances = partial.page_partials.length
      partial_fields = partial.partial_fields.length
      page_partial_fields += (instances * partial_fields)
    end
    page_partial_fields
  end

  def self.check_for_missing_fields
    issues = []
    self.all.includes(partial: [:partial_fields]).includes(:page_partial_fields).each do |page_partial|
      partial_fields = page_partial.partial.partial_fields
      page_partial_fields = page_partial.page_partial_fields
      if partial_fields.length != page_partial_fields.length
        issues << page_partial.id
      end
    end
    issues
  end

  def object_from_partial_fields(partial_fields, page_partial_fields = nil)
    result = {
      id: id,
      order: order,
      visible: visible,
      name: partial.name,
      fieldsCount: partial_fields.length
    }
    page_partial_fields ||= self.page_partial_fields
    page_partial_fields.each_with_index do |page_partial_field, index|
      result[page_partial_field.partial_field_id] = page_partial_field.value
      result["#{page_partial_field.partial_field_id}_style"] = page_partial_field.header_type if partial_fields[index].field_type == "header"
      result["#{page_partial_field.partial_field_id}_page_partial_field_id"] = page_partial_field.id if partial_fields[index].field_type == "page-partial"
    end
    result
  end

  def object
    fields = self.partial_fields
    result = {
      id: id,
      order: order,
      name: partial.name,
      visible: visible,
      fields_count: fields.length
    }
    self.page_partial_fields.each_with_index do |page_partial_field, index|
      result[page_partial_field.partial_field_id] = page_partial_field.value
      result["#{page_partial_field.partial_field_id}_style"] = page_partial_field.header_type if fields[index].field_type == "header"
      result["#{page_partial_field.partial_field_id}_page_partial_field_id"] = page_partial_field.id if fields[index].field_type == "page-partial"
    end
    result
  end

  def fields_hash
    result = {}
    fields = self.partial_fields
    self.page_partial_fields.each_with_index do |page_partial_field, index|
      result[fields[index].name] = page_partial_field
    end
    result
  end

  def field_values_hash(args = {})
    defaults = {
      extend_file_urls: false,
      extend_image_urls: false,
      extend_objects: false,
      extend_page_partials: false,
      extend_webinars: false,
      extend_event_series: false,
    }
    args = defaults.merge(args)

    fields = self.partial_fields

    result = {}
    self.page_partial_fields.each_with_index do |page_partial_field, i|
      partial_field = fields[i]
      if args[:extend_file_urls] && partial_field['field_type'] == 'file'
        value = Upload.find(page_partial_field.value).url
      elsif args[:extend_image_urls] && partial_field['field_type'] == 'image'
        if page_partial_field.value.empty?
          value = ""
        else
          image = Upload.find(page_partial_field.value)
          value = URI::encode("https://process.filestackapi.com/#{ENV['FILESTACK_API_KEY']}/auto_image/compress/quality=value:90/#{image.url.split('/')[-1]}")
        end
      elsif args[:extend_objects] && partial_field['field_type'] == 'object'
        instance = page_partial_field.instance
        value = instance ? instance.field_values_hash(args) : ""
      elsif args[:extend_webinars] && partial_field['field_type'] == 'webinar'
        value = Webinar.find(page_partial_field.value)
      elsif args[:extend_page_partials] && partial_field['field_type'] == 'page-partial'
        value = page_partial_field.nested_page_partial
      elsif args[:extend_event_series] && partial_field['field_type'] == 'event_series'
        value = EventSeries.find_by(id: page_partial_field.value)
      elsif partial_field['field_type'] == 'content_download'
        value = ContentDownload.find(objekt_field.value)
      else
        value = page_partial_field.value
      end
      result[fields[i].name] = value
    end
    result
  end

  def field_header_styles_hash
    result = {}
    fields = PartialField.where(partial_id: partial_id, field_type: "header").order(:order)
    page_partial_fields = PagePartialField.where("page_partial_id = ? AND partial_field_id IN (?)", id, fields.map { |field| field.id }).joins(:partial_field).order('partial_fields.order')
    page_partial_fields.each_with_index do |page_partial_field, index|
      result[fields[index].name] = HeaderType.find(page_partial_field.header_type).name
    end
    result
  end

  def cached_urls
    instance = page_id ? page : PagePartialField.find(page_partial_field_id).page
    instance.cached_urls
  end
end
