# frozen_string_literal: true

class TrackingLink < ApplicationRecord

  belongs_to :creator, class_name: "User", foreign_key: "creator_id"
  belongs_to :publisher, class_name: "User", foreign_key: "publisher_id"
  belongs_to :hider, class_name: "User", foreign_key: "hider_id"
  has_many :tracking_link_redirect_events

  enum status: {
    draft: "draft",
    published: "published",
    hidden: "hidden",
  }

  attr_readonly :slug, :creator_id

  validates :slug,
            uniqueness: true,
            if: :other_published_tracking_links?

  validates :slug,
            presence: true,
            length: {
              maximum: 256,
            },
            format: {
              with: /\A[a-zA-Z0-9-]+\z/,
              message: "only accepts letters, numbers and hyphens",
            }

  validates :hidden_at, presence: true, if: :hidden?
  validates :hider, presence: true, if: :hidden?
  validates :published_at, presence: true, if: :published?
  validates :publisher, presence: true, if: :published?

  validates :destination_url, presence: true, length: {
    maximum: 1000,
  }, format: { with: URI.regexp(%w[http https]) }

  scope :published, -> { where(status: :published) }
  scope :latest_versions, -> { where(latest_version: true) }

  # links that have been published cannot be edited
  def ever_published?
    published_at?
  end

  def other_published_tracking_links?
    self.class.published.any? { |tracking_link| tracking_link.slug == slug } && status == "published"
  end

end
