# frozen_string_literal: true

class User < ApplicationRecord

  validates :email, :name, :team_id, :permission_id, presence: true
  validate :validate_password, if: Proc.new { |user| user.new_record? }
  belongs_to :team
  belongs_to :permission
  has_many :privileges, through: :permission
  has_many :pages, foreign_key: :creator_id
  has_many :blog_posts, as: :author

  has_one(
    :profile_pic,
    class_name: "Upload",
    foreign_key: :id,
    primary_key: :image_id
  )

  attr_accessor :password

  after_create :create_in_aaa
  before_destroy :create_guest_author_objekt
  after_destroy :remove_aaa_membership

  def access
    permission.level
  end

  def image_url
    profile_pic && ApplicationController.helpers.filestack_url(profile_pic) || ActionController::Base.helpers.image_path("admin/default-user-image.jpg")
  end

  def create_guest_author_objekt
    posts = self.blog_posts
    return if posts.empty?

    guest_author_template = Template.find_by(name: "Guest Author")
    template_fields = guest_author_template.fields

    name_field = template_fields.find_by(name: "Name")
    bio_field = template_fields.find_by(name: "Bio")
    image_field = template_fields.find_by(name: "Image")
    twitter_handle = template_fields.find_by(name: "Twitter Handle")

    guest_author_objekt = Objekt.create!(template_id: guest_author_template.id)
    template_id = guest_author_template.id
    objekt_id = guest_author_objekt.id

    ObjektField.create!(field_id: name_field.id, objekt_id: objekt_id, value: self.name)
    ObjektField.create!(field_id: bio_field.id, objekt_id: objekt_id, value: self.bio)
    ObjektField.create!(field_id: image_field.id, objekt_id: objekt_id, value: self.image_id)
    ObjektField.create!(field_id: twitter_handle.id, objekt_id: objekt_id, value: self.twitter_handle)

    posts.each { |post| post.update(author_id: objekt_id, author_type: "Objekt") }
  end

  def cached_urls
    posts_to_clear = []
    self.blog_posts.select { |post| post.active }.each { |post| posts_to_clear << "/blog#{post.url}" }
    posts_to_clear << "/blog"
    posts_to_clear
  end

  private

  def validate_password
    errors.add(:password, :blank) if password.empty?
    errors.add(:password, "must be at least six characters") if password.length < 6
  end

  def create_in_aaa
    encrypted_password = BCrypt::Password.create(self.password).to_s
    AAA.client.sync_membership(
      site_id: "cms",
      member_id: id.to_s,
      email: email,
      name: name,
      encrypted_password: encrypted_password
    )
  end

  def remove_aaa_membership
    AAA.client.delete_membership(site_id: "cms", member_id: id.to_s)
  end

end
