class Webinar < ApplicationRecord

  include Netlifiable

  ENCRYPTION_KEY = ENV.fetch "AUTOMATED_WEBINAR_ENCRYPTION_KEY", "test-key"
  AUTOMATED_WEBINAR_DATE_FORMAT = "%a, %-d %b, %Y %l:%M %P"

  DEFAULT_FAQ_MARKDOWN = "##FAQ\n\nLearn more about what to do here".freeze
  DEFAULT_SIDEBAR_ORDERING = {
    "chat" => {
      "order" => 0,
      "enabled" => "true",
    },
    "faq" => {
      "order" => 1,
      "enabled" => "false",
    },
    "downloads" => {
      "order" => 2,
      "enabled" => "true",
    },
    "replies" => {
      "order" => 3,
      "enabled" => "true",
    },
  }.freeze

  enum webinar_type: { live: 0, automated: 1 }
  enum automation_interval: { every_15_minutes: 0, every_30_minutes: 1, every_1_hour: 2 }
  enum platform: { youtube: 1, vimeo: 2 }

  validates :name, :title, :url, presence: true
  validates :url, uniqueness: true
  validates_date :time, allow_blank: false
  validates_date :time2, allow_blank: true
  validates_date :live_start_date, allow_blank: true
  validates_date :live_end_date, allow_blank: true
  validate :url_valid
  validate :timeframe_and_upgrade_options_valid
  validate :live_dates_and_redirect_valid
  validates_numericality_of :duration, :greater_than_or_equal_to => 1, if: -> { self.webinar_type == "automated" }

  has_many :registrations
  has_many :webinars_event_series
  has_many :event_series, through: :webinars_event_series
  has_many :webinar_objects, foreign_key: :webinar_id, class_name: "WebinarObject"
  has_many :webinar_downloads, -> { where(objekt_name: "download") }, class_name: "WebinarObject", foreign_key: :webinar_id
  has_many :downloads, through: :webinar_downloads, source: :objekt
  has_many :webinar_hosts, -> { where(objekt_name: "host/inviter") }, class_name: "WebinarObject", foreign_key: :webinar_id
  has_many :hosts, through: :webinar_hosts, source: :objekt
  has_many :webinar_offers, dependent: :destroy

  scope :automated, -> { where(webinar_type: "automated") }
  scope :live, -> { where(webinar_type: "live") }

  after_commit :queue_update_duration

  def queue_update_duration
    if previous_changes["replay_video_id"] && replay_video_id
      UpdateWebinarDuration.perform_async(id)
    end
  end

  def url_valid
    if /[^a-z\d\-]/.match?(url)
      errors.add(:url, "is invalid")
    end
  end

  def self.default_sidebar_options
    DEFAULT_SIDEBAR_ORDERING
  end

  def self.default_faq
    DEFAULT_FAQ_MARKDOWN
  end

  def ordered_sidebar_partials
    ordering_map = JSON.parse(sidebar_options).presence || Webinar.default_sidebar_options
    sidebar_configuration.reject do |partial|
      ordering_map[partial[:section_name]].blank? || !["true", true].include?(ordering_map[partial[:section_name]]["enabled"]) || partial[:disabled]
    end.sort_by { |partial| ordering_map[partial[:section_name]]["order"].to_i }
  end

  def sidebar_configuration
    [
      {
        partial_name: "partials/webinars/faq",
        locals: { faq: faq },
        section_name: "faq",
        disabled: faq.blank?,
      },
      {
        partial_name: "partials/webinars/chat",
        section_name: "chat",
      },
      {
        partial_name: "partials/webinars/downloads",
        locals: { downloads: downloads },
        section_name: "downloads",
        render_notifications: true,
        disabled: downloads.empty?,
      },
      {
        partial_name: "partials/webinars/replies",
        section_name: "replies",
        render_notifications: true,
      },
    ]
  end

  def timeframe_and_upgrade_options_valid
    if upgrade_timeframe == "monthly/annual" && upgrade_options == "basic/pro"
      errors.add(:upgrade_timeframe, "and upgrade options not compatible")
    end
    if upgrade_options == "business" && upgrade_timeframe != "monthly"
      errors.add(:upgrade_timeframe, "must be monthly for business")
    end
  end

  def live_dates_and_redirect_valid
    if live_end_date && redirect.empty?
      errors.add(:redirect, "can't be blank with live end date")
    end
    if (live_start_date && live_end_date) && (live_start_date >= live_end_date)
      errors.add(:live_start_date, "must be before live end date")
    end
    if live_start_date
      if (live_start_date >= time) || (time2 && live_start_date >= time2)
        errors.add(:live_start_date, "must be before webinar time(s)")
      end
    end
    if live_end_date
      if (live_end_date <= time) || (time2 && live_end_date <= time2)
        errors.add(:live_end_date, "must be after webinar time(s)")
      end
    end
  end

  # associations

  def basic_replay_bonuses
    if self.basic_replay_bundle
      field_values_hash = Objekt.find(self.basic_replay_bundle).field_values_hash
      field_values_hash.delete("Name")
      field_values_hash.reject! { |_k, v| v.empty? }
      field_values_hash.map { |_k, v| Objekt.find(v) }
    else
      []
    end
  end

  def pro_replay_bonuses
    if self.pro_replay_bundle
      field_values_hash = Objekt.find(self.pro_replay_bundle).field_values_hash
      field_values_hash.delete("Name")
      field_values_hash.reject! { |_k, v| v.empty? }
      field_values_hash.map { |_k, v| Objekt.find(v) }
    else
      []
    end
  end

  def basic_play_bonuses
    if self.basic_play_bundle
      field_values_hash = Objekt.find(self.basic_play_bundle).field_values_hash
      field_values_hash.delete("Name")
      field_values_hash.reject! { |_k, v| v.empty? }
      field_values_hash.map { |_k, v| Objekt.find(v) }
    else
      []
    end
  end

  def pro_play_bonuses
    if self.pro_play_bundle
      field_values_hash = Objekt.find(self.pro_play_bundle).field_values_hash
      field_values_hash.delete("Name")
      field_values_hash.reject! { |_k, v| v.empty? }
      field_values_hash.map { |_k, v| Objekt.find(v) }
    else
      []
    end
  end

  # other

  def utm_campaign
    custom_utm_campaign.empty? ? "#{proper_time.strftime("%Y_%m_%d")}-#{url}" : custom_utm_campaign
  end

  def mt_campaign
    custom_mt_campaign.empty? ? "webinar" : custom_mt_campaign
  end

  def replay_utm_campaign
    custom_utm_campaign.empty? ? "#{previous_or_final.strftime("%Y_%m_%d")}-#{url}" : custom_utm_campaign
  end

  def utm_content
    custom_utm_content.empty? ? "play_page_btn" : "#{custom_utm_content}-play"
  end

  def replay_utm_content
    custom_utm_content.empty? ? "replay_page_btn" : "#{custom_utm_content}-replay"
  end

  def registrations_count
    if recurring
      Registration.where(webinar_id: id, webinar_start_time: proper_time1).count
    else
      registrations.count
    end
  end

  def self.all_times(args = {})
    result = []
    webinars = (args[:type] == "live" ? self.live : self.all)
    webinars.find_each do |webinar|
      object = {
        id: webinar.id,
        name: webinar.name,
        url: webinar.url,
        title: webinar.title,
        registrations: Registration.where(webinar_id: webinar.id, webinar_start_time: webinar.proper_time1).count,
        webinar_type: webinar.webinar_type,
      }
      if webinar.recurring
        object[:time] = webinar.next_recurring_event
        object[:time_parsed] = webinar.next_recurring_event.strftime("%-m/%-d/%y - %-l:%M %P")
        object[:timestamp] = webinar.next_recurring_event.to_i
      else
        object[:time] = webinar.time
        object[:time_parsed] = webinar.time.strftime("%-m/%-d/%y - %-l:%M %P")
        object[:timestamp] = webinar.time.to_i
      end
      result << object
      if webinar.time2
        result << {
          id: webinar.id,
          name: webinar.name,
          url: webinar.url,
          title: webinar.title,
          time: webinar.time,
          time_parsed: webinar.time2.strftime("%-m/%-d/%y - %-l:%M %P"),
          registrations: Registration.where(webinar_id: webinar.id, webinar_start_time: webinar.time2).count,
          timestamp: webinar.time2.to_i,
        }
      end
    end
    result
  end

  def self.firebase_auth_json
    case APP_ENV
    when "test"
      obj = {
        "type": "service_account",
        "project_id": "teachable-cms-test",
        "private_key_id": "d55056f1b3a6fa90698fa71673ab1e3c2244bd57",
        "client_email": "firebase-adminsdk-oohzk@teachable-cms-test.iam.gserviceaccount.com",
        "client_id": "104875738502970617416",
        "auth_uri": "https://accounts.google.com/o/oauth2/auth",
        "token_uri": "https://oauth2.googleapis.com/token",
        "auth_provider_x509_cert_url": "https://www.googleapis.com/oauth2/v1/certs",
        "client_x509_cert_url": "https://www.googleapis.com/robot/v1/metadata/x509/firebase-adminsdk-oohzk%40teachable-cms-test.iam.gserviceaccount.com",
      }
      obj["private_key"] = "-----BEGIN PRIVATE KEY-----\n#{ENV.fetch("FIREBASE_PRIVATE_KEY_TEST").gsub('\n', "\n")}\n-----END PRIVATE KEY-----\n"
    when "development"
      obj = {
        "type": "service_account",
        "project_id": "teachable-cms-development",
        "private_key_id": "d2a63abe86a3e91a9594dc5f6039d825fd067c98",
        "client_email": "firebase-adminsdk-yrjus@teachable-cms-development.iam.gserviceaccount.com",
        "client_id": "100969278749324154486",
        "auth_uri": "https://accounts.google.com/o/oauth2/auth",
        "token_uri": "https://oauth2.googleapis.com/token",
        "auth_provider_x509_cert_url": "https://www.googleapis.com/oauth2/v1/certs",
        "client_x509_cert_url": "https://www.googleapis.com/robot/v1/metadata/x509/firebase-adminsdk-yrjus%40teachable-cms-development.iam.gserviceaccount.com",
      }
      obj["private_key"] = "-----BEGIN PRIVATE KEY-----\n#{ENV.fetch("FIREBASE_PRIVATE_KEY_DEV").gsub('\n', "\n")}\n-----END PRIVATE KEY-----\n"
    when "production"
      obj = {
        "type" => "service_account",
        "project_id" => "teachable-cms",
        "private_key_id" => "7162af3c7f390d3651760017638080c186ca2715",
        "client_email" => "firebase-adminsdk-twwp6@teachable-cms.iam.gserviceaccount.com",
        "client_id" => "112486135441942688393",
        "auth_uri" => "https://accounts.google.com/o/oauth2/auth",
        "token_uri" => "https://accounts.google.com/o/oauth2/token",
        "auth_provider_x509_cert_url" => "https://www.googleapis.com/oauth2/v1/certs",
        "client_x509_cert_url" => "https://www.googleapis.com/robot/v1/metadata/x509/firebase-adminsdk-twwp6%40teachable-cms.iam.gserviceaccount.com",
      }
      obj["private_key"] = "-----BEGIN PRIVATE KEY-----\n#{ENV.fetch("FIREBASE_PRIVATE_KEY").gsub('\n', "\n")}\n-----END PRIVATE KEY-----\n"
    when "staging", /\Areview-(\d)\z/
      obj = {
        "type": "service_account",
        "project_id": "teachable-cms-staging",
        "private_key_id": "e1f097c3cb6c6ec160e0c691df92ebfd427fb959",
        "client_email": "firebase-adminsdk-twof3@teachable-cms-staging.iam.gserviceaccount.com",
        "client_id": "114670426272122058350",
        "auth_uri": "https://accounts.google.com/o/oauth2/auth",
        "token_uri": "https://oauth2.googleapis.com/token",
        "auth_provider_x509_cert_url": "https://www.googleapis.com/oauth2/v1/certs",
        "client_x509_cert_url": "https://www.googleapis.com/robot/v1/metadata/x509/firebase-adminsdk-twof3%40teachable-cms-staging.iam.gserviceaccount.com",
      }
      obj["private_key"] = "-----BEGIN PRIVATE KEY-----\n#{ENV.fetch("FIREBASE_PRIVATE_KEY").gsub('\n', "\n")}\n-----END PRIVATE KEY-----\n"
    end
    obj.to_json
  end

  def create_in_firebase
    firebase = Firebase::Client.new(Webinar.get_firebase_url, Webinar.firebase_auth_json)
    firebase_data = {
      "downloadSwitches" => {
        "0": false,
        "1": false,
        "2": false,
        "3": false,
        "4": false,
        "5": false,
        "6": false,
        "7": false,
      },
      "hangoutStatus" => "Starting",
      "replayVideoStatus" => "unclicked",
      "showUpgradeButton" => false,
      "youTubeID" => "",
    }
    firebase.set("#{self.id}/controls", firebase_data)
  end

  def delete_from_firebase
    firebase = Firebase::Client.new(Webinar.get_firebase_url, Webinar.firebase_auth_json)
    firebase.delete(self.id)
  end

  def clear_chat
    firebase = Firebase::Client.new(Webinar.get_firebase_url)
    firebase.delete("#{self.id}/messages")
    firebase.delete("#{self.id}/users")
  end

  def self.get_firebase_url #rubocop:disable Naming/AccessorMethodName
    case APP_ENV
    when "staging", /\Areview-(\d)\z/
      "https://teachable-cms-staging.firebaseio.com"
    when "test"
      "https://teachable-cms-test.firebaseio.com"
    when "development"
      "https://teachable-cms-development.firebaseio.com"
    when "production"
      "https://teachable-cms.firebaseio.com"
    end
  end

  def firebase_access_token(privileged_user: nil)
    firebase_auth_data = JSON.parse(Webinar.firebase_auth_json)
    now_seconds = Time.now.to_i
    private_key = OpenSSL::PKey::RSA.new firebase_auth_data["private_key"]
    uid = privileged_user.present? ? "User@#{privileged_user.id}" : "Webinar@#{id}"
    payload = {
      :iss => firebase_auth_data["client_email"],
      :sub => firebase_auth_data["client_email"],
      :aud => "https://identitytoolkit.googleapis.com/google.identity.identitytoolkit.v1.IdentityToolkit",
      :iat => now_seconds,
      :exp => now_seconds+(60*10),
      :uid => uid,
      :claims => {:privileged_user => privileged_user.present?}
    }
    JWT.encode payload, private_key, "RS256"
  end

  def decrypt_start_time(token)
    AES.decrypt(token, Webinar::ENCRYPTION_KEY)
  end

  def encrypt_start_time(time)
    AES.encrypt(time.to_s, Webinar::ENCRYPTION_KEY)
  end

  def automated_event_times(force: false)
    return [] if webinar_type == "live" && !force
    current_time = Time.zone.now
    upcoming_time = case automation_interval
                    when "every_15_minutes"
                      DateTimeHelper.time_to_next_quarter_hour(current_time)
                    when "every_30_minutes"
                      DateTimeHelper.time_to_next_half_hour(current_time)
                    when "every_1_hour"
                      DateTimeHelper.time_to_next_hour(current_time)
                    else
                      raise "Unsupported automation_interval: #{automation_interval}"
                    end
    # So the logic below works as follows:
    # When you register, you should be shown the upcoming 2PM and 8PM sessions respectively.
    # So if you get to the registration page at 2:15PM. We should find the upcoming 8PM and the following days 2PM session
    other_times = if current_time > (current_time.beginning_of_day + 14.hours) && current_time <= (current_time.beginning_of_day + 20.hours)
                    [
                      current_time.beginning_of_day + 20.hours,
                      current_time.beginning_of_day.next_day + 14.hours,
                    ]
                  elsif current_time > (current_time.beginning_of_day + 14.hours) && current_time > (current_time.beginning_of_day + 20.hours)
                    [
                      current_time.beginning_of_day.next_day + 14.hours,
                      current_time.beginning_of_day.next_day + 20.hours,
                    ]
                  else
                    [
                      current_time.beginning_of_day + 14.hours,
                      current_time.beginning_of_day + 20.hours,
                    ]
                  end
    webinar_times = [upcoming_time] + other_times

    webinar_times.map! do |time|
      {
        time_key: encrypt_start_time(time),
        time_text: time.strftime(AUTOMATED_WEBINAR_DATE_FORMAT),
      }
    end
    webinar_times.uniq { |time| time[:time_text] }
  end

  def later_time
    if time2
      time > time2 ? time : time2
    else
      time
    end
  end

  def minutes_to_go
    next_time = ((proper_time1 + 90.minutes) > DateTime.now ? proper_time1 : time2)
    (next_time.to_i - DateTime.now.to_i) / 60
  end

  def over?
    minutes_to_go < -90
  end

  # rubocop:disable Security/YAMLLoad
  def next_recurring_event
    # webinar currently happening, or the next one
    first_event = Montrose.r(YAML.load(recurrence)).events.first
    ((first_event + 90.minutes) > Time.now) ? first_event : Montrose.r(YAML.load(recurrence)).events.take(2)[1]
  end

  def previous_or_final
    # allow 90 minutes for webinar, plus 60 minutes for YouTube processing, before new replay shows up
    if recurring
      first_event = Montrose.r(YAML.load(recurrence)).events.first
      event = ((first_event + 150.minutes) > Time.now) ? first_event : Montrose.r(YAML.load(recurrence)).events.take(2)[1]
      event - 1.week
    elsif time2
      ((time2.to_i + 150.minutes) > DateTime.now.to_i) ? time : time2
    else
      time
    end
  end
  # rubocop:enable Security/YAMLLoad

  def proper_time
    # webinar currently happening, or the next/final one
    (time2 && (time.to_i + 90.minutes) <= DateTime.now.to_i) ? time2 : proper_time1
  end

  def proper_time1
    recurring ? next_recurring_event : time
  end

  def two_valid_options
    time2 && ((time + 90.minutes).to_i > DateTime.now.to_i)
  end

  def offers_json
    webinar_offers.order(:start_time).map(&:attributes).to_json
  end

end
