module Testing
  class PageGenerator

    if Rails.env.test? || Rails.env.development?
      require "factory_bot"
      include FactoryBot::Syntax::Methods
    end

    def self.create_page(key, active: true, replace_existing: true)
      raise "Can only be run in test env" if !Rails.env.test?
      FactoryBot.find_definitions if FactoryBot.factories.none?
      new(key, active, replace_existing).create_page
    end

    def initialize(key, active, replace_existing)
      @key              = key
      @active           = active
      @replace_existing = replace_existing
    end

    def create_page
      file_path       = Rails.root.join("config", "testing", "page_definitions", "#{@key}.yml")
      page_definition = YAML.safe_load(File.read(file_path))
      ActiveRecord::Base.transaction do
        if @replace_existing && (existing_page = Page.find_by(url: page_definition["url"]))
          existing_page.destroy!
        end

        page = Page.find_by(url: page_definition["url"]) || generate_page(page_definition)

        create_plans
        create_header_types
        page
      end
    end

    private

    def generation_user
      @_generation_user ||= User.first || FactoryBot.create(:user)
    end

    def generate_page(page_definition)
      # Business logic for making a default page
      page = Page.create!(
        name:                      page_definition["name"],
        url:                       page_definition["url"],
        title_text:                page_definition["title_text"],
        description:               page_definition["description"],
        canonical_url:             page_definition["canonical_url"],
        creator:                   generation_user,
        active:                    @active,
        index_page:                false,
        display_exit_intent_modal: false,
        google_optimize:           false,
      )
      page_definition["partials"].each_with_index do |partial_definition, index|
        generate_page_partial(page, partial_definition, index)
      end
      page
    end

    def generate_page_partial(page, partial_definition, order)
      partial = Partial.find_or_create_by!(name: partial_definition["name"])

      partial_definition["fields"].to_a.each_with_index do |field, index|
        generate_partial_field(partial.id, field, index)
      end

      page_partial = PagePartial.create!(
        page: page,
        order: order,
        visible: true,
        partial: partial
      )

      PagePartialField.create_page_partial_fields!(page_partial)

      partial_definition["fields"].to_a.each do |field|
        next if field["value"].nil?

        page_partial_field = page_partial.page_partial_fields.find { |ppf| ppf.partial_field.name == field["name"] }

        if field["type"] == "object"
          # write logic for generating objekt
          field["value"].each do |template_name, template_values|
            template = Template.find_or_create_by!(name: template_name)
            template.update(plural: template_values["plural"]) if template_values["plural"]
            template["fields"].to_a.each_with_index do |field_definition, field_index|
              template_field = Field.find_by(name: field_definition["name"], template_id: template.id) ||
                               Field.create!(
                                 name:        field_definition["name"],
                                 char_limit:  field_definition["char_limit"],
                                 mandatory:   field_definition["mandatory"],
                                 field_type:  field_definition["type"],
                                 order:       field_index,
                                 object_type: "",
                                 template_id: template.id
                               )

            end
          end
          # page_partial_field.update(value: objekt.id.to_s)
        elsif field["type"] == "image"
          image_upload = Upload.create!(url: field["value"], category: "page_partial", upload_type: "image", uploaded_by: generation_user.id)
          page_partial_field.update!(value: image_upload.id.to_s)
        elsif field["type"] == "webinar"
          Webinar.create!(field.dig("value"))
          page_partial_field.update!(value: field.dig("value", "id"))
        elsif field["type"] == "page-partial"
          nested_page_partial = generate_page_partial(page, field["value"], 0)
          nested_page_partial.update!(page_partial_field: page_partial_field, page_id: nil)
          page_partial_field.update!(value: nested_page_partial.id.to_s)
        else
          page_partial_field.update(value: field["value"], header_type: HeaderType.find_or_create_by!(name: field["header_type"]).try!(:id))
        end
      end
      page_partial
    end

    def generate_partial_field(partial_id, field_definition, order)
      partial_field = PartialField.find_by(name: field_definition["name"], partial_id: partial_id) ||
                      PartialField.create!(
                        name:        field_definition["name"],
                        char_limit:  field_definition["char_limit"],
                        mandatory:   field_definition["mandatory"],
                        field_type:  field_definition["type"],
                        order:       order,
                        object_type: field_definition["object_type"],
                        partial_id:  partial_id
                      )
      case field_definition["type"]
      when "text"
        # Nothing really to do here
      when "number"
        # Nothing really to do here
      when "header"
        # Nothing really to do here
      when "image"
        # Nothing really to do here
      when "color"
        # Nothing really to do here
      when "event_series"
        # Nothing really to do here
      when "webinar"
        # Nothing really to do here
      when "page-partial"
        # Nothing really to do here
      when "markdown"
        # Nothing really to do here
      when "object"
        generate_template(field_definition)
      when "dropdown"
        field_definition["options"].to_a.each_with_index do |option, index|
          MenuOption.create!(value: option, order: index, optionable: partial_field)
        end
      else
        raise "Field definition not acceptable: #{field_definition['type']}"
      end
    end

    def generate_template(template_definitions)
      template_definitions["value"].each do |template_name, template_definition|
        template = Template.find_or_create_by!(name: template_name)
        template.update!(plural: template_definition["plural"] || template.plural)

        fields = template_definition["fields"]

        fields.to_a.each_with_index do |field_definition, index|
          field_type = field_definition["type"]
          field = Field.find_by(name: field_definition["name"]) || Field.create!(
            template:    template,
            name:        field_definition["name"],
            field_type:  field_type,
            mandatory:   field_definition["mandatory"],
            char_limit:  field_definition["char_limit"],
            order:       index,
            object_type: field_definition["object_type"],
          )
          if field_type == "object"
            generate_template(field_definition)
          elsif field_type == "dropwdown"
            field_definition["options"].to_a.each_with_index do |option, option_index|
              MenuOption.create!(value: option, order: option_index, optionable: field)
            end
          end
        end
        # Create Default Instance of said object
        objekt = Objekt.create!(template: template)
        template.fields.each do |field|
          field_with_val = fields.to_a.select do |field_def|
            field_def["value"].present? && field_def["name"] == field.name
          end.first
          if field.field_type == "dropdown"
            value = MenuOption.find_by(optionable_id: field.id, order: 0).value
          elsif field.field_type == "object"
            template_id = Template.find_by(name: field.object_type).id
            value = Objekt.where(template_id: template_id).first.id
          elsif field.field_type == "color"
            value = "#FFFFFF"
          elsif field.field_type == "number"
            value = "0"
          elsif field.field_type == "webinar"
            value = Webinar.first.try!(:id)
          elsif field.field_type == "content_download"
            value = ContentDownload.first.try!(:id)
          elsif field.field_type == "event_series"
            value = EventSeries.first.try!(:id)
          else
            value = ""
          end
          ObjektField.create!(objekt: objekt, field: field, value: get_field_value(field_with_val, field) || value)
        end
      end
    end

    def get_field_value(field_definition, field)
      return nil if field_definition.nil?
      case field_definition["type"]
      when "dropdown"
        MenuOption.find_by(optionable_id: field.id, value: field_definition["value"]).id
      when "object"
        template = Template.find_by(name: field_definition["object_type"])
        Objekt.where(template: template).first.try!(:id)
      else
        field_definition["value"]
      end
    end

    def create_plans
      Plan.find_or_create_by!(name: "All Plans", annual_included: false, monthly_price: 0, order: 0, app_monthly_plan_id: 0)
      Plan.find_or_create_by!(name: "Free", tagline: "Get started for free", annual_included: false, monthly_price: 0, includes_text: "All of our base features", order: 1, app_monthly_plan_id: 4)
      Plan.find_or_create_by!(name: "Basic", tagline: "Create your first online course", annual_included: true, monthly_price: 39, annual_price: 348, includes_text: "All of our base features, plus:", order: 2, app_monthly_plan_id: 153, app_annual_plan_id: 152)
      Plan.find_or_create_by!(name: "Professional", tagline: "Our most popular plan", annual_included: true, monthly_price: 119, annual_price: 1188, includes_text: "Everything in Basic, plus:", order: 3, app_monthly_plan_id: 186, app_annual_plan_id: 185)
      Plan.find_or_create_by!(name: "Business", tagline: "Experience the best of Teachable", annual_included: true, monthly_price: 299, annual_price: 2988, includes_text: "Everything in Professional, plus:", order: 4, app_monthly_plan_id: 218, app_annual_plan_id:219)
    end

    def create_header_types
      ["Option 1", "Option 2"].each_with_index do |option, index|
        HeaderType.create!(name: option, order: index)
      end
    end

  end
end
