##
# Entrypoint for all tracking link redirect related functionality
module TrackingLinks

  RedirectResponse     = Struct.new(:redirect_url, :cookies)
  NEW_ANONYMOUS_COOKIE = "aid".freeze
  ANONYMOUS_COOKIE     = "_afid".freeze
  # All cookies relevant for tracking - TODO add segment cookies here
  RELEVANT_COOKIES     = [NEW_ANONYMOUS_COOKIE, ANONYMOUS_COOKIE]

  def self.track_and_redirect(tracking_link:, request:, cms_user_id: nil)
    event_id     = SecureRandom.uuid

    TeachableLogger.tagged(
      tracking_link_id: tracking_link.id,
      tracking_link_redirect_event_id: event_id,
      tracking_link_slug: tracking_link.slug,
      tracking_link_request_url: request.url
    ) do
      Rails.logger.info "TRACKING_LINK_REDIRECT_BEGIN; Cookies #{request.cookies}"
    end

    # Set an anonymous visitor id if not already set
    cookies = request.cookies.slice(*TrackingLinks::RELEVANT_COOKIES)

    anonymous_cookie     = cookies[ANONYMOUS_COOKIE]
    new_anonymous_cookie = cookies[NEW_ANONYMOUS_COOKIE]

    if new_anonymous_cookie.present? && valid_uuid?(new_anonymous_cookie)
      anonymous_id = new_anonymous_cookie
      cookies[NEW_ANONYMOUS_COOKIE] = { value: anonymous_id, domain: :all }
    elsif anonymous_cookie.present? && valid_uuid?(anonymous_cookie)
      anonymous_id = anonymous_cookie
    else
      anonymous_id = SecureRandom.uuid
    end

    cookies[ANONYMOUS_COOKIE] = { value: anonymous_id, domain: :all }

    redirect_url = RedirectUrl.build(
      request_query_params: request.query_parameters,
      destination_url: tracking_link.destination_url,
      event_id: event_id,
      link_id: tracking_link.id,
      additional_query_params: tracking_link.additional_query_params,
      forward_all_query_params: tracking_link.forward_all_query_params,
      forward_query_params_whitelist: tracking_link.forward_query_params_whitelist,
    )

    Tracker.track_redirect(
      tracking_link_id: tracking_link.id,
      id: event_id,
      request_url: request.url,
      redirect_url: redirect_url,
      referrer: request.referrer,
      ip: request.ip,
      anonymous_id: anonymous_id,
      user_agent: request.user_agent,
      cms_user_id: cms_user_id,
      campaign: tracking_link.metadata["campaign"],
    )

    TeachableLogger.tagged(
      tracking_link_id: tracking_link.id,
      tracking_link_redirect_event_id: event_id,
      tracking_link_slug: tracking_link.slug,
      anonymous_id: anonymous_id,
      tracking_link_request_url: request.url
    ) do
      Rails.logger.info "TRACKING_LINK_REDIRECT_REDIRECTED; Cookies #{request.cookies}"
    end

    RedirectResponse.new(redirect_url, cookies)
  end

  def self.valid_uuid?(uuid)
    uuid_regex = /^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$/
    return !!uuid_regex.match(uuid.downcase)
  end

end
