# frozen_string_literal: true

module TrackingLinks
  class Updater

    attr_reader :tracking_link, :attributes, :updater

    def self.call(tracking_link:, update_attributes:, updater:)
      new(tracking_link, update_attributes, updater).call
    end

    def initialize(tracking_link, update_attributes, updater)
      @tracking_link = tracking_link
      @attributes = update_attributes.with_indifferent_access
      @updater = updater
    end

    def call
      if tracking_link.ever_published? && attributes[:status].to_s != "hidden"
        create_new_link
      else
        update_existing_link
      end
    end

    private

    def create_new_link
      ActiveRecord::Base.transaction do
        new_tracking_link = tracking_link.dup
        new_tracking_link.assign_attributes(attributes)
        tracking_link.update(status: :hidden, hider: updater, hidden_at: Time.now, latest_version: false)
        new_tracking_link = set_audit_attributes(new_tracking_link)
        if new_tracking_link.save
          new_tracking_link
        else
          new_tracking_link.errors.full_messages.each do |message|
            tracking_link.errors.add(:base, :new_record, message: message)
          end
          tracking_link
        end
      end
    end

    def update_existing_link
      tracking_link.assign_attributes(attributes)
      update_audit_attributes
      tracking_link.save
      tracking_link
    end

    def set_audit_attributes(new_tracking_link)
      case new_tracking_link.status
      when "published"
        new_tracking_link.publisher = updater
        new_tracking_link.published_at = Time.now
      when "hidden"
        new_tracking_link.status = "draft"
        new_tracking_link.hider = nil
        new_tracking_link.hidden_at = nil
        new_tracking_link.published_at = nil
        new_tracking_link.publisher = nil
      end
      new_tracking_link
    end

    def update_audit_attributes
      case tracking_link.status
      when "published"
        tracking_link.publisher = updater
        tracking_link.published_at = Time.now
      when "hidden"
        tracking_link.hider = updater
        tracking_link.hidden_at = Time.now
      end
    end

  end
end
