module VcrHelper
  # this compares the bodies of the new_request and the cassette
  # if a stripe token is needed, it makes sure the token exists and removes it for the comparison
  def self.match_body_not_stripe_token
    lambda do |new_request, cassette_request|
      if cassette_request.body.include?('stripe_token')
        new_request_body = new_request.body.split(',')
        cassette_request_body = cassette_request.body.split(',')
        stripe_token_index = cassette_request_body.find_index {|element| element.include?('stripe_token')}
        if new_request_body[stripe_token_index].include?('stripe_token') && new_request_body[stripe_token_index].include?('tok_')
          cassette_request_body.delete_at(stripe_token_index)
          new_request_body.delete_at(stripe_token_index)
          new_request_body == cassette_request_body
        else
          false
        end
      else
        new_request.body == cassette_request.body
      end
    end
  end

  def self.configure
    require "vcr"
    require "webmock"

    WebMock.disable_net_connect!

    VCR.configure do |c|
      c.hook_into :webmock
      c.cassette_library_dir = "features/support/cassettes"
      c.default_cassette_options = {
        match_requests_on: [
          :method,
          VCR.request_matchers.uri_without_param(:_),
          match_body_not_stripe_token
        ],
        record: :once,
      }

      # this is for ignoring circleci host
      c.ignore_hosts(
        "hub",
        "percy.io",
        "www.googleapis.com",
        "teachable-cms-test.firebaseio.com",
        "topmind.teachable.cloud",
        "chromedriver.storage.googleapis.com",
        ENV["LOCAL_IP"],
      )
      c.ignore_localhost = true

      # getting rid of keys and tokens in VCR cassettes
      sensitive_data_patterns = %w[_AUTH _ID _KEY _PASSWORD _SECRET _TOKEN _API]
      ENV.each do |key, value|
        if sensitive_data_patterns.any? { |pattern| key.include?(pattern) }
          next if ["", "true", "false"].include?(value.to_s)
          c.filter_sensitive_data(key.to_s) { value.to_s }
        end
      end
    end
  end

  # makes a cassette based on the scenario's name
  # if you use the @vcr-record tag, it will delete the old cassette and allow recording.
  # if the tag is not there, only replaying is allowed.
  def self.around_scenario(scenario)
    directory = scenario.feature.location.file.sub(".feature", "").sub(%r{^.*features/}, "")
    filename = scenario.name.split("\n").first.delete("'").gsub(/\W/, "_").squeeze("_").downcase
    cassette_name = File.join(directory, filename)
    if scenario.tags.any? { |tag| tag.name == "@vcr-record" }
      file = File.join(VCR.configuration.cassette_library_dir, "#{cassette_name}.yml")
      puts "\e[33mUsing @vcr-record tag to re-record #{file.sub(Rails.root.to_s, "").sub(%r{^/}, "")}"
      puts "Don't forget to remove tag before committing.\e[0m"
      File.unlink(file) if File.exist?(file)
      record_mode = :once
    else
      record_mode = :none
    end
    VCR.use_cassette(cassette_name, record: record_mode, allow_playback_repeats: true) do |cassette|
      yield
    end
  end

end
