import React from 'react'
import { connect } from 'react-redux'
import { bindActionCreators } from 'redux'
import Modal from 'react-modal'
import ModalSelect from './modal-select'
import isEqual from 'lodash/isEqual';
import cloneDeep from 'lodash/cloneDeep'
import sortBy from 'lodash/sortBy'
import Admin from './../admin'
import ConfirmDelete from './confirm-delete'
import { SelectModalStyles, ConfirmDeleteModalStyles } from './helpers/modal-styles'
import { fetchEntity, createEntity, updateEntity, deleteEntity } from '../actions/index'
import { withExitPrompt } from './util/with-exit-prompt'

interface BannerDetailsProps {
  banner?: any;
  bannerFieldsConfig?: any;
  bannerFieldsObject?: any;
  bannerPages?: any;
  bannerWebinars?: any;
  createEntity?: any;
  deleteEntity?: any;
  entities?: any;
  errors?: string[];
  fetchEntity?: any;
  pages?: any;
  updateEntity?: any;
  webinars?: any;
}

interface BannerDetailsState extends ComponentState {
  deleteModalOpen: boolean;
  pagesModalOpen: boolean;
  webinarsModalOpen: boolean;
  banner: any;
  bannerSaved: any;
  bannerFieldsConfig: any;
  bannerFieldsObject: any;
  bannerFieldsObjectSaved: any;
  inclusionMethods: any;
  bannerPages: any;
  bannerWebinars: any;
  pages: any;
  webinars: any;
}

class BannerDetails extends React.Component<BannerDetailsProps, BannerDetailsState> {

  constructor(props) {
    super(props);
    this.state = {
      fetching: true,
      changesToSave: false,
      justSaved: false,
      bannerFieldsConfig: {},
      banner: {},
      bannerSaved: {},
      bannerFieldsObject: {},
      bannerFieldsObjectSaved: {},
      bannerPages: [],
      bannerWebinars: [],
      errors: [],
      pages: [],
      webinars: [],
      deleteModalOpen: false,
      pagesModalOpen: false,
      webinarsModalOpen: false,
      inclusionMethods: [{ text: "whitelist" }, { text: "blacklist" }]
    };
  }

  componentDidMount() {
    let pathDirectories = window.location.pathname.split('/');
    this.props.fetchEntity({
      id: pathDirectories[pathDirectories.length - 1],
      directory: "banners",
      entityName: "banner"
    }).then(() => {
      this.setState({
        fetching: false,
        banner: this.props.banner,
        bannerSaved: cloneDeep(this.props.banner),
        bannerFieldsObject: this.props.bannerFieldsObject,
        bannerFieldsObjectSaved: cloneDeep(this.props.bannerFieldsObject),
        bannerFieldsConfig: this.props.bannerFieldsConfig,
        pages: this.props.pages,
        bannerPages: this.props.bannerPages,
        webinars: this.props.webinars,
        bannerWebinars: this.props.bannerWebinars,
        changesToSave: false
      }, () => {
        Admin.setUpNiceSelect({ selector: 'select', func: Admin.changeField.bind(this, this.changeFieldArgs()) });
      });
    });
  }

  clickSave() {
    this.setState({
      fetching: true,
      justSaved: true
    }, () => {
      let pathDirectories = window.location.pathname.split('/');
      this.props.updateEntity({
        id: pathDirectories[pathDirectories.length - 1],
        directory: "banners",
        entityName: "banner",
        entity: this.state.banner,
        additionalData: {
          bannerFields: this.state.bannerFieldsObject
        }
      }).then(() => {
        this.setState({
          fetching: false,
          banner: this.props.banner,
          bannerSaved: cloneDeep(this.props.banner),
          changesToSave: false
        });
      }, () => {
        this.setState({
          fetching: false,
          errors: this.props.errors
        });
      });
    });
  }

  checkForChanges() {
    return !isEqual(this.state.banner, this.state.bannerSaved) || !isEqual(this.state.bannerFieldsObject, this.state.bannerFieldsObjectSaved);
  }

  changeFieldArgs() {
    return {
      thing: "banner",
      errorsArray: this.state.errors,
      changesFunction: this.checkForChanges.bind(this)
    }
  }

  clickAddPage() {
    this.setState({
      pagesModalOpen: true
    });
  }

  clickAddWebinar() {
    this.setState({
      webinarsModalOpen: true
    });
  }

  closeModal() {
    this.setState({
      pagesModalOpen: false,
      webinarsModalOpen: false
    });
  }

  selectPage(e) {
    var pageId = e.target.dataset.id;
    this.setState({
      fetching: true,
      pagesModalOpen: false
    }, () => {
      this.props.createEntity({
        directory: "banner_pages",
        arrayName: "bannerPages",
        entityName: "bannerPage",
        entity: {
          pageId,
          bannerId: this.state.banner.id
        }
      }).then(() => {
        this.setState({
          fetching: false,
          bannerPages: this.props.bannerPages
        });
      });
    });
  }

  selectWebinar(e) {
    var webinarId = e.target.dataset.id;
    this.setState({
      fetching: true,
      webinarsModalOpen: false
    }, () => {
      this.props.createEntity({
        directory: "banner_webinars",
        arrayName: "bannerWebinars",
        entityName: "bannerWebinar",
        entity: {
          webinarId,
          bannerId: this.state.banner.id
        }
      }).then(() => {
        this.setState({
          fetching: false,
          bannerWebinars: this.props.bannerWebinars
        });
      });
    });
  }

  clickDeletePage(e) {
    var pageId = e.target.dataset.id;
    this.setState({
      fetching: true
    }, () => {
      this.props.deleteEntity({
        directory: "banner_pages",
        id: pageId,
        callback: (response) => {
          this.setState({
            fetching: false,
            bannerPages: response.bannerPages
          });
        }
      });
    });
  }

  clickDeleteWebinar(e) {
    var webinarId = e.target.dataset.id;
    this.setState({
      fetching: true
    }, () => {
      this.props.deleteEntity({
        directory: "banner_webinars",
        id: webinarId,
        callback: (response) => {
          this.setState({
            fetching: false,
            bannerWebinars: response.bannerWebinars
          });
        }
      });
    });
  }

  render() {
    return(
      <div id="banner-details" className="component details-component">
        <h1>Banner details</h1>
        <div className="white-box">
          { Admin.renderSpinner(this.state.fetching) }
          { Admin.renderGrayedOut(this.state.fetching) }
          <div className="row">
            { Admin.renderField.bind(this)({ columnWidth: 4, entity: "banner", property: "name" }) }
            { Admin.renderField.bind(this)({ columnWidth: 4, entity: "banner", property: "bannerKey", columnHeader: "Type", readOnly: true }) }
            { Admin.renderDropDown.bind(this)({
              columnWidth: 2,
              entity: "banner",
              property: "inclusionMethod",
              optionLabel: "text",
              optionValue: "text"
            }) }
            { Admin.renderBooleanDropDown.bind(this)({ columnWidth: 2, entity: "banner", property: "active" }) }
          </div>
          { Admin.renderDetailsButtons.call(this) }
          <hr className="margin-top" />
          { this.renderBannerFields.call(this) }
          <hr />
          <div className="row">
            <div className="col-xs-12">
              <table className="admin-table no-hover with-button">
                <thead>
                  <tr>
                    <th>{ `${this.state.banner.inclusionMethod == "whitelist" ? "Included" : "Excluded"} Pages` }</th>
                  </tr>
                </thead>
                <tbody>
                  <tr><td></td><td></td></tr>
                  { sortBy(this.state.bannerPages, "name").map((bannerPage, index) => {
                    return(
                      <tr key={ index }>
                        <td>{ bannerPage.name }</td>
                        <td><div className="x-button" onClick={ this.clickDeletePage.bind(this) } data-id={ bannerPage.id }></div></td>
                      </tr>
                    );
                  }) }
                </tbody>
              </table>
              <a className="green-outline-button small m-bottom" onClick={ this.clickAddPage.bind(this) }>Add Page</a>
            </div>
          </div>
          <hr />
          <div className="row">
            <div className="col-xs-12">
              <table className="admin-table no-hover with-button">
                <thead>
                  <tr>
                    <th>{ `${this.state.banner.inclusionMethod == "whitelist" ? "Included" : "Excluded"} Webinars` }</th>
                  </tr>
                </thead>
                <tbody>
                  <tr><td></td><td></td></tr>
                  { sortBy(this.state.bannerWebinars, "name").map((bannerWebinar, index) => {
                    return(
                      <tr key={ index }>
                        <td>{ bannerWebinar.name }</td>
                        <td><div className="x-button" onClick={ this.clickDeleteWebinar.bind(this) } data-id={ bannerWebinar.id }></div></td>
                      </tr>
                    );
                  }) }
                </tbody>
              </table>
              <a className="green-outline-button small" onClick={ this.clickAddWebinar.bind(this) }>Add Webinar</a>
            </div>
          </div>
        </div>
        <Modal isOpen={ this.state.pagesModalOpen } onRequestClose={ this.closeModal.bind(this) } contentLabel="Modal" style={ SelectModalStyles }>
          <ModalSelect options={ this.state.pages } property="name" func={ this.selectPage.bind(this) } />
        </Modal>
        <Modal isOpen={ this.state.webinarsModalOpen } onRequestClose={ this.closeModal.bind(this) } contentLabel="Modal" style={ SelectModalStyles }>
          <ModalSelect options={ this.state.webinars } property="name" func={ this.selectWebinar.bind(this) } />
        </Modal>
        <Modal isOpen={ this.state.deleteModalOpen } onRequestClose={ Admin.closeDeleteModal.bind(this) } contentLabel="Modal" style={ ConfirmDeleteModalStyles }>
          <ConfirmDelete entityName="banner" skipCheck={ true } closeModal={ Admin.closeDeleteModal.bind(this) } closeAndFetch={ Admin.closeDeleteModalAndFetch.bind(this) } delete={ Admin.deleteEntity.bind(this) } />
        </Modal>
      </div>
    );
  }

  renderBannerFields() {
    return(
      <div className="row">
        { Object.keys(this.state.bannerFieldsObject).sort().map((key) => {
          let fieldConfig = this.props.bannerFieldsConfig[key];
          if (key !== 'prototype') {
            switch (fieldConfig['type']) {
              case 'Boolean':
                return(
                  <div key={ key }>
                    { Admin.renderBooleanDropDown.bind(this)({ columnWidth: 6, entity: "bannerFieldsObject", property: key, columnHeader: key }) }
                  </div>
                );
              case 'Dropdown':
                let options = fieldConfig.options.map((option) => { return { text: option } });
                return(
                  <div key={ key }>
                    { Admin.renderDropDown.bind(this)({
                      columnWidth: 6,
                      entity: "bannerFieldsObject",
                      property: key,
                      options,
                      optionLabel: "text",
                      optionValue: "text"
                    }) }
                  </div>
                );
              default:
                return(
                  <div key={ key }>
                    { Admin.renderField.bind(this)({ columnWidth: 6, entity: "bannerFieldsObject", property: key, columnHeader: key, placeholder: fieldConfig.placeholder }) }
                  </div>
                );
            }
          } else {
            return null;
          }
        }) }
      </div>
    );
  }

  componentDidUpdate() {
    $('.match-height-layout').matchHeight();
  }
}

const mapStateToProps = (reducers, props) => {
  return {
    banner: reducers.standardReducer.banner,
    bannerFieldsConfig: reducers.standardReducer.bannerFieldsConfig,
    bannerFieldsObject: reducers.standardReducer.bannerFieldsObject,
    entities: reducers.standardReducer.entities,
    errors: reducers.standardReducer.errors,
    pages: reducers.standardReducer.pages,
    bannerPages: reducers.standardReducer.bannerPages,
    webinars: reducers.standardReducer.webinars,
    bannerWebinars: reducers.standardReducer.bannerWebinars
  };
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntity, createEntity, updateEntity, deleteEntity }, dispatch);
}

export default connect(mapStateToProps, mapDispatchToProps)(withExitPrompt(BannerDetails));
