import React from 'react'
import { connect } from 'react-redux'
import { bindActionCreators } from 'redux'
import Modal from 'react-modal'
import isEqual from 'lodash/isEqual'
import cloneDeep from 'lodash/cloneDeep'
import TextEditor from './editor/text-editor'
import Admin from './../../admin'
import ConfirmDelete from '../confirm-delete'
import { ConfirmDeleteModalStyles } from '../helpers/modal-styles'
import { fetchEntity, createEntity, updateEntity, deleteEntity } from '../../actions/index'
import { withExitPrompt } from '../util/with-exit-prompt'
import { Field } from '../field'
import { Dropdown, BooleanDropdown } from '../drop-down'

enum TabNames {
  Standard = 'Standard',
  SEO = 'SEO',
  Content = 'Content'
}

type AuthorType = 'User' | 'Objekt'

interface Author {
  displayText: string
  value: string
  polymorphicTypeValue: AuthorType
}

interface BlogPost {
  id?: number
  active?: boolean
  editorContentState?: any
  htmlContent?: any
  indexPage?: boolean
  authorId?: string
  authorType?: AuthorType
}



interface BlogPostDetailsProps {
  allCategories?: any
  authors: Author[]
  blogPost: BlogPost
  categories: number[]
  deleteModalOpen?: boolean
  errors: string[]
  fetching?: boolean
  fetchEntity: any
  updateEntity: any
}

interface BlogPostDetailsState extends ComponentState {
  allCategories: any[]
  authors: any[]
  blogPost: BlogPost
  blogPostSaved: BlogPost
  categories: number[]
  categoriesSaved: number[]
  currentTab: TabNames
  editorIsSaved: boolean
}

const tabNames = [TabNames.Standard, TabNames.SEO, TabNames.Content];

class BlogPostDetails extends React.Component<BlogPostDetailsProps, BlogPostDetailsState> {

  state: BlogPostDetailsState = {
    allCategories: [],
    authors: [],
    blogPost: {},
    blogPostSaved: {},
    categories: [],
    categoriesSaved: [],
    currentTab: TabNames.Standard,
    deleteModalOpen: false,
    editorIsSaved: true,
    errors: [],
    fetching: true
  };

  componentDidMount() {
    let pathDirectories = window.location.pathname.split('/');
    this.props.fetchEntity({
      id: pathDirectories[pathDirectories.length - 1],
      directory: "blog_posts",
      entityName: "blogPost"
    }).then(() => {
      let blogPost = this.props.blogPost;
      this.setState({
        fetching: false,
        allCategories: this.props.allCategories,
        authors: this.props.authors,
        blogPost,
        blogPostSaved: cloneDeep(blogPost),
        categories: this.props.categories,
        categoriesSaved: cloneDeep(this.props.categories),
        changesToSave: false
      });
    });
  }

  clickTab(tabName) {
    this.setState({
      currentTab: tabName
    });
  }

  clickSave() {
    this.setState({
      fetching: true,
      justSaved: true
    });
    let pathDirectories = window.location.pathname.split('/');
    this.props.updateEntity({
      id: pathDirectories[pathDirectories.length - 1],
      directory: 'blog_posts',
      entityName: 'blogPost',
      entity: this.state.blogPost,
      additionalData: {
        categories: this.state.categories
      }
    }).then(() => {
      this.setState({
        fetching: false,
        blogPost: this.props.blogPost,
        blogPostSaved: cloneDeep(this.props.blogPost),
        categories: this.props.categories,
        categoriesSaved: cloneDeep(this.props.categories),
        changesToSave: false
      });
    }, () => {
      this.setState({
        fetching: false,
        errors: this.props.errors
      });
    });
  }

  checkForChanges() {
    return !isEqual(this.state.blogPost, this.state.blogPostSaved) || !isEqual(this.state.categories, this.state.categoriesSaved);
  }

  changeFieldArgs() {
    return {
      thing: "blogPost",
      errorsArray: this.state.errors,
      changesFunction: this.checkForChanges.bind(this)
    };
  }

  updateBlogPost(blogPost) {
    this.setState({
      blogPost,
      changesToSave: !isEqual(blogPost, this.state.blogPostSaved)
    });
  }

  render() {
    return(
      <div>
        <div id="blog-post-details" className="component details-component">
          <div className="clearfix">
            <h1 className="search-box-header">Blog Post Details</h1>
            { this.renderTabs() }
          </div>
          <div className="white-box">
            { Admin.renderSpinner(this.state.fetching) }
            { Admin.renderGrayedOut(this.state.fetching) }
            { this.renderTabContents.call(this) }
            <div className="row">
              <div className="col-xs-12">
                <a className={ "orange-button standard-button-width btn" + Admin.renderDisabledButtonClass(this.state.fetching || !this.state.changesToSave) } onClick={ this.clickSave.bind(this) }>
                  { Admin.saveButtonText.call(this) }
                </a>
                <a className={ "delete-button" + Admin.renderDisabledButtonClass(this.state.fetching) } onClick={ Admin.changeState.bind(this, "deleteModalOpen", true) }>
                  Delete
                </a>
                <a className={ 'preview-button light-orange-button float-button ' + Admin.renderDisabledButtonClass(this.state.fetching) } href={ `/admin/blog_posts/${this.state.blogPost.id}/preview` } target="_blank">
                  Preview
                </a>
              </div>
            </div>
          </div>
        </div>
        <Modal isOpen={ this.state.deleteModalOpen } onRequestClose={ Admin.closeDeleteModal.bind(this) } contentLabel="Modal" style={ ConfirmDeleteModalStyles }>
          <ConfirmDelete entityName={ 'blog post' } skipCheck={ true } closeModal={ Admin.closeDeleteModal.bind(this) } closeAndFetch={ Admin.closeDeleteModalAndFetch.bind(this) } delete={ Admin.deleteEntity.bind(this) } />
        </Modal>
      </div>
    );
  }

  getOnChange(key: keyof BlogPost) {
    return (e: React.ChangeEvent<HTMLSelectElement>) =>
      this.updateBlogPost({...this.state.blogPost, [key]: e.target.value })
  }

  onAuthorSelect = (e: React.ChangeEvent<HTMLSelectElement>) => {
    const selectedIndex = parseInt(e.target.value)
    const newAuthor = this.props.authors[selectedIndex]
    this.updateBlogPost({
      ...this.state.blogPost,
      authorId: newAuthor.value,
      authorType: newAuthor.polymorphicTypeValue
    })
  }

  renderTabContents() {
    switch(this.state.currentTab) {
      case 'Standard':
        const selectedAuthor = this.props.authors && this.props.authors.findIndex(author =>
          author.value === this.state.blogPost.authorId && author.polymorphicTypeValue === this.state.blogPost.authorType
        )
        return([
          <div className="row" key={ 1 }>
            { Admin.renderField.bind(this)({ columnWidth: 7, entity: 'blogPost', property: 'title' }) }
            <Field columnWidth={3} className="dropdown-limit-6" columnHeader="Author">
              <Dropdown onChange={this.onAuthorSelect} value={selectedAuthor}>
                {this.props.authors && this.props.authors.map(({ displayText, value }, idx) => (
                  <option key={value} value={idx}>{displayText}</option>
                ))}
              </Dropdown>
            </Field>
            <Field columnWidth={2} className="boolean-dropdown" columnHeader="Active">
              <BooleanDropdown data-field="active" value={this.state.blogPost.active} onChange={this.getOnChange('active')} />
            </Field>
          </div>,
          <div className="row" key={ 2 }>
            { Admin.renderField.bind(this)({ columnWidth: 10, entity: 'blogPost', property: 'url', placeholder: '/how-to-crush-it' }) }
            { Admin.renderField.bind(this)({ columnWidth: 2, entity: 'blogPost', property: 'date' }) }
          </div>,
          <div className="row" key={ 3 }>
            { Admin.renderArrayCheckboxes.bind(this)({
              columnWidth: 3,
              entity: 'blogPost',
              optionsArrayName: 'allCategories',
              selectedOptionsArrayName: 'categories',
              columnHeader: 'Categories',
              optionLabel: 'name',
              optionValue: 'value'
            }) }
            { Admin.renderImageField.bind(this)({
              columnWidth: 2,
              entity: 'blogPost',
              property: 'coverPhotoId',
              imageUrlProperty: 'coverPhotoUrl',
              createImageRecord: true,
              columnHeader: 'Cover Photo'
            }) }
            { Admin.renderImageField.bind(this)({
                columnWidth: 2,
                entity: 'blogPost',
                property: 'mobileCoverPhotoId',
                imageUrlProperty: 'mobileCoverPhotoUrl',
                createImageRecord: true,
                columnHeader: 'Mobile Cover Photo'
              }) }
            { Admin.renderField.bind(this)({ columnWidth: 5, entity: 'blogPost', property: 'coverPhotoAltText' }) }
          </div>
        ]);
      case 'SEO':
        return([
          <div className="row" key={ 1 }>
            { Admin.renderField.bind(this)({ columnWidth: 10, entity: 'blogPost', property: 'titleText' }) }
            <Field columnWidth={2} className="boolean-dropdown" columnHeader="Index Page">
              <BooleanDropdown value={this.state.blogPost.indexPage} onChange={this.getOnChange('indexPage')} />
            </Field>
          </div>,
          <div className="row" key={ 2 }>
            { Admin.renderField.bind(this)({ columnWidth: 12, entity: 'blogPost', property: 'description' }) }
          </div>,
          <div className="row" key={ 3 }>
            { Admin.renderField.bind(this)({ columnWidth: 12, entity: 'blogPost', property: 'canonicalUrl' }) }
          </div>
        ]);
      case 'Content':
        return(
          <div className="row">
            <div className="col-xs-12">
              <TextEditor blogPost={ this.state.blogPost } updateBlogPost={ this.updateBlogPost.bind(this) } />
            </div>
          </div>
        );
      default:
        return null;
    }
  }

  renderTabs() {
    return(
      <div className="tabs-row">
        { tabNames.map((tabName, index) => this.renderTab(tabName, index)) }
      </div>
    );
  }

  renderTab(tabName, index) {
    const className = `tab${this.state.currentTab === tabName ? ' selected' : ''}`;
    return (
      <div className={ className } key={ index } onClick={ () => this.clickTab(tabName) }>
        { tabName }
      </div>
    );
  }

  componentDidUpdate() {
    $('.match-height-layout').matchHeight();
  }
}

const mapStateToProps = (reducers, props) => {
  return reducers.standardReducer;
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntity, createEntity, updateEntity, deleteEntity }, dispatch);
}

export default connect(mapStateToProps, mapDispatchToProps)(withExitPrompt(BlogPostDetails));
