import React from 'react'
import { connect } from 'react-redux'
import { bindActionCreators } from 'redux'
import Admin from './../../admin'
import Tools from './../../../app/assets/javascripts/teachable/tools'
import { fetchBlogSettings, updateBlogSettings } from '../../actions/index'
import upperFirst from 'lodash/upperFirst';
import snakeCase from 'lodash/snakeCase';
import flattenDeep from 'lodash/flattenDeep';
import isEqual from 'lodash/isEqual';
import cloneDeep from 'lodash/cloneDeep'
import { withExitPrompt } from '../util/with-exit-prompt'

interface BlogSettingsProps {
  blogSettings: any;
  categories: any;
  contentDownloads: any[];
  errors: string[];
  fetchBlogSettings: any;
  updateBlogSettings: any;
}

interface BlogSettingsState extends ComponentState {
  blogSettings: any;
  blogSettingsSaved: any;
  categories: any;
  contentDownloads: any[];
  currentTab: number;
  tabErrors: any;
  tabNames: string[];
}

class BlogSettings extends React.Component<BlogSettingsProps, BlogSettingsState> {

  constructor(props) {
    super(props);

    this.state = {
      fetching: true,
      blogSettings: {},
      blogSettingsSaved: {},
      contentDownloads: [],
      categories: [],
      currentTab: 0,
      errors: [],
      justSaved: false,
      tabNames: ['Homepage', 'Resources'],
      tabErrors: {
        Homepage: this.formatTabErrors(['homepage - header', 'homepage - subheader', 'homepage - contentDownload1WorkflowID', 'homepage - contentDownload1EventName']),
        Resources: this.formatTabErrors(['resources - header', 'resources - subheader', 'resources - featuredContentDownload1EventName', 'resources - featuredContentDownload1WorkflowID'])
      }
    };
  }

  componentDidMount() {
    this.props.fetchBlogSettings().then(() => {
      const { contentDownloads, categories } = this.props;
      const categoryNames = Object.keys(categories).map(c => categories[c].name);
      let { blogSettings } = this.props;
      blogSettings = {
        ...blogSettings,
        ...this.formattedCategoryData(categories)
      };
      let stateToSave = {
        fetching: false,
        blogSettings,
        blogSettingsSaved: cloneDeep(blogSettings),
        categories,
        contentDownloads,
        tabNames: this.state.tabNames.concat(categoryNames),
      };
      this.setState(stateToSave, () => {
        Admin.setUpNiceSelect({ selector: 'select', func: Admin.changeField.bind(this, this.changeFieldArgs()) });
        const getCategoryErrors = (categoryNamespace) => {
          return [
            `${categoryNamespace} - name`,
            `${categoryNamespace} - url`,
            `${categoryNamespace} - titleText`,
            `${categoryNamespace} - description`,
            `${categoryNamespace} - header`,
            `${categoryNamespace} - subheader`,
            `${categoryNamespace} - contentDownload1Id`,
            `${categoryNamespace} - contentDownload1EventName`,
            `${categoryNamespace} - contentDownload1WorkflowId`,
            `${categoryNamespace} - contentDownload2Id`,
            `${categoryNamespace} - contentDownload2EventName`,
            `${categoryNamespace} - contentDownload2WorkflowId`,
          ]
        };
        Object.keys(categories).forEach((namespace) => {
          this.state.tabErrors[namespace] = this.formatTabErrors(getCategoryErrors(namespace));
        });
      });
    });
  }

  formattedCategoryData(categories) {
    const formattedCategoryData = {};
    Object.keys(categories).forEach((namespace) => {
      Object.keys(categories[namespace]).forEach((k) => {
        const newName = `${namespace} - ${k}`;
        formattedCategoryData[newName] = categories[namespace][k];
      });
    });
    return formattedCategoryData;
  }

  formattedFormData() {
    const formattedData = {};
    const { blogSettings } = this.state;
    delete blogSettings.prototype;
    const uniqPrefixes = Object.keys(blogSettings).map(k => k.toLowerCase().split(' - ')[0])
    uniqPrefixes.forEach((uniquePrefix) => {
      formattedData[uniquePrefix] = {};
      Object.keys(blogSettings).forEach((formDataKey) => {
        if (formDataKey.startsWith(uniquePrefix)) {
          if (!formDataKey.match(/homepage|resources/)) {
            formattedData[uniquePrefix][snakeCase(formDataKey.split(' - ')[1])] = blogSettings[formDataKey];
          } else {
            const formattedDataKey = (
              Tools.unCamelCase(upperFirst(formDataKey))
                .replace(/([1-9])([A-Z])/g, '$1 $2')
                .replace(/([a-z])([1-9])/g, '$1 $2')
                .replace(/( - [a-z])/, m => m.toUpperCase())
                .replace('Id', 'ID')
            );
            formattedData[uniquePrefix][formattedDataKey] = blogSettings[formDataKey];
          }
        }
      });
    });
    return formattedData;
  }

  checkForChanges() {
    return !isEqual(this.state.blogSettings, this.state.blogSettingsSaved);
  }

  changeFieldArgs() {
    return {
      thing: 'blogSettings',
      errorsArray: this.state.errors,
      changesFunction: this.checkForChanges.bind(this)
    }
  }

  clickTab(tabIndex) {
    this.setState({
      currentTab: tabIndex
    });
  }

  clickSave() {
    const data = this.formattedFormData();
    this.setState({
      fetching: true,
      justSaved: true
    });
    this.props.updateBlogSettings(data).then(() => {
      const { categories } = this.props;
      let { blogSettings } = this.props;
      blogSettings = {
        ...blogSettings,
        ...this.formattedCategoryData(categories)
      };
      this.setState({
        fetching: false,
        changesToSave: false,
        blogSettings,
        blogSettingsSaved: cloneDeep(blogSettings),
        categories
      });
    }, () => {
      this.setState({
        fetching: false,
        errors: this.props.errors
      });
    });
  }

  render() {
    return(
      <div id="blog-header-component" className="component details-component">
        <div className="clearfix">
          <h1 className="search-box-header">Blog Settings</h1>
          { this.renderTabs() }
        </div>
        <div className="white-box">
          { Admin.renderSpinner(this.state.fetching) }
          { Admin.renderGrayedOut(this.state.fetching) }
          { this.renderTabContents() }
          <div>
            <a className={ "btn orange-button standard-button-width" + Admin.renderDisabledButtonClass(this.state.fetching || !this.state.changesToSave) } onClick={ this.clickSave.bind(this) }>
              { Admin.saveButtonText.call(this) }
            </a>
          </div>
        </div>
      </div>
    );
  }

  renderHomepageTab() {
    return(
      <div className="tab-content">
        <div className="row">
          { Admin.renderField.bind(this)({ columnWidth: 12, entity: 'blogSettings', property: 'homepage - header', columnHeader: 'Header' }) }
        </div>
        <div className="row">
          { Admin.renderField.bind(this)({ columnWidth: 12, entity: 'blogSettings', property: 'homepage - subheader', columnHeader: 'Subheader' }) }
        </div>
        <div className="row">
          { Admin.renderDropDown.bind(this)({
            columnWidth: 7,
            columnHeader: 'Content Download 1',
            entity: 'blogSettings',
            property: 'homepage - contentDownload1',
            optionsArrayName: 'contentDownloads',
            optionLabel: 'displayText',
            optionValue: 'value',
            limit: 6
          }) }
          { Admin.renderField.bind(this)({ columnWidth: 3, entity: 'blogSettings', property: 'homepage - contentDownload1EventName', columnHeader: 'Content Download 1 Event Name' }) }
          { Admin.renderField.bind(this)({ columnWidth: 2, entity: 'blogSettings', property: 'homepage - contentDownload1WorkflowId', columnHeader: 'Workflow ID' }) }
        </div>
        <div className="row">
          { Admin.renderDropDown.bind(this)({
            columnWidth: 7,
            columnHeader: 'Content Download 2',
            entity: 'blogSettings',
            property: 'homepage - contentDownload2',
            optionsArrayName: 'contentDownloads',
            optionLabel: 'displayText',
            optionValue: 'value',
            limit: 6
          }) }
          { Admin.renderField.bind(this)({ columnWidth: 3, entity: 'blogSettings', property: 'homepage - contentDownload2EventName', columnHeader: 'Content Download 2 Event Name' }) }
          { Admin.renderField.bind(this)({ columnWidth: 2, entity: 'blogSettings', property: 'homepage - contentDownload2WorkflowId', columnHeader: 'Workflow ID' }) }
        </div>
      </div>
    );
  }

  renderResourcesTab() {
    const { categories } = this.state;
    const categoryNames = categories && {
      1: categories.category1.name,
      2: categories.category2.name
    };
    return (
      <div className="tab-content">
        <div className="row">
          { Admin.renderField.bind(this)({ columnWidth: 12, entity: 'blogSettings', property: 'resources - header', columnHeader: 'Header' }) }
        </div>
        <div className="row">
          { Admin.renderField.bind(this)({ columnWidth: 12, entity: 'blogSettings', property: 'resources - subheader', columnHeader: 'Subheader' }) }
        </div>
        <div className="row">
          { Admin.renderDropDown.bind(this)({
            columnWidth: 7,
            columnHeader: 'Featured Content Download',
            entity: 'blogSettings',
            property: 'resources - featuredContentDownload',
            optionsArrayName: 'contentDownloads',
            optionLabel: 'displayText',
            optionValue: 'value',
            limit: 6
          }) }
          { Admin.renderField.bind(this)({ columnWidth: 3, entity: 'blogSettings', property: 'resources - featuredContentDownloadEventName', columnHeader: 'Event Name' }) }
          { Admin.renderField.bind(this)({ columnWidth: 2, entity: 'blogSettings', property: 'resources - featuredContentDownloadWorkflowId', columnHeader: 'Workflow ID' }) }
        </div>
        <div className="row">
          { Admin.renderDropDown.bind(this)({
            columnWidth: 7,
            columnHeader: `${categoryNames[1]} Content Download 1`,
            entity: 'blogSettings',
            property: 'resources - category1ContentDownload1',
            optionsArrayName: 'contentDownloads',
            optionLabel: 'displayText',
            optionValue: 'value',
            limit: 6
          }) }
        </div>
        <div className="row">
          { Admin.renderDropDown.bind(this)({
            columnWidth: 7,
            columnHeader: `${categoryNames[1]} Content Download 2`,
            entity: 'blogSettings',
            property: 'resources - category1ContentDownload2',
            optionsArrayName: 'contentDownloads',
            optionLabel: 'displayText',
            optionValue: 'value',
            limit: 6
          }) }
        </div>
        <div className="row">
          { Admin.renderDropDown.bind(this)({
            columnWidth: 7,
            columnHeader: `${categoryNames[1]} Content Download 3`,
            entity: 'blogSettings',
            property: 'resources - category1ContentDownload3',
            optionsArrayName: 'contentDownloads',
            optionLabel: 'displayText',
            optionValue: 'value',
            limit: 6
          }) }
        </div>
        <div className="row">
          { Admin.renderDropDown.bind(this)({
            columnWidth: 7,
            columnHeader: `${categoryNames[1]} Content Download 4`,
            entity: 'blogSettings',
            property: 'resources - category1ContentDownload4',
            optionsArrayName: 'contentDownloads',
            optionLabel: 'displayText',
            optionValue: 'value',
            limit: 6
          }) }
        </div>
        <div className="row">
          { Admin.renderDropDown.bind(this)({
            columnWidth: 7,
            columnHeader: `${categoryNames[1]} Content Download 5`,
            entity: 'blogSettings',
            property: 'resources - category1ContentDownload5',
            optionsArrayName: 'contentDownloads',
            optionLabel: 'displayText',
            optionValue: 'value',
            limit: 6
          }) }
        </div>
        <br />
        <div className="row">
          { Admin.renderDropDown.bind(this)({
            columnWidth: 7,
            columnHeader: `${categoryNames[2]} Content Download 1`,
            entity: 'blogSettings',
            property: 'resources - category2ContentDownload1',
            optionsArrayName: 'contentDownloads',
            optionLabel: 'displayText',
            optionValue: 'value',
            limit: 6
          }) }
        </div>
        <div className="row">
          { Admin.renderDropDown.bind(this)({
            columnWidth: 7,
            columnHeader: `${categoryNames[2]} Content Download 2`,
            entity: 'blogSettings',
            property: 'resources - category2ContentDownload2',
            optionsArrayName: 'contentDownloads',
            optionLabel: 'displayText',
            optionValue: 'value',
            limit: 6
          }) }
        </div>
        <div className="row">
          { Admin.renderDropDown.bind(this)({
            columnWidth: 7,
            columnHeader: `${categoryNames[2]} Content Download 3`,
            entity: 'blogSettings',
            property: 'resources - category2ContentDownload3',
            optionsArrayName: 'contentDownloads',
            optionLabel: 'displayText',
            optionValue: 'value',
            limit: 6
          }) }
        </div>
        <div className="row">
          { Admin.renderDropDown.bind(this)({
            columnWidth: 7,
            columnHeader: `${categoryNames[2]} Content Download 4`,
            entity: 'blogSettings',
            property: 'resources - category2ContentDownload4',
            optionsArrayName: 'contentDownloads',
            optionLabel: 'displayText',
            optionValue: 'value',
            limit: 6
          }) }
        </div>
        <div className="row">
          { Admin.renderDropDown.bind(this)({
            columnWidth: 7,
            columnHeader: `${categoryNames[2]} Content Download 5`,
            entity: 'blogSettings',
            property: 'resources - category2ContentDownload5',
            optionsArrayName: 'contentDownloads',
            optionLabel: 'displayText',
            optionValue: 'value',
            limit: 6
          }) }
        </div>
        <br />
      </div>
    );
  }

  renderCategoryTab(tabIndex) {
    const { tabNames, categories } = this.state;
    const categoryName = tabNames[tabIndex];
    const categoryNamespace = Object.keys(categories).filter(c => categories[c].name === categoryName)[0];
    return (
      <div className="tab-content">
        <div className="row">
          { Admin.renderField.bind(this)({ columnWidth: 3, entity: 'blogSettings', property: `${categoryNamespace} - name`, columnHeader: 'Name', readOnly: true }) }
          { Admin.renderField.bind(this)({ columnWidth: 3, entity: 'blogSettings', property: `${categoryNamespace} - url`, columnHeader: 'Url', readOnly: true }) }
          { Admin.renderField.bind(this)({ columnWidth: 6, entity: 'blogSettings', property: `${categoryNamespace} - titleText`, columnHeader: 'Title Text' }) }
        </div>
        <div className="row">
          { Admin.renderField.bind(this)({ columnWidth: 12, entity: 'blogSettings', property: `${categoryNamespace} - description`, columnHeader: 'Description' }) }
        </div>
        <div className="row">
          { Admin.renderField.bind(this)({ columnWidth: 5, entity: 'blogSettings', property: `${categoryNamespace} - header`, columnHeader: 'Header' }) }
          { Admin.renderField.bind(this)({ columnWidth: 7, entity: 'blogSettings', property: `${categoryNamespace} - subheader`, columnHeader: 'Subheader' }) }
        </div>
        <div className="row">
          { Admin.renderDropDown.bind(this)({
            columnWidth: 7,
            columnHeader: 'Content Download 1',
            entity: 'blogSettings',
            property: `${categoryNamespace} - contentDownload1Id`,
            optionsArrayName: 'contentDownloads',
            optionLabel: 'displayText',
            optionValue: 'value',
            limit: 6
          }) }
          { Admin.renderField.bind(this)({ columnWidth: 3, entity: 'blogSettings', property: `${categoryNamespace} - contentDownload1EventName`, columnHeader: 'Content Download 1 Event Name' }) }
          { Admin.renderField.bind(this)({ columnWidth: 2, entity: 'blogSettings', property: `${categoryNamespace} - contentDownload1WorkflowId`, columnHeader: 'Workflow ID' }) }
        </div>
        <div className="row">
          { Admin.renderDropDown.bind(this)({
            columnWidth: 7,
            columnHeader: 'Content Download 2',
            entity: 'blogSettings',
            property: `${categoryNamespace} - contentDownload2Id`,
            optionsArrayName: 'contentDownloads',
            optionLabel: 'displayText',
            optionValue: 'value',
            limit: 6
          }) }
          { Admin.renderField.bind(this)({ columnWidth: 3, entity: 'blogSettings', property: `${categoryNamespace} - contentDownload2EventName`, columnHeader: 'Content Download 2 Event Name' }) }
          { Admin.renderField.bind(this)({ columnWidth: 2, entity: 'blogSettings', property: `${categoryNamespace} - contentDownload2WorkflowId`, columnHeader: 'Workflow ID' }) }
        </div>
      </div>
    );
  }

  renderTabContents() {
    let tabContents;
    switch (this.state.currentTab) {
      case 0:
        tabContents = this.renderHomepageTab();
        break;
      case 1:
        tabContents = this.renderResourcesTab();
        break;
      case 2:
      case 3:
      case 4:
        tabContents = this.renderCategoryTab(this.state.currentTab);
        break;
      default:
        break;
    }
    return tabContents;
  }

  renderTabs() {
    return (
      <ul className="tabs-row">
        { this.state.tabNames.map((tabName, i) => this.renderTab(tabName, i)) }
      </ul>
    );
  }

  renderTab(tabName, i) {
    return (
      <div className={ `tab${this.state.currentTab === i ? ' selected' : ''}`} key={ i } onClick={ () => this.clickTab(i) }>
        { tabName }
      </div>
    );
  }

  formatTabErrors(tabProperties) {
    const tabErrors = flattenDeep(tabProperties.map(property => Admin.errors[property]));
    return new Set(tabErrors);
  }

  assignTabErrors() {
    $('.tab').removeClass('error');
    const { errors, tabErrors, categories } = this.state;
    errors.forEach((error) => {
      Object.keys(tabErrors).forEach((tabName) => {
        if (tabErrors[tabName].has(error)) {
          const tabClassName = ((categories[tabName] && categories[tabName].name) || tabName).replace(/\s/g, '-');
          $(`.${tabClassName}-tab`).addClass('error');
        }
      });
    });
  }

  componentDidUpdate() {
    this.assignTabErrors();
    Admin.resetNiceSelect({
      selector: 'select',
      func: Admin.changeField.bind(this, this.changeFieldArgs()),
      removeOrphanedDivs: true
    });
  }
}

const mapStateToProps = (reducers, props) => {
  return reducers.standardReducer;
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchBlogSettings, updateBlogSettings }, dispatch);
}

export default connect(mapStateToProps, mapDispatchToProps)(withExitPrompt(BlogSettings));
