import React from 'react';
import { STYLE_BUTTON_SIZE } from '../../../util/toolbar-util';
import _BlockEntity from '../_common/_block_entity';
import _MediaTextInput from './_media_text_input';
import _MediaResizeCorner from './_media_resize_corner';
import _DeleteButton from '../_common/_delete_button';

const ALIGN_OPTIONS = {
  left: ['align-left', 'align-floatLeft'],
  center: ['align-center'],
  right: ['align-right', 'align-floatRight']
};

const RESIZE_CORNER_DIRECTIONS = ['ne', 'nw', 'se', 'sw'];

const DELETE_BUTTON_HEIGHT = 55;
const SPACE_BEFORE_TEXT_FIELDS = 10;
const TEXT_FIELD_HEIGHT = 56;

export default class _Media extends _BlockEntity {
  numTextFields: number
  
  constructor(props) {
    super(props);
    this.handleScroll = this.handleScroll.bind(this);
  }

  getInitialEntityDataValues() {
    let initialValues = { ...this.getOwnEntityData() };
    if (initialValues.width) initialValues.width = parseInt(initialValues.width);
    if (initialValues.height) initialValues.height = parseInt(initialValues.height);
    return initialValues;
  }

  // SETUP -----

  specificState() {
    return {
      isResizing: false,
      isMounted: false,

      previewWidth: null,
      previewHeight: null,
      width: this.initialEntityDataValues.width,
      height: this.initialEntityDataValues.height,
      clip: null,
    };
  }

  // PUBLIC -----

  renderEntity() {
    return <p>GOT NO ENTITY!!</p>;
  }

  entityStyles() {
    const { isResizing, width, previewWidth } = this.state;
    if (!this.state.isMounted) { return { display: 'block' }; }

    return {
      width: isResizing ? previewWidth : width,
    };
  }

  previewResize(previewWidth, previewHeight) {
    this.setState({
      isResizing: true,
      previewWidth,
      previewHeight
    });
  }

  resize(resizeFactor) {
    let { width, height } = this.state;
    width *= resizeFactor;
    height *= resizeFactor;
    this.setState({
      isResizing: false,
      width,
      height,
      previewWidth: null,
      previewHeight: null,
    }, () => this.mergeOwnEntityData({ width, height }));
  }

  // LIFECYCLE -----

  componentDidMount() {
    super.componentDidMount();
    window.addEventListener('scroll', this.handleScroll);
    $('#text-editor-admin').on('scroll', this.handleScroll);

    this.numTextFields = $(this.selfSelector + ' .media-text-input').length;

    setTimeout( () => {
      this.setState({
        width: this.state.width || this.refs.entity["offsetWidth"],
        height: this.state.height || this.refs.entity["offsetHeight"]
      }, () => this.setState({ isMounted: true }, () => this.forceUpdate()));
    }, 500);
  }

  componentWillUnmount() {
    super.componentWillUnmount();
    $('#text-editor-admin').off('scroll', this.handleScroll);
    window.removeEventListener('scroll', this.handleScroll);
  }

  // HANDLERS -----

  handleScroll() {
    if (this.state.isSelected) { this.forceUpdate(); }
  }

  // RENDERING -----

  render() {
    const props = {
      ref: 'wrapper',
      className: 'media-component',
      onMouseDown: this.handleMouseDown.bind(this),
      'data-remain-read-only': 'true'
    }
    return (
      <div { ...props } >
        { this.renderEntity() }
        { this.renderEditUI() }
      </div>
    );
  }

  renderLinkIndicator() {
    const href =  this.getOwnEntityData().href;
    const hasLink = href && href !== '';
    if (!hasLink) return null;

    const { isSelected } = this.state;

    const props = {
      className: 'link-indicator ' + (isSelected ? 'selected' : ''),
      src: window.Images['linkPressed'],
    };

    return <img { ...props } />;
  }

  renderEditUI() {
    const props = {
      className: 'media-border ui-container ' + (this.state.isSelected ? ' selected' : ''),
      style: this._getBorderPositionStyles(),
    }
    return (
      <div { ...props } >
        { this.renderAlignButtons('left') }
        { this.renderAlignButtons('center') }
        { this.renderAlignButtons('right') }
        { this.renderTextInputs() }
        { this.renderResizeCorners() }
        { this.renderDeleteButton() }
      </div>
    );
  }

  renderResizeCorners() {
    const { width, height, isSelected } = this.state;
    const props = {
      width,
      height,
      resize: this.resize.bind(this),
      previewResize: this.previewResize.bind(this),
      toolbarHeight: this._getToolbarHeight(),
      isSelected
    };
    return RESIZE_CORNER_DIRECTIONS.map((dir) => {
      return <_MediaResizeCorner { ...props } key={ dir } direction={ dir }/>
    });
  }

  renderAlignButtons(direction) {
    const { renderStyleButton } = this.props.blockProps;
    const buttonOptions = {
      isCollapsed: !this.state.isSelected || this.state.isResizing,
    }
    const containerClassName = (
      'align-options-container '
      + direction
      + (this._isUnderSized() ? ' undersized' : '')
    );

    const alignOptions = ALIGN_OPTIONS[direction];
    return (
      <div className={ containerClassName }>
        { alignOptions.map(dir => renderStyleButton(dir, 'meta', buttonOptions)) }
      </div>
    );
  }

  renderTextInputs() {
    const isCollapsed = !this.state.isSelected || this.state.isResizing;
    switch (this.entityType) {
      case 'IMAGE':
        return (
          <div className={ 'media-text-input-container' + (isCollapsed ? ' collapsed' : '') }>
            <_MediaTextInput
              isCollapsed={ isCollapsed }
              resourceName="Alt Text"
              handleChange={ (value) => this.mergeOwnEntityData({ alt: value }) }
              value={ this.initialEntityDataValues.alt }
            />
            <_MediaTextInput
              isCollapsed={ isCollapsed }
              resourceName="Href"
              handleChange={ (value) => this.mergeOwnEntityData({ href: value }) }
              value={ this.initialEntityDataValues.href }
            />
            <_MediaTextInput
              isCollapsed={ isCollapsed }
              resourceName="Id"
              handleChange={ (value) => this.mergeOwnEntityData({ id: value }) }
              value={ this.initialEntityDataValues.id }
            />
            <_MediaTextInput
              isCollapsed={ isCollapsed }
              resourceName="Src"
              value={ this.initialEntityDataValues.src }
              isReadonly
            />
          </div>
        );
      default:
        return null;
    }
  }

  // PRIVATE -----

  _getEntitySize() {
    const entity = this.refs.entity as HTMLElement;
    if (!entity) { return { width: 0, height: 0 }; }
    const { width, height } = entity.getBoundingClientRect();
    return { width, height };
  }

  _calculateClip() {
    const entity = this.refs.entity as HTMLElement;
    const entityRect = entity.getBoundingClientRect();
    const editorRect = $('#text-editor-admin')[0].getBoundingClientRect();

    const diffTops = (editorRect.top - entityRect.top + DELETE_BUTTON_HEIGHT);
    if (diffTops > 0) {
      const topOffset = diffTops - DELETE_BUTTON_HEIGHT;
      return { clip: `rect(${topOffset}px, 9999px, 9999px, -9999px)` };
    }

    const totalTextFieldHeight = SPACE_BEFORE_TEXT_FIELDS + this.numTextFields * TEXT_FIELD_HEIGHT;
    const diffBottom = (entityRect.bottom + totalTextFieldHeight - editorRect.bottom);
    if (diffBottom > 0) {
      const bottomOffset = entityRect.height + totalTextFieldHeight - diffBottom;
      return { clip: `rect(-9999px, 9999px, ${bottomOffset}px, -9999px)` };
    }

    return {};
  }

  _getBorderPositionStyles() {
    const entity = this.refs.entity as HTMLElement;
    if (!entity) {
      return { top: 0, left: 0, width: 0, height: 0 };
    } else {
      const { top, left, width, height } = entity.getBoundingClientRect();
      return { top, left, width, height, ...this._calculateClip() }
    }
  }

  _isUnderSized() {
    return (this.state.width <= (STYLE_BUTTON_SIZE * 2));
  }

  _getToolbarHeight() {
    return (window.outerHeight - window.innerHeight);
  }
}
