import createStyles from 'draft-js-custom-styles';
import { stateToHTML } from 'draft-js-export-html';
import ImageEntity from '../custom-components/block/image';
import CTAEntity from '../custom-components/block/cta';
import TestimonialEntity from '../custom-components/block/testimonial';
import CustomHTMLEntity from '../custom-components/block/custom-html';
import TipEntity from '../custom-components/block/tip';
import { CUSTOM_COMPONENT_RENDERERS } from './custom-component-renderers';
import {
  CUSTOM_STYLES_MAP,
  BLOCK_HEADER_STYLES,
  BLOCK_LIST_STYLES,
  BLOCK_RENDER_MAP,
  META_STYLES,
  CUSTOM_ENTITY_TYPES,
  DEFAULT_BLOCK_INLINE_STYLES,
  DEFAULT_ENTITY_BLOCK_STYLES,
} from './custom-styles';


const { styles, customStyleFn, exporter } = createStyles(['color'], 'CUSTOM_', CUSTOM_STYLES_MAP);

export default class StyleManager {
  block: {
    header: any
    list: string[]
  }
  blockRenderMap: any
  customComponentRenderers: any
  customEntity: any
  customStylesMap: any
  defaultBlockInline: any
  defaultEntityBlock: any
  exporter: any
  inline: string[]
  meta: any

  constructor() {
    this.customStylesMap = CUSTOM_STYLES_MAP;
    this.inline = Object.keys(this.customStylesMap).filter(s => !s.startsWith('PSEUDO_'));

    this.blockRenderMap = BLOCK_RENDER_MAP;
    this.block = {
      header: BLOCK_HEADER_STYLES,
      list: BLOCK_LIST_STYLES
    };

    this.meta = META_STYLES;
    this.exporter = exporter;
    this.defaultBlockInline = DEFAULT_BLOCK_INLINE_STYLES;
    this.defaultEntityBlock = DEFAULT_ENTITY_BLOCK_STYLES;
    this.customComponentRenderers = CUSTOM_COMPONENT_RENDERERS;
    this.customEntity = CUSTOM_ENTITY_TYPES;
  }

  customStyleFn(style, block) {
    return customStyleFn(style, block);
  }

  currentColor(editorState) {
    return styles.color.current(editorState);
  }

  toggleColor(editorState, style) {
    return styles.color.toggle(editorState, style);
  }

  blockStyleFn(block, contentState) {
    const classes = [];
    const blockType = block.getType();
    const blockData = block.getData();
    classes.push(blockType);
    classes.push('align-' + (blockData.get('align') || 'left'));
    switch(blockType) {
      case 'unordered-list-item':
      case 'ordered-list-item':
        classes.push('depth-' + (blockData.get('depth') || '0'));
        break;
      case 'atomic':
        const entityKey = block.getEntityAt(0);
        const entity = entityKey && contentState.getEntity(entityKey);
        if (!entity) { break; }
        const entityType = entity.getType().toLowerCase();
        classes.push('custom-block', entityType, `custom-block-entity-${entityKey}`);
      default:
        break;
    }
    return classes.join(' ');
  }

  blockRendererFn(block, contentState, customBlockProps) {
    if (block.getType() === 'atomic') {
      const entityKey = block.getEntityAt(0);
      const entityType = entityKey && contentState.getEntity(entityKey).getType();
      const blockRendererObj = {
        editable: false,
        props: {
          ...customBlockProps,
          defaultBlockStyles: this.defaultEntityBlock[entityType]
        }
      };
      switch (entityType) {
        case 'IMAGE':
          blockRendererObj["component"] = ImageEntity;
          break;
        case 'CTA':
          blockRendererObj["component"] = CTAEntity;
          break;
        case 'HTML':
          blockRendererObj["component"] = CustomHTMLEntity;
          break;
        case 'TESTIMONIAL':
          blockRendererObj["component"] = TestimonialEntity;
          break;
        case 'TIP':
          blockRendererObj["component"] = TipEntity;
          break;
      }
      return blockRendererObj;
    } else {
      return null;
    }
  }

  exportHTML(editorState) {
    const contentState = editorState.getCurrentContent();
    const options = this._HTMLOptions(editorState, contentState);
    const HTML = stateToHTML(contentState, options);
    return this._parseHTMLListItems(HTML);
  }

  // EXPORT HTML OPTIONS/HELPERS -----

  _HTMLOptions(editorState, contentState) {
    return {
      inlineStyles: this._HTMLInlineStyles(editorState),
      blockStyleFn: this._HTMLBlockStyleFn(contentState),
      entityStyleFn: this._HTMLEntityStyleFn(),
      blockRenderers: { atomic: (block) => this._HTMLCustomBlockRenderer(block, contentState) },
    };
  }

  _HTMLCustomBlockRenderer(block, contentState) {
    const entityKey = block.getEntityAt(0);
    const entity = entityKey && contentState.getEntity(entityKey);
    const entityType = entity && entity.getType();
    if (!entity) return;
    // if (!entity || ['IMAGE'].includes(entityType)) { return; }
    const rendererFn = this.customComponentRenderers[entityType];
    const entityData = entity.getData();
    const classNames = this.blockStyleFn(block, contentState);
    return rendererFn(entityData, classNames);
  }

  _HTMLEntityStyleFn() {
    return (entity) => {
      const entityType = entity.get('type');
      const entityData = entity.getData();

      switch (entityType) {
        case 'LINK':
          return {
            element: 'a',
            attributes: {
              class: 'link',
              href: entityData.url,
              target: entityData.url[0] == '#' ? '_self' : '_blank'
            }
          }
        default:
          return null;
      }
    };
  }

  _HTMLInlineStyles(editorState) {
    const inlineStyles = this.exporter(editorState);
    Object.keys(inlineStyles).filter(s => s.startsWith('PSEUDO_')).forEach(s => {
      delete inlineStyles[s];
    });
    return inlineStyles;
  }

  _HTMLBlockStyleFn(contentState) {
    return (block) => {
      const blockStyleObj = {
        attributes: {
          class: this.blockStyleFn(block, contentState) + ' public',
        }
      };

      const entityKey = block.getEntityAt(0);
      const entity = entityKey && contentState.getEntity(entityKey);

      if (block.getType() === 'atomic' && entity) {
        const entityData = entity.getData();
        const { width, height } = entityData;
        if (width && height) {
          blockStyleObj["style"] = {
            width,
            height
          }
        }
      }

      return blockStyleObj;
    }
  }

  _parseHTMLListItems(html) {
    const openLiTagRegex = new RegExp([
      '(<li)',           // Open <li> tag, one time
      '(\s*\w*[^\/<]*)', // Any whitespace, letters, anything else except / and <
      '(>)',             // Closing tag, one time
    ].join(''), 'g');

    return (
      html
        .replace(openLiTagRegex, (match) => match + '<div><span>')
        .replace(/<\/li>/g, '<\/span><\/div><\/li>')
    );
  }
}
