import React from "react";
import { connect } from 'react-redux'
import { bindActionCreators } from 'redux'
import Modal from "react-modal";
import isEqual from 'lodash/isEqual'
import cloneDeep from 'lodash/cloneDeep'
import sortBy from 'lodash/sortBy'
import ModalSelect from './modal-select';
import ConfirmDelete from "./confirm-delete";
import Admin from "./../admin";
import { SelectModalStyles, ConfirmDeleteModalStyles } from './helpers/modal-styles';
import { fetchEntity, createEntity, updateEntity, deleteEntity } from '../actions/index'
import { withExitPrompt } from "./util/with-exit-prompt";

interface EventSeriesWebinarType {
  name: string;
  id: string;
  time: string;
}

interface EventSeriesDetailsProps {
  createEntity: any;
  deleteEntity: any;
  errors: string[];
  fetchEntity: any;
  eventSeries: any;
  otherWebinars: any[];
  updateEntity: any;
  webinars: any[];
}

interface EventSeriesDetailsState extends ComponentState {
  deleteModalOpen: boolean;
  eventSeries: any;
  eventSeriesSaved: any;
  webinars: EventSeriesWebinarType[];
  otherWebinars: EventSeriesWebinarType[];
  webinarsModalOpen: boolean;
}

class EventSeriesDetails extends React.Component<EventSeriesDetailsProps, EventSeriesDetailsState> {

  constructor(props) {
    super(props);
    this.state = {
      fetching: true,
      changesToSave: false,
      justSaved: false,
      eventSeries: {},
      eventSeriesSaved: {},
      errors: [],
      deleteModalOpen: false,
      webinars: [],
      otherWebinars: [],
      webinarsModalOpen: false
    };
  }

  componentDidMount() {
    let pathDirectories = window.location.pathname.split('/');
    this.props.fetchEntity({
      id: pathDirectories[pathDirectories.length - 1],
      directory: "event_series",
      entityName: "eventSeries"
    }).then(() => {
      this.setState({
        fetching: false,
        eventSeries: this.props.eventSeries,
        eventSeriesSaved: cloneDeep(this.props.eventSeries),
        webinars: this.props.webinars,
        otherWebinars: this.props.otherWebinars,
        changesToSave: false
      }, () => {
        Admin.setUpNiceSelect({ selector: 'select', func: Admin.changeField.bind(this, this.changeFieldArgs()) });
      });
    });
  }

  clickSave() {
    this.setState({
      fetching: true,
      justSaved: true
    });
    this.props.updateEntity({
      id: this.state.eventSeries.id,
      directory: "event_series",
      entityName: "eventSeries",
      entity: this.state.eventSeries
    }).then(() => {
      this.setState({
        fetching: false,
        eventSeries: this.props.eventSeries,
        eventSeriesSaved: cloneDeep(this.props.eventSeries),
        changesToSave: false
      });
    }, () => {
      this.setState({
        fetching: false,
        errors: this.props.errors
      });
    });
  }

  checkForChanges() {
    return !isEqual(this.state.eventSeries, this.state.eventSeriesSaved);
  }

  changeFieldArgs() {
    return {
      thing: "eventSeries",
      errorsArray: this.state.errors,
      changesFunction: this.checkForChanges.bind(this)
    };
  }

  clickAddWebinar() {
    this.setState({
      webinarsModalOpen: true
    });
  }

  closeModal() {
    this.setState({
      webinarsModalOpen: false
    });
  }

  selectWebinar(e) {
    let webinarId = e.target.dataset.id;
    let eventSeriesId = this.state.eventSeries.id;
    this.setState({
      fetching: true,
      webinarsModalOpen: false
    }, () => {
      this.props.createEntity({
        directory: `event_series/${eventSeriesId}/webinars/${webinarId}`,
        arrayName: "webinars",
        entityName: "webinarsEventSeries",
        entity: {
          webinarId,
          eventSeriesId
        }
      }).then(() => {
        this.setState({
          fetching: false,
          webinars: this.props.webinars
        });
      });
    });
  }

  clickXButton(e) {
    this.setState({
      fetching: true
    });
    this.props.deleteEntity({
      directory: `event_series/${this.state.eventSeries.id}/webinars`,
      id: e.target.dataset.id,
      callback: (response) => {
        this.setState({
          fetching: false,
          webinars: response.webinars,
          otherWebinars: response.otherWebinars
        });
      }
    });
  }

  render() {
    return(
      <div id="event-series-details" className="component details-component">
        <h1>Event Series details</h1>
        <div className="white-box">
          { Admin.renderSpinner(this.state.fetching) }
          { Admin.renderGrayedOut(this.state.fetching) }
          <div className="row">
            { Admin.renderField.bind(this)({ columnWidth: 6, entity: "eventSeries", property: "name" }) }
            { Admin.renderField.bind(this)({ columnWidth: 6, entity: "eventSeries", property: "description" }) }
          </div>
          <div className="row">
            <div className="col-xs-12">
              <h2>Iterable Metadata</h2>
              <pre>{ JSON.stringify(this.state.eventSeries.iterableMetadata, null, 4) }</pre>
            </div>
          </div>
          <hr />
          <table className="admin-table no-hover contains-links">
            <thead>
              <tr>
                <th>Name</th>
                <th>Date</th>
              </tr>
            </thead>
            <tbody>
              <tr><td></td><td></td></tr>
              { sortBy(this.state.webinars, "utc").map((webinar, index) => {
                return(
                  <tr key={ index }>
                    <td className="name-column">
                      <a href={ `/admin/webinars/${webinar.id}` }>
                        { webinar.name }
                      </a>
                    </td>
                    <td>
                      <a href={ `/admin/webinars/${webinar.id}` }>
                        { webinar.time }
                      </a>
                      <div className="x-button" onClick={ this.clickXButton.bind(this) } data-id={ webinar.id }></div>
                    </td>
                  </tr>
                );
              }) }
            </tbody>
          </table>
          <a className="green-outline-button small" onClick={ this.clickAddWebinar.bind(this) }>Add Webinar</a>
          <hr />
          { this.renderButton() }
        </div>
        <Modal isOpen={ this.state.webinarsModalOpen } onRequestClose={ this.closeModal.bind(this) } contentLabel="Modal" style={ SelectModalStyles }>
          <ModalSelect options={ this.state.otherWebinars } property={ "name" } func={ this.selectWebinar.bind(this) } />
        </Modal>
        <Modal isOpen={ this.state.deleteModalOpen } onRequestClose={ Admin.closeDeleteModal.bind(this) } contentLabel="Modal" style={ ConfirmDeleteModalStyles }>
          <ConfirmDelete entityName={ "event series" } skipCheck={ true } closeModal={ Admin.closeDeleteModal.bind(this) } closeAndFetch={ Admin.closeDeleteModalAndFetch.bind(this) } delete={ Admin.deleteEntity.bind(this) } />
        </Modal>
      </div>
    );
  }

  renderButton() {
    if (this.state.changesToSave) {
      var buttonText = "Save";
    } else {
      var buttonText = this.state.justSaved ? "Saved" : "No Changes";
    }
    return(
      <div>
        <a id="update-button" className={ "orange-button standard-button-width btn" + Admin.renderDisabledButtonClass(this.state.fetching || !this.state.changesToSave) } onClick={ this.clickSave.bind(this) }>
          { buttonText }
        </a>
        <a id="delete" className={ "delete-button " + Admin.renderDisabledButtonClass(this.state.fetching) } onClick={ Admin.changeState.bind(this, "deleteModalOpen", true) }>
          Delete
        </a>
      </div>
    );
  }

  componentDidUpdate() {
    $(".match-height-layout").matchHeight();
  }
}

const mapStateToProps = (reducers, props) => {
  return reducers.standardReducer;
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntity, createEntity, updateEntity, deleteEntity }, dispatch);
}

export default connect(mapStateToProps, mapDispatchToProps)(withExitPrompt(EventSeriesDetails));
