import React from 'react'
import Modal from 'react-modal'
import { connect } from 'react-redux'
import { bindActionCreators } from 'redux'
import isEqual from 'lodash/isEqual'
import cloneDeep from 'lodash/cloneDeep'
import sortBy from 'lodash/sortBy'
import Admin from './../admin'
import Tools from './../../app/assets/javascripts/teachable/tools'
import PagePartial from './page-partial'
import PagePartialsNew from './page-partials-new'
import ReusableImages from './reusable-images'
import NewEntity from './new-entity'
import ConfirmDelete from './confirm-delete'
import { PartialsModalStyles, CloningModalStyles, ReusableImagesModalStyles, ConfirmDeleteModalStyles } from './helpers/modal-styles'
import { fetchEntity, createEntity, updateEntity, deleteEntity, rearrangeEntities } from '../actions/index'
import { withExitPrompt } from './util/with-exit-prompt'

interface Page {
  active: boolean;
  canonicalUrl: string;
  createdBy: string;
  description: string;
  displayExitIntentModal: boolean;
  googleOptimize: string;
  id: number;
  indexPage: boolean;
  name: string;
  notes: string;
  titleText: string;
  url: string;
  theme: string;
}

interface Partial extends ComponentState {
  fieldsCount: number;
  id: number;
  visible: boolean;
}

interface PagePartialType {
  id: number;
  order: number;
}

interface PageDetailsProps {
  createEntity: any;
  deleteEntity: any;
  errors: string[];
  fetchEntity: any;
  headerTypes?: string[];
  page: Page;
  pagePartials: any[];
  partials: Partial[];
  rearrangeEntities: any;
  updateEntity: any;
}

interface PageDetailsState extends ComponentState {
  cloningModalOpen: boolean;
  deleteModalOpen: boolean;
  fetchingPagePartials: boolean;
  handleAddPartial: boolean;
  headerTypes: string[];
  page: Page;
  pagePartials: PagePartialType[];
  pageSaved: Page;
  partialModalOpen: boolean;
  partials: Partial[];
  receiveReusableImageFunction: any;
  reusableImagesModalOpen: boolean;
}

class PageDetails extends React.Component<PageDetailsProps, PageDetailsState> {

  constructor(props) {
    super(props);

    this.state = {
      changesToSave: false,
      cloningModalOpen: false,
      deleteModalOpen: false,
      errors: [],
      fetching: true,
      fetchingPagePartials: false,
      handleAddPartial: null,
      justSaved: false,
      page: {} as Page,
      pagePartials: [],
      pageSaved: {} as Page,
      partials: [],
      headerTypes: [],
      partialModalOpen: false,
      reusableImagesModalOpen: false,
      receiveReusableImageFunction: null
    };
  }

  componentDidMount() {
    let pathDirectories = window.location.pathname.split('/');
    this.props.fetchEntity({
      id: pathDirectories[pathDirectories.length - 1],
      directory: "pages",
      entityName: "page"
    }).then(() => {
      this.setState({
        fetching: false,
        page: this.props.page,
        pageSaved: cloneDeep(this.props.page),
        partials: this.props.partials,
        pagePartials: this.props.pagePartials,
        headerTypes: this.props.headerTypes,
        changesToSave: false
      }, () => {
        Admin.setUpNiceSelect({ selector: 'select', func: Admin.changeField.bind(this, this.changeFieldArgs()) });
        this.setUpDragAndDrop.call(this);
      });
    });
  }

  deletePagePartial(id) {
    this.setState({
      fetchingPagePartials: true
    });
    this.props.deleteEntity({
      directory: 'page_partials',
      id,
      callback: (response) => {
        this.setState({
          fetchingPagePartials: false,
          pagePartials: response.pagePartials
        }, this.setUpDragAndDrop.bind(this));
      }
    });
  }

  setUpDragAndDrop() {
    // set up draggable elements
    var draggableSelector = '.page-partial';
    $(draggableSelector).draggable({
      cursor: '-webkit-grabbing',
      handle: '.handle',
      helper() { return '<div></div>'; },
      stop: this.dragEndHandler
    });
    // set up droppable elements
    var droppableSelector = '.drop-zone';
    $(droppableSelector).droppable({
      accept: Admin.canIDrop,
      tolerance: 'pointer',
      over: this.dragOverHandler,
      out: this.dragOutHandler,
      drop: this.dropHandler.bind(this)
    });
  }

  dragEndHandler(e) {
    $('.handle').removeClass('grabbing');
    $('.dragging-this').removeClass('dragging-this');
    $('.drop-zone-highlight').removeClass('drop-zone-highlight');
  }

  dragOverHandler(e) {
    e.target.classList.add('drop-zone-highlight');
  }

  dragOutHandler(e) {
    e.target.classList.remove('drop-zone-highlight');
  }

  dropHandler(e, ui) {
    var draggedIndex = ui.draggable[0].dataset.index;
    var dropZoneIndex = e.target.dataset.index;
    var currentOrder = {};
    this.state.pagePartials.forEach((pagePartial) => {
      currentOrder[pagePartial.order] = pagePartial.id;
    });
    var newOrder = Tools.rearrangeFields(currentOrder, draggedIndex, dropZoneIndex);
    this.setState({
      fetchingPagePartials: true
    });
    this.props.rearrangeEntities({
      directory: 'page_partials',
      newOrder,
      additionalData: {
        pageId: this.state.page.id
      }
    }).then(() => {
      this.setState({
        fetchingPagePartials: false,
        pagePartials: this.props.pagePartials
      }, this.setUpDragAndDrop.bind(this));
    });
  }

  clickSave() {
    this.setState({
      fetching: true,
      justSaved: true
    }, () => {
      this.props.updateEntity({
        id: this.state.page.id,
        directory: "pages",
        entityName: "page",
        entity: this.state.page
      }).then(() => {
        this.setState({
          fetching: false,
          page: this.props.page,
          pageSaved: cloneDeep(this.props.page),
          changesToSave: false
        });
      }, () => {
        this.setState({
          fetching: false,
          errors: this.props.errors
        });
      });
    });
  }

  clickAddPartial() {
    this.setState({
      partialModalOpen: true,
      errors: []
    });
  }

  addPartial(partialId) {
    const pageId = window.location.pathname.split("/")[3];
    this.setState({
      fetchingPagePartials: true
    });
    this.props.createEntity({
      directory: 'page_partials',
      entityName: 'pagePartial',
      entity: {
        page_id: pageId,
        partial_id: partialId
      }
    }).then(() => {
      this.setState({
        fetchingPagePartials: false,
        pagePartials: this.props.pagePartials
      }, this.setUpDragAndDrop.bind(this));
    });
  }

  clickCopy() {
    this.setState({
      cloningModalOpen: true,
      errors: []
    });
  }

  closeModal() {
    this.setState({
      partialModalOpen: false,
      cloningModalOpen: false,
      reusableImagesModalOpen: false,
      deleteModalOpen: false
    });
  }

  checkForChanges() {
    return !isEqual(this.state.page, this.state.pageSaved);
  }

  changeFieldArgs() {
    return {
      thing: "page",
      errorsArray: this.state.errors,
      changesFunction: () => this.checkForChanges()
    }
  }

  hasOpenModal() {
    return this.state.partialModalOpen || this.state.cloningModalOpen;
  }

  openReusableImagesModal(receiveReusableImageFunction) {
    this.setState({
      reusableImagesModalOpen: true,
      receiveReusableImageFunction: receiveReusableImageFunction
    });
  }

  render() {
    return (
      <div id="page-details" className="component details-component">
        <h1>Page details</h1>
        <p className="created-by">Created by: {this.state.page.createdBy || ""}</p>
        <div className="white-box">
          {Admin.renderSpinner(this.state.fetching)}
          {Admin.renderGrayedOut(this.state.fetching)}
          <div className="page-info">
            <div className="row">
              {Admin.renderField.bind(this)({ columnWidth: 4, entity: "page", property: "name" })}
              {Admin.renderField.bind(this)({ columnWidth: 4, entity: "page", property: "url" })}
              {Admin.renderField.bind(this)({ columnWidth: 4, entity: "page", property: "canonicalUrl" })}
            </div>
            <div className="row">
              {Admin.renderField.bind(this)({ columnWidth: 8, entity: "page", property: "titleText" })}
              {Admin.renderDropDown.bind(this)({ columnWidth: 2, entity: "page", property: "active", boolean: true, maxOptions: 2 })}
              {Admin.renderDropDown.bind(this)({ columnWidth: 2, entity: "page", property: "indexPage", boolean: true, maxOptions: 2 })}
            </div>
            <div className="row">
              {Admin.renderField.bind(this)({ columnWidth: 8, entity: "page", property: "description", columnHeader: "Meta Description Tag" })}
              {Admin.renderDropDown.bind(this)({ columnWidth: 2, entity: "page", property: "displayExitIntentModal", columnHeader: "Exit Intent Modal?", boolean: true, maxOptions: 2 })}
              {Admin.renderDropDown.bind(this)({ columnWidth: 2, entity: "page", property: "googleOptimize", columnHeader: "Google Optimize?", boolean: true, maxOptions: 2 })}
            </div>
            <div className="row">
              {Admin.renderTextBox.bind(this)({ columnWidth: 6, entity: "page", property: "notes", columnHeader: "Internal Notes", rows: 5 })}
              {Admin.renderImageField.bind(this)({ columnWidth: 2, entity: "page", property: "ogImageId", imageUrlProperty: "ogImageUrl", columnHeader: "Social Media Thumbnail", createImageRecord: true })}
              {Admin.renderDropDown.bind(this)({ columnWidth: 2, entity: "page", property: "theme", columnHeader: "Theme", options: [["default", "Default"], ["rebrand", "Rebrand"]], optionValue: 0, optionLabel: 1 })}
              {Admin.renderBooleanDropDown.bind(this)({ columnWidth: 2, entity: "page", property: "liveChat" })}
            </div>
            {this.renderButtons()}
          </div>
          {this.renderPagePartials()}
        </div>
        <Modal isOpen={this.state.partialModalOpen} onRequestClose={this.closeModal.bind(this)} contentLabel="Modal" style={PartialsModalStyles}>
          <PagePartialsNew addPartial={this.addPartial.bind(this)} closeModal={this.closeModal.bind(this)} partials={this.state.partials} />
        </Modal>
        <Modal isOpen={this.state.cloningModalOpen} onRequestClose={this.closeModal.bind(this)} contentLabel="Modal" style={CloningModalStyles}>
          <NewEntity entityName="page" initialEntity={{ name: "", url: "/", titleText: "", creatorId: Admin.currentUser.id, cloneId: this.state.page.id }} redirect={true} />
        </Modal>
        <Modal isOpen={this.state.reusableImagesModalOpen} onRequestClose={this.closeModal.bind(this)} contentLabel="Modal" style={ReusableImagesModalStyles}>
          <ReusableImages receiveReusableImageFunction={this.state.receiveReusableImageFunction} closeModal={this.closeModal.bind(this)} />
        </Modal>
        <Modal isOpen={this.state.deleteModalOpen} onRequestClose={this.closeModal.bind(this)} contentLabel="Modal" style={ConfirmDeleteModalStyles}>
          <ConfirmDelete entityName={'page'} skipCheck={true} closeModal={this.closeModal.bind(this)} closeAndFetch={Admin.closeDeleteModalAndFetch.bind(this)} delete={Admin.clickDelete.bind(this)} />
        </Modal>
      </div>
    );
  }

  renderButtons() {
    let saveButtonText;
    if (this.state.changesToSave) {
      saveButtonText = 'Save';
    } else {
      saveButtonText = this.state.justSaved ? 'Saved' : 'No Changes';
    }
    return (
      <div className="buttons-container">
        <a className={"orange-button standard-button-width btn" + Admin.renderDisabledButtonClass(this.state.fetching || !this.state.changesToSave)} onClick={this.clickSave.bind(this)}>
          {saveButtonText}
        </a>
        {this.renderDeleteButton()}
        <a id="clone-button" className={"light-orange-button" + Admin.renderDisabledButtonClass(this.state.fetching)} onClick={this.clickCopy.bind(this)}>
          Copy
        </a>
        <a id="preview-button" className={"light-orange-button" + Admin.renderDisabledButtonClass(this.state.fetching)} href={window.location.pathname + "/preview"} target="_blank">
          Preview
        </a>
      </div>
    );
  }

  renderDeleteButton() {
    if (+$('div#access').html() >= 2) {
      return (
        <a className={"delete-button" + Admin.renderDisabledButtonClass(this.state.fetching)} onClick={Admin.changeState.bind(this, "deleteModalOpen", true)}>
          Delete
        </a>
      );
    } else {
      return null;
    }
  }

  renderPagePartials() {
    let pagePartials = sortBy(this.state.pagePartials, "order");
    return (
      <div className="page-partials-index">
        {Admin.renderSpinner(this.state.fetchingPagePartials)}
        {this.renderPagePartialsIndexGrayedOut(this.state.fetchingPagePartials)}
        <h2 id="partials-header">Partials</h2>
        <div id="top-drop-zone" className="drop-zone" data-index="-1"></div>
        {pagePartials.map((partial, i) => this.renderPagePartial(partial, i, this.openReusableImagesModal))}
        <a className={"green-outline-button" + Admin.renderDisabledButtonClass(this.state.fetching)} onClick={() => this.clickAddPartial()}>
          Add Partial
        </a>
      </div>
    );
  }

  renderPagePartial(pagePartial, dragIndex, openReusableImagesModal) {
    return (
      <div key={pagePartial.id}>
        <PagePartial
          passedPagePartial={pagePartial}
          index={dragIndex}
          deletePagePartial={this.deletePagePartial.bind(this)}
          openReusableImagesModal={openReusableImagesModal.bind(this)}
          partials={this.state.partials}
          headerTypes={this.state.headerTypes}
        />
        <div className="drop-zone" data-index={dragIndex}></div>
      </div>
    );
  }

  renderPagePartialsIndexGrayedOut(fetching) {
    if (fetching) {
      return <div className="page-partials-index-grayed-out"></div>;
    } else {
      return null;
    }
  }

  componentDidUpdate() {
    $('.match-height-layout').matchHeight();
    Admin.resetNiceSelect({
      selector: '.page-info select',
      func: Admin.changeField.bind(this, this.changeFieldArgs())
    });
  }
}

const mapStateToProps = (reducers, props) => {
  return reducers.standardReducer;
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntity, createEntity, updateEntity, deleteEntity, rearrangeEntities }, dispatch);
}

export default connect(mapStateToProps, mapDispatchToProps)(withExitPrompt(PageDetails));
