import React from 'react'
import { connect } from 'react-redux'
import { bindActionCreators } from 'redux'
import Modal from 'react-modal'
import cloneDeep from 'lodash/cloneDeep'
import sortBy from 'lodash/sortBy'
import ModalSelect from './modal-select'
import Admin from './../admin'
import Tools from './../../app/assets/javascripts/teachable/tools'
import { fetchEntities, createEntity, updateEntity, deleteEntity, rearrangeEntities } from '../actions/index'
import { SelectModalStyles } from './helpers/modal-styles'
import { withExitPrompt } from './util/with-exit-prompt'

interface PlanIndexItemProps {
  createEntity: any;
  deleteEntity: any;
  errors: string[];
  features: any;
  fetchEntities: any;
  plan: any;
  planFeatures: any;
  rearrangeEntities: any;
  tooltips: any[];
  updateEntity: any;
}

interface PlanIndexItemState extends ComponentState {
  changesToSave: boolean;
  errors: string[];
  features: any[];
  modalOpen: boolean;
  plan: any;
  planSaved: any;
  planFeatures: any[];
  tooltips: any[];
}

class PlanIndexItem extends React.Component<PlanIndexItemProps, PlanIndexItemState> {

  constructor(props) {
    super(props);
    this.state = {
      fetching: true,
      changesToSave: false,
      justSaved: false,
      modalOpen: false,
      plan: this.props.plan,
      planSaved: cloneDeep(this.props.plan),
      planFeatures: [],
      features: [],
      tooltips: this.props.tooltips,
      errors: []
    };
  }

  componentDidMount() {
    $('#plan-' + this.state.plan.id + ' .type-select').niceSelect().on('change', Admin.changeField.bind(this, this.changeFieldArgs()));
    this.props.fetchEntities(`plan_features?plan_id=${this.state.plan.id}`).then(() => {
      this.setState({
        fetching: false,
        planFeatures: this.props.planFeatures,
        features: this.props.features
      }, () => {
        let draggableSelector = '#plan-' + this.props.plan.id + ' .plan-features-index-item';
        $(draggableSelector).draggable({
          cursor: '-webkit-grabbing',
          handle: '.handle',
          helper: () => { return '<div></div>'; },
          stop: this.dragEndHandler.bind(this)
        });
        let droppableSelector = '#plan-' + this.props.plan.id + ' .drop-zone';
        $(droppableSelector).droppable({
          accept: Admin.canIDrop,
          tolerance: 'pointer',
          over: this.dragOverHandler.bind(this),
          out: this.dragOutHandler.bind(this),
          drop: this.dropHandler.bind(this)
        });
        Admin.resetNiceSelect({
          selector: '#plan-' + this.state.plan.id + ' .tooltip-select',
          func: this.changeTooltip.bind(this)
        });
      });
    });
  }

  mouseDownHandler(e) {
    $('.handle').addClass('grabbing');
    e.target.parentElement.classList.add('dragging-this');
  }

  mouseUpHandler(e) {
    $('.handle').removeClass('grabbing');
    e.target.parentElement.classList.remove('dragging-this');
  }

  dragEndHandler(e) {
    $('.handle').removeClass('grabbing');
    $('.dragging-this').removeClass('dragging-this');
    $('.drop-zone-highlight').removeClass('drop-zone-highlight');
  }

  dragOverHandler(e) {
    e.target.classList.add('drop-zone-highlight');
  }

  dragOutHandler(e) {
    e.target.classList.remove('drop-zone-highlight');
  }

  dropHandler(e, ui) {
    let draggedIndex = ui.draggable.data().index;
    let dropZoneIndex = e.target.dataset.index;
    let currentOrder = {};
    this.state.planFeatures.forEach((planFeature) => {
      currentOrder[planFeature.order] = planFeature.id;
    });
    let newOrder = Tools.rearrangeFields(currentOrder, draggedIndex, dropZoneIndex);
    this.setState({
      fetching: true
    });
    this.props.rearrangeEntities({
      directory: "plan_features",
      newOrder,
      additionalData: {
        planId: ui.draggable.data().plan
      }
    }).then(() => {
      this.setState({
        fetching: false,
        planFeatures: this.props.planFeatures
      })
    });
  }

  checkForChanges() {
    let keys = Object.keys(this.state.plan);
    for (let i = 0; i < keys.length; i++) {
      if (this.state.plan[keys[i]] !== this.state.planSaved[keys[i]]) {
        return true;
      }
    }
    return false;
  }

  changeFieldArgs() {
    return {
      thing: "plan",
      errorsArray: this.state.errors,
      changesFunction: this.checkForChanges.bind(this)
    }
  }

  clickSave() {
    this.setState({
      fetching: true
    });
    this.props.updateEntity({
      id: this.state.plan.id,
      directory: "plans",
      entityName: "plan",
      entity: this.state.plan
    }).then(() => {
      this.setState({
        fetching: false,
        plan: this.props.plan,
        planSaved: cloneDeep(this.props.plan),
        justSaved: true,
        changesToSave: false
      });
    }, () => {
      this.setState({
        errors: this.props.errors
      });
    });
  }

  clickTrash(e) {
    this.setState({
      fetching: true
    });
    this.props.deleteEntity({
      directory: "plan_features",
      id: e.target.parentElement.dataset.id,
      callback: (response) => {
        this.setState({
          fetching: false,
          planFeatures: response.planFeatures
        });
      }
    });
  }

  clickAddFeature() {
    this.setState({
      modalOpen: true
    });
  }

  selectFeature(e) {
    this.setState({
      modalOpen: false
    });
    let featureId = e.target.dataset.id;
    this.props.createEntity({
      directory: "plan_features",
      entityName: "PlanFeature",
      entity: {
        plan_id: this.state.plan.id,
        feature_id: featureId
      }
    }).then(() => {
      this.setState({
        fetching: false,
        planFeatures: this.props.planFeatures
      }, () => {
        Admin.resetNiceSelect({
          selector: '#plan-' + this.state.plan.id + ' .tooltip-select',
          func: this.changeTooltip.bind(this)
        });
      });
    });
  }

  changeTooltip(e) {
    let tooltipId = e.target.value;
    this.setState({
      fetching: true
    });
    let planFeatureId = parseInt(e.target.parentElement.dataset.id, 10);
    const planFeature = this.state.planFeatures.find(({ id: pid }) => pid === planFeatureId);
    planFeature.tooltipId = tooltipId;
    this.props.updateEntity({
      id: planFeatureId,
      directory: "plan_features",
      entityName: "PlanFeature",
      entity: planFeature
    }).then(() => {
      this.setState({
        fetching: false,
        plan: this.props.plan,
        planSaved: cloneDeep(this.props.plan),
        changesToSave: false
      });
    });
  }

  closeModal() {
    this.setState({ modalOpen: false });
  }

  unusedFeatures() {
    let usedFeatureIds = this.state.planFeatures.map((planFeature) => {
      return planFeature.featureId;
    });
    let unusedFeatures = [];
    this.state.features.forEach((feature) => {
      if (usedFeatureIds.indexOf(feature.id) === -1) {
        unusedFeatures.push(feature);
      }
    })
    return unusedFeatures;
  }

  render() {
    return(
      <div id={ "plan-" + this.state.plan.id } className="plan">
        { Admin.renderSpinner(this.state.fetching) }
        { Admin.renderGrayedOut(this.state.fetching) }
        { this.renderPlanInfo() }
        { this.renderButtons() }
        { this.renderPlanFeatures() }
        <Modal isOpen={ this.state.modalOpen } onRequestClose={ this.closeModal.bind(this) } contentLabel="Modal" style={ SelectModalStyles }>
          <ModalSelect options={ this.unusedFeatures() } property="text" func={ this.selectFeature.bind(this) } />
        </Modal>
      </div>
    );
  }

  renderPlanInfo() {
    if (this.props.plan.id !== 1) {
      return(
        <div>
          <div className="row">
            <div className="col-xs-2">
              <h2>Name</h2>
              <input className={ Admin.errorClass(this.state.errors, Admin.errors.name) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.plan.name } data-field="name" />
              { Admin.renderFieldError(this.state.errors, Admin.errors.name) }
            </div>
            <div className="col-xs-3">
              <h2>Tagline</h2>
              <input className={ Admin.errorClass(this.state.errors, Admin.errors.tagline) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.plan.tagline } data-field="tagline" />
              { Admin.renderFieldError(this.state.errors, Admin.errors.tagline) }
            </div>
            <div className="col-xs-3">
              <h2>Type</h2>
              <select className={ "type-select" } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ Tools.convertBooleanToTFString(this.state.plan.annualIncluded) } data-field="annualIncluded">
                <option value="t">Monthly/Annual</option>
                <option value="f">Monthly Only</option>
              </select>
            </div>
            <div className="col-xs-2">
              <h2>Monthly Price</h2>
              <input className={ Admin.errorClass(this.state.errors, Admin.errors.monthlyPrice) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.plan.monthlyPrice } data-field="monthlyPrice" />
              { Admin.renderFieldError(this.state.errors, Admin.errors.monthlyPrice) }
            </div>
            <div className={ "col-xs-2" + (this.state.plan.annualIncluded ? "" : " hidden") }>
              <h2>Annual Price</h2>
              <input className={ Admin.errorClass(this.state.errors, Admin.errors.annualPrice) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.plan.annualPrice === 0 ? "0" : this.state.plan.annualPrice || "" } data-field="annualPrice" />
              { Admin.renderFieldError(this.state.errors, Admin.errors.annualPrice) }
            </div>
          </div>
          <div className="row">
            <div className="col-xs-6">
              <h2>Includes Text</h2>
              <input className={ Admin.errorClass(this.state.errors, Admin.errors.includesText) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.plan.includesText } data-field="includesText" />
              { Admin.renderFieldError(this.state.errors, Admin.errors.includesText) }
            </div>
            { this.renderFedoraIDFields() }
          </div>
        </div>
      );
    } else {
      return(
        <h2 id="all-plans-header">All Plans</h2>
      );
    }
  }

  renderFedoraIDFields() {
    if (+Admin.currentUser.access >= 2) {
      return(
        <div>
          <div className="col-xs-3">
            <h2>Fedora Monthly Plan ID</h2>
            <input className={ Admin.errorClass(this.state.errors, Admin.errors.appMonthlyPlanId) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.plan.appMonthlyPlanId } data-field="appMonthlyPlanId" />
            { Admin.renderFieldError(this.state.errors, Admin.errors.appMonthlyPlanId) }
          </div>
          <div className={ "col-xs-3" + (this.state.plan.annualIncluded ? "" : " hidden") }>
            <h2>Fedora Annual Plan ID</h2>
            <input className={ Admin.errorClass(this.state.errors, Admin.errors.appAnnualPlanId) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.plan.appAnnualPlanId } data-field="appAnnualPlanId" />
            { Admin.renderFieldError(this.state.errors, Admin.errors.appAnnualPlanId) }
          </div>
        </div>
      );
    } else {
      return null;
    }
  }

  renderButtons() {
    if (this.props.plan.id !== 1) {
      return(
        <a className={ "btn orange-button standard-button-width" + Admin.renderDisabledButtonClass(this.state.fetching || !this.state.changesToSave) } onClick={ this.clickSave.bind(this) }>
          { Admin.saveButtonText.call(this) }
        </a>
      );
    } else {
      return null;
    }
  }

  renderPlanFeatures() {
    return(
      <div className="plan-features-index">
        <h2>Features</h2>
        <div id={ this.state.plan.id + "-top-drop-zone" } className="drop-zone" data-index="-1"></div>
        { sortBy(this.state.planFeatures, "order").map((planFeature, index) => {
          const feature = this.state.features.find(({ id: fid }) => fid === planFeature.featureId);
          let text = feature ? feature.text : "(loading)";
          return(
            <div key={ index }>
              <div className="plan-features-index-item" data-id={ planFeature.id } data-index={ index } data-plan={ planFeature.planId }>
                { text }
                <img className="trash-can" src={ window.Images.trashCan } onClick={ this.clickTrash.bind(this) } />
                <img className="handle" src={ window.Images.handle } onMouseDown={ this.mouseDownHandler } onMouseUp={ this.mouseUpHandler } />
                <select className="tooltip-select" onChange={ this.changeTooltip.bind(this) } value={ planFeature.tooltipId === null ? "none" : planFeature.tooltipId } data-field="tooltipId">
                  <option value="none">No Tooltip</option>
                  { this.state.tooltips.map((tooltip, index) => {
                    return(
                      <option key={ index } value={ tooltip.id }>
                        { tooltip.text.slice(0, 12) + "..." }
                      </option>
                    )
                  }) }
                </select>
              </div>
              <div className="drop-zone" data-index={ index }></div>
            </div>
          );
        }) }
        <a className={ "green-outline-button" + Admin.renderDisabledButtonClass(this.state.fetching) } onClick={ this.clickAddFeature.bind(this) }>
          Add Feature
        </a>
      </div>
    );
  }

  componentDidUpdate() {
    $('.match-height-layout').matchHeight();
  }
}

const mapStateToProps = (reducers, props) => {
  let obj = {};
  obj = {
    planFeatures: reducers.standardReducer.planFeatures,
    features: reducers.standardReducer.features,
    tooltips: reducers.standardReducer.tooltips
  };
  if (reducers.standardReducer.plan && reducers.standardReducer.plan.id === props.plan.id) {
    obj['plan'] = reducers.standardReducer.plan;
    obj['errors'] = reducers.standardReducer.errors;
  }
  return obj;
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntities, createEntity, updateEntity, deleteEntity, rearrangeEntities }, dispatch);
}

export default connect(mapStateToProps, mapDispatchToProps)(withExitPrompt(PlanIndexItem));
