import React from 'react'
import { connect } from 'react-redux'
import { bindActionCreators } from 'redux'
import Modal from 'react-modal'
import ModalSelect from './modal-select'
import isEqual from 'lodash/isEqual'
import cloneDeep from 'lodash/cloneDeep'
import pull from 'lodash/pull'
import Admin from './../admin'
import Tools from './../../app/assets/javascripts/teachable/tools'
import ConfirmDelete from './confirm-delete'
import { SelectModalStyles, ConfirmDeleteModalStyles } from './helpers/modal-styles'
import { fetchEntity, updateEntity, deleteEntity } from '../actions/index'
import { withExitPrompt } from './util/with-exit-prompt'

interface ScheduledTaskDetailsProps {
  banners: any[];
  blogPosts: any[];
  errors: string[];
  fetchEntity: any;
  pages: any[];
  redirects: any[];
  scheduledTask: any;
  updateEntity: any;
}

interface ScheduledTaskDetailsState extends ComponentState {
  banners?: any[];
  blogPosts?: any[];
  deleteModalOpen: boolean;
  entitiesModalOpen: boolean;
  pages?: any[];
  redirects?: any[];
  scheduledTask: any;
  scheduledTaskSaved: any;
}

const properProperty = {
  "Banner": "name",
  "BlogPost": "title",
  "Page": "name",
  "Redirect": "url",
};

class ScheduledTaskDetails extends React.Component<ScheduledTaskDetailsProps, ScheduledTaskDetailsState> {

  constructor(props) {
    super(props);
    this.state = {
      fetching: true,
      changesToSave: false,
      justSaved: false,
      scheduledTask: {},
      scheduledTaskSaved: {},
      errors: [],
      deleteModalOpen: false,
      entitiesModalOpen: false
    };
  }

  componentDidMount() {
    let pathDirectories = window.location.pathname.split("/");
    this.props.fetchEntity({
      id: pathDirectories[pathDirectories.length - 1],
      directory: "scheduled_tasks",
      entityName: "scheduledTask"
    }).then(() => {
      this.setState({
        fetching: false,
        scheduledTask: this.props.scheduledTask,
        scheduledTaskSaved: cloneDeep(this.props.scheduledTask),
        pages: this.props.pages,
        banners: this.props.banners,
        redirects: this.props.redirects,
        blogPosts: this.props.blogPosts,
        changesToSave: false
      }, () => {
        Admin.setUpNiceSelect({ selector: 'select', func: Admin.changeField.bind(this, this.changeFieldArgs()) });
      });
    });
  }

  clickSave() {
    this.setState({
      fetching: true,
      justSaved: true
    });
    this.props.updateEntity({
      id: this.state.scheduledTask.id,
      directory: "scheduled_tasks",
      entityName: "scheduledTask",
      entity: this.state.scheduledTask
    }).then(() => {
      this.setState({
        fetching: false,
        scheduledTask: this.props.scheduledTask,
        scheduledTaskSaved: cloneDeep(this.props.scheduledTask),
        changesToSave: false
      });
    }, () => {
      this.setState({
        fetching: false,
        errors: this.props.errors
      });
    });
  }

  checkForChanges() {
    return !isEqual(this.state.scheduledTask, this.state.scheduledTaskSaved);
  }

  changeFieldArgs() {
    return {
      thing: "scheduledTask",
      errorsArray: this.state.errors,
      changesFunction: this.checkForChanges.bind(this),
      beforeSave: (newThing, key, value) => {
        if (key == "schedulableType") {
          newThing["schedulableId"] = "";
          Admin.removeFieldError(this.state.errors, "schedulableId");
        }
      }
    }
  }

  schedulableName() {
    var id = this.state.scheduledTask.schedulableId;
    if (id) {
      switch (this.state.scheduledTask.schedulableType) {
        case "Banner":
          return this.state.banners.find(({ id: bannerId }) => id === bannerId).name;
        case "BlogPost":
          return this.state.blogPosts.find(({ id: postId }) => id === postId).title;
        case "Page":
          return this.state.pages.find(({ id: pageId }) => id === pageId).name;
        case "Redirect":
          const redirect = this.state.redirects.find(({ id: redirectId }) => id === redirectId);
          return `${redirect.url} → ${redirect.redirect}`;
        default:
          return null;
      }
    } else {
      return "(None)";
    }
  }

  properOptions() {
    switch (this.state.scheduledTask.schedulableType) {
      case "Banner":
        return this.state.banners;
        break;
      case "BlogPost":
        return this.state.blogPosts;
        break;
      case "Page":
        return this.state.pages;
        break;
      case "Redirect":
        return this.state.redirects;
        break;
      default:
        return null;
    }
  }

  closeModal() {
    this.setState({
      entitiesModalOpen: false
    });
  }

  selectEntity(e) {
    let scheduledTask = this.state.scheduledTask;
    scheduledTask.schedulableId = e.target.dataset.id;
    let errors = this.state.errors;
    pull(errors, "Schedulable can't be blank");
    this.setState({
      scheduledTask,
      errors,
      entitiesModalOpen: false
    }, () => {
      let changesToSave = this.changeFieldArgs().changesFunction.call();
      this.setState({
        changesToSave: changesToSave
      });
    });
  }

  render() {
    return(
      <div id="scheduled-task-details" className="component details-component">
        <h1>Scheduled Task details</h1>
        <div className="white-box">
          { Admin.renderSpinner(this.state.fetching) }
          { Admin.renderGrayedOut(this.state.fetching) }
          <div className="row">
            <div className="col-xs-3">
              <h2>Time</h2>
              <input className={ Admin.errorClass(this.state.errors, Admin.errors.time) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.scheduledTask.time || "" } data-field="time" placeholder="11/5/19 2:00 pm" />
              { Admin.renderFieldError(this.state.errors, Admin.errors.time) }
            </div>
            <div className="col-xs-2">
              <h2>Type</h2>
              <div className="clearfix">
                <select onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.scheduledTask.schedulableType } data-field="schedulableType">
                  <option value="Banner">Banner</option>
                  <option value="BlogPost">Blog Post</option>
                  <option value="Page">Page</option>
                  <option value="Redirect">Redirect</option>
                </select>
              </div>
              { Admin.renderFieldError(this.state.errors, []) }
            </div>
            <div className="col-xs-5">
              <h2>{ this.state.scheduledTask.id ? Tools.unCamelCase(this.state.scheduledTask.schedulableType) : "(Loading)" }</h2>
              <input className={ Admin.errorClass(this.state.errors, Admin.errors.schedulableId) } readOnly={ true } value={ this.schedulableName() || "" } />
              { this.renderLink() }
              { Admin.renderFieldError(this.state.errors, []) }
            </div>
            <div className="col-xs-2">
              <h2>Action</h2>
              <div className="clearfix">
                <select onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.scheduledTask.action || "" } data-field="action">
                  <option value="publish">Publish</option>
                  <option value="unpublish">Unpublish</option>
                </select>
              </div>
              { Admin.renderFieldError(this.state.errors, []) }
            </div>
          </div>
          { this.renderButtons() }
        </div>
        <Modal isOpen={ this.state.entitiesModalOpen } onRequestClose={ this.closeModal.bind(this) } contentLabel="Modal" style={ SelectModalStyles }>
          <ModalSelect options={ this.properOptions()  } property={ properProperty[this.state.scheduledTask.schedulableType] } func={ this.selectEntity.bind(this) } />
        </Modal>
        <Modal isOpen={ this.state.deleteModalOpen } onRequestClose={ Admin.closeDeleteModal.bind(this) } contentLabel="Modal" style={ ConfirmDeleteModalStyles }>
          <ConfirmDelete entityName="scheduled task" skipCheck={ true } closeModal={ Admin.closeDeleteModal.bind(this) } closeAndFetch={ Admin.closeDeleteModalAndFetch.bind(this) } delete={ Admin.deleteEntity.bind(this) } />
        </Modal>
      </div>
    );
  }

  renderLink() {
    if (this.state.scheduledTask.id) {
      return(
        <a className="link-below-field" onClick={ Admin.changeState.bind(this, "entitiesModalOpen", true) }>{ `Select ${Tools.unCamelCase(this.state.scheduledTask.schedulableType)}` }</a>
      );
    } else {
      return null;
    }
  }

  renderButtons() {
    if (this.state.scheduledTask.status === "pending") {
      if (this.state.changesToSave) {
        var buttonText = "Save";
      } else {
        var buttonText = this.state.justSaved ? "Saved" : "No Changes";
      }
      return(
        <div>
          <a id="update-button" className={ "orange-button standard-button-width btn" + Admin.renderDisabledButtonClass(this.state.fetching || !this.state.changesToSave) } onClick={ this.clickSave.bind(this) }>
            { buttonText }
          </a>
          <a id="delete" className={ "delete-button " + Admin.renderDisabledButtonClass(this.state.fetching) } onClick={ Admin.changeState.bind(this, "deleteModalOpen", true) }>
            Delete
          </a>
        </div>
      );
    } else {
      return null;
    }
  }

  componentDidUpdate() {
    $('.match-height-layout').matchHeight();
    Admin.resetNiceSelect({
      selector: 'select',
      func: Admin.changeField.bind(this, this.changeFieldArgs())
    });
  }
}

const mapStateToProps = (reducers, props) => {
  return reducers.standardReducer;
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntity, updateEntity, deleteEntity }, dispatch);
}

export default connect(mapStateToProps, mapDispatchToProps)(withExitPrompt(ScheduledTaskDetails));
