import React from 'react'
import { connect } from 'react-redux'
import { bindActionCreators } from 'redux'
import Modal from 'react-modal'
import ChangeCase from 'change-case'
import orderBy from 'lodash/orderBy'
import truncate from 'lodash/truncate'
import Admin from "./../admin"
import { fetchEntities } from '../actions/index'

interface StandardIndexProps {
  columns: any[];
  columnClasses?: string[];
  columnHeaders?: string[];
  columnLinks?: string[];
  columnTimestamps?: string[];
  columnWidths?: string[];
  directory?: string;
  ellipses?: string[];
  entities?: any[];
  entityName?: string;
  entityNamePlural?: string;
  fetchEntities: any;
  header?: string;
  modalDimensions: any;
  modalRows?: number;
  useNewPage: boolean;
}

interface StandardIndexState {
  entities: any[],
  directory?: string;
  fetching?: boolean;
  entityNamePlural?: string;
  newEntityModalOpen?: boolean;
  searchProperty?: string;
  searchText?: string;
}

class StandardIndex extends React.Component<StandardIndexProps, StandardIndexState> {

  entityNamePlural = this.props.entityNamePlural || `${this.props.entityName}s`
  directory = this.props.directory || ChangeCase.snakeCase(this.entityNamePlural)
  state = {
    fetching: true,
    searchProperty: this.props.columns[0],
    searchText: '',
    newEntityModalOpen: false,
    entities: []
  }

  componentDidMount() {
    this.props.fetchEntities(this.directory).then(() => {
      this.setState({
        fetching: false,
        entities: this.props.entities
      });
    });
  }

  clickNew() {
    if (this.props.useNewPage) {
      window.location.href = `/admin/${this.directory}/new`;
    } else {
      Admin.clickNew.call(this);
    }
  }

  updateIndex(entities) {
    this.setState({
      newEntityModalOpen: false,
      entities
    });
  }

  render() {
    const children = React.Children.map(
      this.props.children,
      (child) => {
        return React.cloneElement(child as React.ReactElement<any>, {
          entityName: this.props.entityName,
          entityNamePlural: this.entityNamePlural,
          callback: this.updateIndex.bind(this)
        });
      }
    );

    let filteredEntities = Admin.filterSearchText(this.state.entities, this.state.searchText, this.state.searchProperty);
    const timestampColumn = this.timestampColumn(this.state.searchProperty);

    return(
      <div className="component">
        <div className="clearfix">
          <h1 className="search-box-header">{ this.props.header || ChangeCase.titleCase(this.entityNamePlural) }</h1>
          <a className={ "btn orange-button float-button" + Admin.renderDisabledButtonClass(this.state.fetching) } onClick={ this.clickNew.bind(this) }>Add { ChangeCase.titleCase(this.props.entityName) }</a>
          <input className="search-box margin" onChange={ Admin.changeStateToTarget.bind(this, 'searchText') } value={ this.state.searchText } />
        </div>
        <div className="white-box">
          { Admin.renderSpinner(this.state.fetching) }
          { Admin.renderGrayedOut(this.state.fetching) }
          <table className="admin-table contains-links sortable">
            <thead>
              <tr>
                { this.props.columns.map((column, index) => {
                  return(
                    <th key={ index } style={ this.columnWidth(index) }>
                      <div className={ Admin.sortClass.bind(this)(column) } onClick={ Admin.changeState.bind(this, 'searchProperty', column) }>
                        { this.props.columnHeaders && this.props.columnHeaders[index] ? this.props.columnHeaders[index] : ChangeCase.titleCase(column) }
                      </div>
                    </th>
                  );
                })}
              </tr>
            </thead>
            <tbody>
              <tr>
                { this.props.columns.map((_, index) => {
                  return(
                    <td key={ index }></td>
                  );
                })}
              </tr>
              { orderBy(filteredEntities, [timestampColumn ? this.sortByTimestamp.bind(this, timestampColumn) : Admin.commonSort.bind(this)], [timestampColumn ? 'desc' : 'asc']).map((entity, index) => {
                return(
                  <tr key={ index }>
                    { this.props.columns.map((column, index) => {
                      return(
                        <td key={ index } className={ this.props.columnClasses ? this.props.columnClasses[index] : '' }>
                          <a href={ `${this.directory}/${entity.id}${this.props.columnLinks && this.props.columnLinks[index] ? this.props.columnLinks[index] : ''}` }>
                            { this.renderValue(entity[column], index) }
                          </a>
                        </td>
                      );
                    })}
                  </tr>
                );
              })}
            </tbody>
          </table>
        </div>
        <Modal isOpen={ this.state.newEntityModalOpen } onRequestClose={ Admin.closeModals.bind(this) } contentLabel="Modal" style={ Admin.newEntityModalStyles(this.props.modalDimensions, this.props.modalRows) }>
          { children }
        </Modal>
      </div>
    );
  }

  columnWidth(index) {
    if (this.props.columnWidths && this.props.columnWidths[index]) {
      return {
        width: +this.props.columnWidths[index]
      };
    } else {
      return null;
    }
  }

  timestampColumn(searchProperty) {
    if (this.props.columnTimestamps) {
      const columnIndex = this.props.columns.indexOf(searchProperty);
      const timestampColumn = this.props.columnTimestamps[columnIndex];
      return timestampColumn;
    } else {
      return null;
    }
  }

  renderValue(value, index) {
    if (this.props.ellipses && this.props.ellipses[index]) {
      return truncate(value, { length: parseInt(this.props.ellipses[index], 10) + 3 });
    } else {
      return value;
    }
  }

  sortByTimestamp(timestampColumn, entity) {
    return entity[timestampColumn];
  }

  componentDidUpdate() {
    Admin.matchColumnHeight();
  }
}

const mapStateToProps = (reducers, props) => {
  return {
    entities: reducers.standardReducer[ChangeCase.camelCase(props.entityNamePlural || `${props.entityName}s`)]
  };
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntities }, dispatch);
}

export default connect(mapStateToProps, mapDispatchToProps)(StandardIndex);
