import React from 'react'
import { connect } from 'react-redux'
import { bindActionCreators } from 'redux'
import Modal from 'react-modal'
import ChangeCase from 'change-case'
import padStart from 'lodash/padStart';
import isEqual from 'lodash/isEqual'
import cloneDeep from 'lodash/cloneDeep'
import pull from 'lodash/pull'
import ConfirmDelete from './confirm-delete'
import NewEntity from './new-entity'
import Admin from './../admin'
import Tools from './../../app/assets/javascripts/teachable/tools'
import { ConfirmDeleteModalStyles } from './helpers/modal-styles'
import { TrackingLink } from './tracking-link/tracking-link-types'
import { fetchEntity, fetchDataForNew, fetchDataForCopy, createEntity, updateEntity, deleteEntity } from '../actions/index'
import { withExitPrompt } from './util/with-exit-prompt'
import { Field } from './field';
import { LinkCheck } from './link-check';
import { Errors } from './errors';

interface TrackingLinkDetailsProps {
  id: number;
  createEntity: any;
  copy: boolean;
  errors: string[];
  fetchDataForCopy: any;
  fetchDataForNew: any;
  fetchEntity: any;
  new: boolean;
  plans: any[];
  trackingLink: TrackingLink;
  updateEntity: any;
  deleteEntity: any;
}

interface DropDownType {
  text: string;
}

interface TrackingLinkDetailsState extends ComponentState {
  audiences: DropDownType[];
  channels: DropDownType[];
  customSlugModalOpen: boolean;
  deleteModalOpen: boolean;
  paramModalOpen: boolean;
  placements: DropDownType[];
  plans: any[];
  statuses: DropDownType[];
  trackingLink?: TrackingLink;
  trackingLinkSaved?: TrackingLink;
}

const metadataKeys = [
  'audience',
  'campaign',
  'channel',
  'content',
  'medium',
  'placement',
  'source',
  'subcampaign',
  'subchannel'
];

const queryParameterKeys = [
  'couponCode',
  'planId',
  'upgradeSrc',
  'cx'
]

class TrackingLinkDetails extends React.Component<TrackingLinkDetailsProps, TrackingLinkDetailsState> {

  state: TrackingLinkDetailsState = {
    changesToSave: false,
    deleteModalOpen: false,
    paramModalOpen: false,
    customSlugModalOpen: false,
    errors: [],
    fetching: true,
    trackingLink: {
      slug: this.generateSlug(),
      description: '',
      status: 'published',
      forwardAllQueryParams: true,
      forwardQueryParamsWhitelist: []
    },
    trackingLinkSaved: {},
    plans: [],
    statuses: [
      { text: 'draft' },
      { text: 'published' },
      { text: 'hidden' }
    ],
    channels: [
      { text: 'podcast' },
      { text: 'radio' },
      { text: 'tv' },
      { text: 'out-of-home' },
      { text: 'affiliate' },
      { text: 'press' },
      { text: 'events' },
      { text: 'email' },
      { text: 'referral' },
      { text: 'teachable-properties' },
      { text: 'paid-social' },
      { text: 'paid-partners' },
      { text: 'paid-search-brand'},
      { text: 'paid-search-non-brand'},
      { text: 'display-programmatic' },
      { text: 'mixed-media' },
      { text: 'streaming-audio' },
      { text: 'streaming-video' },
      { text: 'direct-mail' },
      { text: 'unpaid-social' },
      { text: 'unpaid-partners' },
      { text: 'messaging-service' },
      { text: 'sales' },
      { text: 'customer-success' },
    ],
    audiences: [
      { text: 'general' },
      { text: 'personal-development' },
      { text: 'business-marketing' },
      { text: 'finance-Investing' },
      { text: 'software-technology' },
      { text: 'health-wellness' },
      { text: 'fitness-nutrition' },
      { text: 'family-parenting' },
      { text: 'fashion-beauty' },
      { text: 'arts-crafting' },
      { text: 'design-ux' },
      { text: 'music-instruments' },
      { text: 'photo-video' },
      { text: 'food-drink' },
      { text: 'acting-entertainment' },
      { text: 'academics-languages' },
      { text: 'spirituality-worship' }
    ],
    placements: [
      { text: 'paid' },
      { text: 'unpaid' }
    ]
  }

  generateSlug() {
    let now = new Date();
    return `${now.getFullYear()}${padStart(now.getMonth().toString() + 1, 2, '0')}${padStart(now.getDate().toString(), 2, '0')}${padStart(now.getHours().toString(), 2, '0')}${padStart(now.getMinutes().toString(), 2, '0')}${padStart(now.getSeconds().toString(), 2, '0')}`
  }

  componentDidMount() {
    if (this.props.copy) {
      this.props.fetchDataForCopy({
        directory: 'tracking_links',
        id: window.Common.params.id
      }).then(() => {
        let trackingLink = this.state.trackingLink;
        let fetchedTrackingLink = this.convertDatabaseRecordToFrontendTrackingLink.call(this);
        this.setState({
          fetching: false,
          trackingLink: Object.assign(fetchedTrackingLink, { slug: trackingLink.slug, status: trackingLink.status }),
          plans: this.convertPlans(this.props.plans)
        }, () => {
          Admin.setUpNiceSelect({
            selector: 'select',
            func: Admin.changeField.bind(this, this.changeFieldArgs())
          });
        });
      });
    } else if (this.props.new) {
      this.props.fetchDataForNew({
        directory: 'tracking_links'
      }).then(() => {
        this.setState({
          fetching: false,
          plans: this.convertPlans(this.props.plans)
        }, () => {
          Admin.setUpNiceSelect({
            selector: 'select',
            func: Admin.changeField.bind(this, this.changeFieldArgs())
          });
        });
      });
    } else {
      this.props.fetchEntity({
        directory: "tracking_links",
        id: this.props.id
      }).then(() => {
        let trackingLink = this.convertDatabaseRecordToFrontendTrackingLink.call(this);
        let plans = this.convertPlans(this.props.plans);
        let statuses = [
          { text: 'published' },
          { text: 'hidden' }
        ];
        if (trackingLink.status === 'draft') {
          statuses.push({ text: 'draft' });
        }
        this.setState({
          fetching: false,
          trackingLink,
          trackingLinkSaved: cloneDeep(trackingLink),
          plans,
          statuses
        }, () => {
          Admin.setUpNiceSelect({
            selector: 'select',
            func: Admin.changeField.bind(this, this.changeFieldArgs())
          });
        });
      });
    }
  }

  checkForChanges() {
    return !isEqual(this.state.trackingLink, this.state.trackingLinkSaved);
  }

  changeFieldArgs() {
    return {
      thing: "trackingLink",
      errorsArray: this.state.errors,
      changesFunction: this.checkForChanges.bind(this),
    }
  }

  convertPlans(propsPlans) {
    let result = [];
    propsPlans.forEach((plan) => {
      result.push({
        text: `${plan.name} - Monthly ($${Tools.formatCurrency(plan.monthlyPrice)})`,
        value: plan.appMonthlyPlanId
      });
      if (plan.annualIncluded) {
        result.push({
          text: `${plan.name} - Annual ($${Tools.formatCurrency(plan.annualPrice)})`,
          value: plan.appAnnualPlanId
        });
      }
    });
    return result;
  }

  convertDatabaseRecordToFrontendTrackingLink() {
    let record = this.props.trackingLink;
    let metadata = record.metadata;
    let result = {
      id: record.id,
      creatorName: record.creatorName,
      destinationUrl: record.destinationUrl,
      forwardAllQueryParams: record.forwardAllQueryParams,
      slug: record.slug,
      status: record.status,
      forwardQueryParamsWhitelist: record.forwardQueryParamsWhitelist
    };
    metadataKeys.forEach((key) => {
      result[ChangeCase.camelCase(key)] = metadata[key] || "";
    });
    let additionalQueryParams = record.additionalQueryParams;
    Object.keys(additionalQueryParams).forEach((key) => {
      result[ChangeCase.camelCase(key)] = additionalQueryParams[key].toString();
    });
    return result;
  }

  convertToNestedObject(keys, trackingLink) {
    let result = {};
    keys.forEach((key) => {
      if (trackingLink[key]) {
        result[ChangeCase.snakeCase(key)] = trackingLink[key];
      }
      delete trackingLink[key];
    });
    return result;
  }

  clickAddParam() {
    this.setState({
      paramModalOpen: true
    });
  }

  addParam(param) {
    let text = param.text;
    if (text) {
      let trackingLink = cloneDeep(this.state.trackingLink);
      trackingLink.forwardQueryParamsWhitelist.push(text);
      trackingLink.forwardQueryParamsWhitelist.sort();
      this.setState({
        trackingLink
      }, () => {
        this.setState({
          changesToSave: this.checkForChanges()
        });
      });
    }
    Admin.closeModals.call(this);
  }

  clickDeleteParam(e) {
    let param = e.target.dataset.param;
    let trackingLink = this.state.trackingLink;
    let whitelist = trackingLink.forwardQueryParamsWhitelist;
    pull(whitelist, param);
    this.setState({
      trackingLink
    }, () => {
      this.setState({
        changesToSave: this.checkForChanges()
      });
    });
  }

  copyToClipboard(e) {
    e.preventDefault();
    navigator.clipboard.writeText(`https://www.teachable.com/r/${this.state.trackingLink.slug}`);
  }

  changeSlug(obj) {
    let trackingLink = this.state.trackingLink;
    trackingLink["slug"] = obj.slug;
    let errors = this.state.errors;
    errors = pull(errors, "Slug has already been taken");
    this.setState({
      errors,
      trackingLink,
      customSlugModalOpen: false
    });
  }

  clickNew() {
    this.setState({
      fetching: true
    });
    let trackingLink = cloneDeep(this.state.trackingLink);
    trackingLink.metadata = this.convertToNestedObject(metadataKeys, trackingLink);
    trackingLink.additionalQueryParams = this.convertToNestedObject(queryParameterKeys, trackingLink);
    this.props.createEntity({
      entityName: 'trackingLink',
      directory: 'tracking_links',
      entity: trackingLink
    }).then(() => {
      this.setState({
        changesToSave: false,
      }, () => {
        window.location.pathname = `/admin/tracking_links/${this.props.trackingLink.slug}`;
      })
    }, () => {
      this.setState({
        fetching: false,
        errors: this.props.errors
      });
    });
  }

  clickSave() {
    this.setState({
      fetching: true,
      justSaved: true
    }, () => {
      let trackingLink = this.state.trackingLink;
      trackingLink.metadata = this.convertToNestedObject(metadataKeys, trackingLink);
      trackingLink.additionalQueryParams = this.convertToNestedObject(queryParameterKeys, trackingLink);
      this.props.updateEntity({
        id: this.state.trackingLink.id,
        directory: "tracking_links",
        entityName: "trackingLink",
        entity: trackingLink
      }).then(() => {
        let trackingLink = this.convertDatabaseRecordToFrontendTrackingLink.call(this);
        this.setState({
          fetching: false,
          trackingLink,
          trackingLinkSaved: cloneDeep(trackingLink),
          changesToSave: false
        }, () => {
          Admin.resetNiceSelect({
            selector: 'select',
            removeOrphanedDivs: true,
            func: Admin.changeField.bind(this, this.changeFieldArgs())
          });
        });
      }, () => {
        this.setState({
          fetching: false,
          errors: this.props.errors
        });
      });
    });
  }

  setUrl = (destinationUrl) => {
    this.setState(({ trackingLink }) => ({
      trackingLink: {
        ...trackingLink,
        destinationUrl
      }
    }))
  }

  updateDestinationUrl = (e: React.ChangeEvent<HTMLInputElement>) => {
    const destinationUrl = e.target.value.split(' ').join('-');

    this.setState(({ trackingLink }) => ({
      trackingLink: {
        ...trackingLink,
        destinationUrl
      }
    }))
  }

  render() {
    const { trackingLink } = this.state;
    let previewUrl = '';
    let destinationUrl;
    try {
      destinationUrl = new URL(trackingLink.destinationUrl || '');
    } catch (e) {
      // parsing failed because the link is invalid, it's fine.
    }
    if (destinationUrl) {
      const params = new URLSearchParams(destinationUrl.search);

      queryParameterKeys.forEach(key =>
        trackingLink[key] && params.append(ChangeCase.snakeCase(key), trackingLink[key]))

      params.append('link_id', ( trackingLink.id || 'XXXX' ).toString())
      params.append('event_id', 'XXXXXXXX-XXXXX')

      const searchString = params.toString()
      destinationUrl.search = searchString ? `?${searchString}` : ''
      previewUrl = destinationUrl.toString()
    }
    return (
      <div id="tracking-link-new-copy-details" className="component details-component">
        <h1 className="search-box-header">{ this.props.new ? `${this.props.copy ? 'Copy' : 'New'} Tracking Link` : "Tracking Link details" }</h1>
        { this.renderCreatedByBox.call(this) }
        <div className="white-box mini teachable-font-medium m-bottom">
          <div className="inline-block m-right">{ `https://www.teachable.com/r/${this.state.trackingLink.slug}` }</div>
          { this.renderCustomizeSlugButton.call(this) }
          <a className="link" onClick={ this.copyToClipboard.bind(this) }>Copy to Clipboard</a>
          <div className={ this.state.errors.indexOf("Slug has already been taken") === -1 ? "hidden" : "" } style={ { color: "red", marginTop: "10px" } }>Slug has already been taken</div>
        </div>
        <div className="white-box">
          { Admin.renderSpinner(this.state.fetching) }
          { Admin.renderGrayedOut(this.state.fetching) }
          <div className="row no-field-error">
            <Field columnWidth={12} columnHeader="Link destination preview">
              <div>
                {previewUrl}&nbsp; {/* just to ensure the div doesn't collapse when empty*/}
              </div>
            </Field>
          </div>
          <hr/>
          <div className="row">
            <Field columnWidth={9} columnHeader="* Destination Url">
              <input type="text" data-field="destinationUrl" value={trackingLink.destinationUrl || ''} onChange={this.updateDestinationUrl} />
              {destinationUrl && (
                <LinkCheck url={trackingLink.destinationUrl} onConfirm={this.setUrl} />
              )}
              <Errors stateErrors={this.state.errors} fieldErrors={Admin.errors.destinationUrl} />
            </Field>
            { Admin.renderDropDown.bind(this)({
                columnWidth: 2,
                entity: "trackingLink",
                property: "status",
                optionsArrayName: "statuses",
                optionLabel: "text",
                optionValue: "text"
              }) }
          </div>
          <hr />
          <div className="row">
            <div className="col-xs-6">
              <h3>Attribution Information</h3>
              { Admin.renderDropDown.bind(this)({
                  columnWidth: 12,
                  includeBlank: true,
                  entity: "trackingLink",
                  property: "channel",
                  optionLabel: "text",
                  optionValue: "text",
                  limit: 6,
                  columnHeader: "* Channel"
                }) }
              { Admin.renderField.bind(this)({ columnWidth: 12, entity: "trackingLink", property: "subchannel", placeholder: "The Ringer", autoFormat: (input) => input.toLowerCase().replace(/ +/, '-') }) }
              { Admin.renderField.bind(this)({ columnWidth: 12, entity: "trackingLink", property: "source", placeholder: "bill-simmons-podcast", autoFormat: Admin.hyphenFormat, columnHeader: "* Source" }) }
              { Admin.renderField.bind(this)({ columnWidth: 12, entity: "trackingLink", property: "campaign", placeholder: "creator-challenge-2019", autoFormat: Admin.hyphenFormat }) }
              { Admin.renderField.bind(this)({ columnWidth: 12, entity: "trackingLink", property: "subcampaign", placeholder: "us-daypart-preroll-insert", autoFormat: Admin.hyphenFormat }) }
              { Admin.renderField.bind(this)({ columnWidth: 12, entity: "trackingLink", property: "content", placeholder: "creator-challenge-offer-v1", autoFormat: Admin.hyphenFormat }) }
              { Admin.renderField.bind(this)({ columnWidth: 12, entity: "trackingLink", property: "medium", placeholder: "newsletter", autoFormat: Admin.hyphenFormat }) }
              { Admin.renderDropDown.bind(this)({
                  columnWidth: 12,
                  includeBlank: true,
                  entity: "trackingLink",
                  property: "audience",
                  optionLabel: "text",
                  optionValue: "text",
                  limit: 6,
                  columnHeader: "* Audience"
                }) }
              { Admin.renderDropDown.bind(this)({
                  columnWidth: 12,
                  includeBlank: true,
                  entity: "trackingLink",
                  property: "placement",
                  optionLabel: "text",
                  optionValue: "text",
                  columnHeader: "* Placement"
                }) }
            </div>
            <div className="col-xs-6">
              <h3>Additional Params</h3>
              { Admin.renderField.bind(this)({ columnWidth: 12, entity: "trackingLink", property: "couponCode", autoFormat: Admin.hyphenUpperCaseFormat }) }
              { Admin.renderField.bind(this)({ columnWidth: 12, entity: "trackingLink", property: "upgradeSrc", autoFormat: Admin.hyphenFormat, columnHeader: "Upgrade Source" }) }
              { Admin.renderField.bind(this)({ columnWidth: 12, entity: "trackingLink", property: "cx", autoFormat: Admin.hyphenFormat, columnHeader: "CX" }) }
              { Admin.renderDropDown.bind(this)({
                  columnWidth: 12,
                  entity: "trackingLink",
                  property: "planId",
                  columnHeader: "Plan",
                  optionsArrayName: "plans",
                  optionLabel: "text",
                  optionValue: "value",
                  skipChangeCase: true,
                  limit: 6,
                  includeBlank: true
                }) }
              { Admin.renderBooleanDropDown.bind(this)({ columnWidth: 5, entity: "trackingLink", property: "forwardAllQueryParams" }) }
              { this.renderWhitelistedParams() }
            </div>
          </div>
          <hr />
          { this.renderButtons.call(this) }
        </div>
        <Modal isOpen={ this.state.deleteModalOpen } onRequestClose={ Admin.closeDeleteModal.bind(this) } contentLabel="Modal" style={ ConfirmDeleteModalStyles }>
          <ConfirmDelete entityName="trackingLink" entityDirectory="tracking_links" entityId={ this.state.trackingLink.id } closeModal={ Admin.closeDeleteModal.bind(this) } closeAndFetch={ Admin.closeDeleteModalAndFetch.bind(this) } delete={ () => {this.props.deleteEntity({directory: "tracking_links", id: this.state.trackingLink.id}) }} />
        </Modal>
        <Modal isOpen={ this.state.paramModalOpen } onRequestClose={ Admin.closeModals.bind(this) } contentLabel="Modal" style={ Admin.newEntityModalStyles({ width: 500 }, 1) }>
          <NewEntity entityName="param" initialEntity={ { text: "" } } frontEndCallback={ this.addParam.bind(this) } />
        </Modal>
        <Modal isOpen={ this.state.customSlugModalOpen } onRequestClose={ Admin.closeModals.bind(this) } contentLabel="Modal" style={ Admin.newEntityModalStyles({ width: 500 }, 1) }>
          <NewEntity entityName="customSlug" initialEntity={ { slug: "" } } frontEndCallback={ this.changeSlug.bind(this) } buttonText="Change Slug" />
        </Modal>
      </div>
    );
  }

  renderCustomizeSlugButton() {
    if (this.props.new) {
      return(
        <a className="link m-right" onClick={ Admin.changeState.bind(this, "customSlugModalOpen", true) }>Customize Slug</a>
      );
    } else {
      return null;
    }
  }

  renderCreatedByBox() {
    if (!this.props.new) {
      return(
        <p className="created-by">Created by: { this.state.trackingLink.creatorName || "" }</p>
      );
    } else {
      return null;
    }
  }

  renderButtons() {
    if (this.props.new) {
      return(
        <div className="row">
          <div className="col-xs-12">
            <a className={ "orange-button standard-button-width btn" + Admin.renderDisabledButtonClass(this.state.fetching) } onClick={ this.clickNew.bind(this) }>
              { this.props.new ? 'Create' : 'Save' }
            </a>
          </div>
        </div>
      );
    } else {
      return(
        <div className="row">
          <div className="col-xs-12">
            <a className={ "orange-button standard-button-width btn" + Admin.renderDisabledButtonClass(this.state.fetching || !this.state.changesToSave) } onClick={ this.clickSave.bind(this) }>
              { Admin.saveButtonText.call(this) }
            </a>
            <a className={ "delete-button " + Admin.renderDisabledButtonClass(this.state.fetching) } onClick={ Admin.changeState.bind(this, "deleteModalOpen", true) }>
              Delete
            </a>
          </div>
        </div>
      );
    }
  }

  renderWhitelistedParams() {
    if (this.state.trackingLink.forwardQueryParamsWhitelist && !this.state.trackingLink.forwardAllQueryParams) {
      return(
        <div className="row">
          <div className="col-xs-12">
            <table className="admin-table no-hover with-button">
              <thead>
                <tr>
                  <th>Whitelisted Query Params</th>
                </tr>
              </thead>
              <tbody>
                <tr><td></td><td></td></tr>
                { this.state.trackingLink.forwardQueryParamsWhitelist.map((param, index) => {
                    return(
                      <tr key={ index }>
                        <td>{ param }</td>
                        <td><div className={ `x-button` } onClick={ this.clickDeleteParam.bind(this) } data-param={ param }></div></td>
                      </tr>
                    );
                }) }
              </tbody>
            </table>
            <a className={ `green-outline-button small m-bottom` } onClick={ this.clickAddParam.bind(this) }>Add Query Param</a>
          </div>
        </div>
      );
    } else {
      return null;
    }
  }

  componentDidUpdate() {
    $('.match-height-layout').matchHeight();
  }
}

const mapStateToProps = (reducers, props) => {
  return reducers.standardReducer;
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntity, fetchDataForNew, fetchDataForCopy, createEntity, updateEntity, deleteEntity }, dispatch);
}

export default connect(mapStateToProps, mapDispatchToProps)(withExitPrompt(TrackingLinkDetails));
