import React from 'react'
import { connect } from 'react-redux'
import { bindActionCreators } from 'redux'
import Modal from 'react-modal'
import JobStatus from './job-status'
import Admin from './../admin'
import orderBy from 'lodash/orderBy'
import truncate from 'lodash/truncate';
import { fetchEntities, exportTrackingLinks } from '../actions/index'
import { JobModalStyles } from './helpers/modal-styles'
import { TrackingLinkBulkUpload } from './tracking-link/tracking-link-bulk-upload'

interface JobType {
  id: number;
  text: string;
}

interface TrackingLinksIndexProps {
  exportTrackingLinks: any;
  fetchEntities: any;
  job: JobType;
  trackingLinks: any[];
}

interface TrackingLinksIndexState extends ComponentState {
  modalOpen: boolean;
  job: JobType;
  trackingLinks: any[];
}

class TrackingLinksIndex extends React.Component<TrackingLinksIndexProps, TrackingLinksIndexState> {

  constructor(props) {
    super(props);
    this.state = {
      job: null,
      fetching: true,
      trackingLinks: [],
      modalOpen: false,
      searchText: "",
      searchProperty: "createdAt"
    };
  }

  componentDidMount() {
    this.props.fetchEntities("tracking_links").then(() => {
      this.setState({
        fetching: false,
        trackingLinks: this.props.trackingLinks
      });
    });
  }

  clickNew() {
    window.location.href = '/admin/tracking_links/new';
  }

  clickExport() {
    this.setState({
      fetching: true
    });
    this.props.exportTrackingLinks().then(() => {
      const { job } = this.props;
      this.setState({
        fetching: false,
        job
      });
    });
  }

  exportDone(job) {
    this.setState({
      job: null
    });
    window.location.href = job.metadata.fileUrl;
  }

  clickCopy(id) {
    window.location.href = `/admin/tracking_links/copy?id=${id}`;
  }

  sortByTimestamp(trackingLink) {
    return trackingLink.createdAtTimestamp;
  }

  render() {
    const { trackingLinks, searchText, searchProperty } = this.state;
    const filteredTrackingLinks = Admin.filterSearchText(trackingLinks, searchText, searchProperty);
    return(
      <div id="tracking-links-index" className="component">
        <div className="clearfix">
          <h1 className="search-box-header">Tracking Links</h1>
          <a className={ "orange-button float-button btn" + Admin.renderDisabledButtonClass(this.state.fetching) } onClick={ this.clickExport.bind(this) }>Export All</a>
          <a className={ "orange-button float-button btn button-margin-right" + Admin.renderDisabledButtonClass(this.state.fetching) } onClick={ this.clickNew.bind(this) }>Add Tracking Link</a>
          <TrackingLinkBulkUpload />
          <input className="search-box margin" onChange={ Admin.changeSearchText.bind(this) } value={ this.state.searchText } />
        </div>
        <div className="white-box">
          { Admin.renderSpinner(this.state.fetching) }
          { Admin.renderGrayedOut(this.state.fetching) }
          <table className='admin-table contains-links'>
            <thead>
              <tr>
                <th><div className={ Admin.sortClass.bind(this)("createdAt") } onClick={ Admin.clickHeader.bind(this, "createdAt") }>Created At</div></th>
                <th><div className={ Admin.sortClass.bind(this)("creatorName") } onClick={ Admin.clickHeader.bind(this, "creatorName") }>Created By</div></th>
                <th><div className={ Admin.sortClass.bind(this)("slug") } onClick={ Admin.clickHeader.bind(this, "slug") }>Slug</div></th>
                <th><div className={ Admin.sortClass.bind(this)("destinationUrl") } onClick={ Admin.clickHeader.bind(this, "destinationUrl") }>Destination</div></th>
                <th></th>
              </tr>
            </thead>
            <tbody>
              <tr><td></td><td></td><td></td><td></td><td></td></tr>
              { orderBy(filteredTrackingLinks, [this.state.searchProperty === 'createdAt' ? this.sortByTimestamp.bind(this) : Admin.commonSort.bind(this)], [(this.state.searchProperty === 'createdAt') ? 'desc' : 'asc']).map((trackingLink, index) => {
                return(
                  <tr key={ index } data-id={ trackingLink.id }>
                    <td>
                      <a href={ `/admin/tracking_links/${trackingLink.slug}` }>
                        { trackingLink.createdAtParsed }
                      </a>
                    </td>
                    <td>
                      <a href={ `/admin/tracking_links/${trackingLink.slug}` }>
                        { trackingLink.creatorName }
                      </a>
                    </td>
                    <td>
                      <a href={ `/admin/tracking_links/${trackingLink.slug}` }>
                        { trackingLink.slug }
                      </a>
                    </td>
                    <td>
                      <a href={ `/admin/tracking_links/${trackingLink.slug}` }>
                        { truncate(trackingLink.destinationUrl, { length: 53 }) }
                      </a>
                    </td>
                    <td>
                      <div className="table-copy-button" onClick={ this.clickCopy.bind(this, trackingLink.id) }></div>
                    </td>
                  </tr>
                );
              }) }
            </tbody>
          </table>
        </div>
        <Modal isOpen={ !!this.state.job } contentLabel="Modal" style={ JobModalStyles }>
          <JobStatus job={ this.state.job } jobDone={ this.exportDone.bind(this) } />
        </Modal>
      </div>
    );
  }

  componentDidUpdate() {
    $('.match-height-layout').matchHeight();
  }
}

const mapStateToProps = (reducers, props) => {
  return reducers.standardReducer;
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntities, exportTrackingLinks }, dispatch);
}

export default connect(mapStateToProps, mapDispatchToProps)(TrackingLinksIndex);
