import React from 'react'
import { connect } from 'react-redux'
import { bindActionCreators } from 'redux'
import Modal from 'react-modal'
import NewEntity from './new-entity'
import ChangeCase from 'change-case'
import upperFirst from 'lodash/upperFirst';
import sortBy from 'lodash/sortBy';
import isEqual from 'lodash/isEqual';
import cloneDeep from 'lodash/cloneDeep'
import ModalSelect from './modal-select'
import Admin from './../admin'
import Tools from './../../app/assets/javascripts/teachable/tools'
import WebinarObject from './webinar-object'
import WebinarOfferIndexItem from './webinar-offer-index-item'
import WebinarCopy from './webinar-copy'
import ConfirmDelete from './confirm-delete'
import { SelectModalStyles, ConfirmDeleteModalStyles } from './helpers/modal-styles'
import { fetchEntity, createEntity, updateEntity, deleteEntity } from '../actions/index'
import { withExitPrompt } from './util/with-exit-prompt'

interface sidebarOption {
  enabled; boolean;
  name: string;
  order: number;
}

interface Download {
  id: number;
  name: string;
  order: number;
}

interface Host {
  id: number;
  name: string;
  order: number;
}

interface Bundle {
  id: number;
  name: string;
}

interface SelectType {
  label: string;
  value: string;
}

interface Offer {
  id?: number;
  duration: number;
  startTime: number;
  ctaButtonText: string;
  ctaUrl: string;
  ctaButtonType: string;
  description: string;
  title: string;
}

interface Platform extends SelectType {
  label: "Vimeo" | "YouTube";
  value: "vimeo" | "youtube";
}

interface WebinarType extends SelectType {
  label: "Live" | "Automated";
  value: "live" | "automated";
}

interface AutomationInterval extends SelectType {
  label: "Every 15 Minutes" | "Every 30 Minutes" | "Every 1 Hour";
  value: "every_15_minutes" | "every_30_minutes" | "every_1_hour";
}

interface Webinar {
  id?: number;
  name: string;
  url: string;
  title: string;
  date: string;
  recurring?: boolean;
  time: string;
  time2: string;
  hostIds: number[];
  header: string;
  subheader: string;
  bulletPoints?: string;
  customModalImage: string;
  replayVideoId: string;
  thankYouVideoId: string;
  customUtmContent?: string;
  customUtmCampaign?: string;
  customMtCampaign?: string;
  upgradeTimeframe: string;
  liveStartDate: string;
  liveEndDate: string;
  redirect: string;
  basicReplayBundle: string;
  basicPlayBundle: string;
  proReplayBundle: string;
  proPlayBundle: string;
  upgradeOptions: string;
  couponCode: string;
  faq: string;
  sidebarOptions: any;
  automationInterval: string;
  webinarType: string;
  automatedHostHeader?: string;
  automatedHostDescription?: string;
  automatedHostImageUrl?: string;
  automatedHeroImageUrl?: string;
  upcomingSessionId?: string;
  automatedPostEventRedirect?: string;
  duration?: number;
  description?: string;
  expectedDuration?: number;
  automatedCtaHeader?: string;
}

interface WebinarDetailsProps {
  bundles: any[];
  createEntity: any;
  deleteEntity: any;
  downloads: any[];
  errors: string[];
  fetchEntity: any;
  hosts: any[];
  otherDownloads: any[];
  otherHosts: any[];
  otherObjects: any[];
  selectedObjects: any[];
  offers: Offer[];
  updateEntity: any;
  webinar: Webinar;
}

interface WebinarDetailsState extends ComponentState {
  copyModalOpen?: boolean;
  deleteModalOpen?: boolean;
  downloadsModalOpen?: boolean;
  offerModalOpen?: boolean;
  hostsModalOpen?: boolean;
  webinar?: Webinar;
  webinarSaved?: Webinar;
  downloads?: Download[];
  otherDownloads?: Download[];
  hosts?: Host[];
  otherHosts?: Host[];
  offers?: Offer[];
  offer?: Offer;
  bundles?: Bundle[];
  platforms?: Platform[];
  webinarTypes?: WebinarType[];
  automationIntervals?: AutomationInterval[];
}

const CopyModalStyles = {
  overlay: {
    background: 'rgba(0, 0, 0, 0.50)'
  },
  content: {
    background: 'white',
    padding: 0,
    margin: 'auto',
    maxWidth: 1000,
    height: 240
  }
};

const anchorPoints = [
  { text: "at", value: "start" },
  { text: "with", value: "end" }
];

const ctaButtonTypes = [
  { text: "Green Button", value: "green_button" },
  { text: "Orange Button", value: "orange_button" },
  { text: "Link", value: "link" }
];

class WebinarDetails extends React.Component<WebinarDetailsProps, WebinarDetailsState> {

  constructor(props) {
    super(props);
    this.state = {
      fetching: true,
      downloadsModalOpen: false,
      offerModalOpen: false,
      hostsModalOpen: false,
      copyModalOpen: false,
      deleteModalOpen: false,
      webinar: {} as Webinar,
      webinarSaved: {} as Webinar,
      errors: [],
      bundles: [],
      downloads: [],
      otherDownloads: [],
      hosts: [],
      offers: [],
      otherHosts: [],
      platforms: [
        { label: "YouTube", value: "youtube" },
        { label: "Vimeo", value: "vimeo" }
      ],
      webinarTypes: [
        { label: "Automated", value: "automated" },
        { label: "Live", value: "live" }
      ],
      automationIntervals: [
        { label: "Every 15 Minutes", value: "every_15_minutes" },
        { label: "Every 30 Minutes", value: "every_30_minutes" },
        { label: "Every 1 Hour", value: "every_1_hour" }
      ]
    };
  }

  componentDidMount() {
    this.fetchWebinarData()
  }

  fetchWebinarData(){
    let pathDirectories = window.location.pathname.split('/');
    return this.props.fetchEntity({
      id: pathDirectories[pathDirectories.length - 1],
      directory: "webinars",
      entityName: "webinar"
    }).then(() => {
      let webinar = this.props.webinar;
      webinar.duration = webinar.duration && Math.round(webinar.duration / 60);
      this.setState({
        fetching: false,
        webinar: webinar,
        webinarSaved: cloneDeep(webinar),
        bundles: this.props.bundles,
        downloads: this.props.downloads,
        otherDownloads: this.props.otherDownloads,
        hosts: this.props.hosts,
        otherHosts: this.props.otherHosts,
        offers: this.props.offers
      }, () => {
        Admin.setUpNiceSelect({
          selector: '#webinar-details select',
          func: Admin.changeField.bind(this, this.changeFieldArgs())
        });
        this.setUpSidebarOptionsDragAndDrop();
      });
    });
  }

  checkForChanges() {
    return !isEqual(this.state.webinar, this.state.webinarSaved);
  }

  changeFieldArgs() {
    return {
      thing: 'webinar',
      errorsArray: this.state.errors,
      changesFunction: this.checkForChanges.bind(this)
    }
  }

  setUpSidebarOptionsDragAndDrop() {
    $('.sidebar-table td').draggable({
      cursor: '-webkit-grabbing',
      handle: '.handle',
      helper: function() { return '<div></div>'; },
      stop: this.dragEndHandler
    });
    $('.sidebar-table .top-drop-zone, .sidebar-table .drop-zone').droppable({
      accept: Admin.canIDrop, // note that top-drop-zone and bottom-drop-zone within this component will automatically not be droppable (since they're within the draggable td element), this function is just for the bottom-drop-zone in the component directly above
      tolerance: 'pointer',
      over: this.dragOverHandler,
      out: this.dragOutHandler,
      drop: this.dropHandler.bind(this)
    });
  }

  clickSave() {
    this.setState({
      fetching: true,
      justSaved: true
    }, () => {
      let pathDirectories = window.location.pathname.split('/');
      let updatedWebinar = cloneDeep(this.state.webinar);
      updatedWebinar.duration = updatedWebinar.duration && Math.round(updatedWebinar.duration / 60);
      updatedWebinar.sidebarOptions = JSON.stringify(updatedWebinar.sidebarOptions)
      this.props.updateEntity({
        id: pathDirectories[pathDirectories.length - 1],
        directory: "webinars",
        entityName: "webinar",
        entity: updatedWebinar
      }).then(() => {
        let webinar = this.props.webinar;
        webinar.duration = Math.round(webinar.duration / 60);
        this.setState({
          fetching: false,
          webinar: webinar,
          webinarSaved: cloneDeep(webinar),
          changesToSave: false
        });
      }, () => {
        this.setState({
          fetching: false,
          errors: this.props.errors
        });
      });
    });
  }

  clickAddDownload() {
    this.setState({
      downloadsModalOpen: true
    });
  }

  clickAddOffer() {
    this.setState({
      offerModalOpen: true
    });
  }

  clickAddHost() {
    this.setState({
      hostsModalOpen: true
    });
  }

  clickXButton(event, name) {
    var id = event.target.dataset.id;
    this.setState({
      fetching: true
    }, () => {
      this.props.deleteEntity({
        directory: 'webinar_objects',
        id,
        callback: (response) => {
          let name = response.templateName;
          if (name === 'host/inviter') {
            name = 'host';
          }
          this.setState({
            fetching: false,
            [`${name}s`]: response.selectedObjects,
            [`other${name}s`]: response.otherObjects
          });
        }
      });
    });
  }

  selectWebinarObject(name, e) {
    let id = e.target.dataset.id;
    let objektName = (name === 'host' ? 'host/inviter' : name);
    this.setState({
      fetching: true,
      [`${name}sModalOpen`]: false
    }, () => {
      this.props.createEntity({
        directory: 'webinar_objects',
        entityName: 'webinarObject',
        entity: {
          webinarId: this.state.webinar.id,
          objektName,
          objektId: id
        }
      }).then(() => {
        this.setState({
          fetching: false,
          [`${name}s`]: this.props.selectedObjects,
          [`other${ChangeCase.titleCase(name)}s`]: this.props.otherObjects
        });
      });
    });
  }

  clickCopy() {
    this.setState({
      copyModalOpen: true
    });
  }

  closeModal() {
    this.setState({
      downloadsModalOpen: false,
      hostsModalOpen: false,
      offerModalOpen: false
    });
  }

  mouseDownHandler(e) {
    $('.handle, a, input, textarea, .x-button, .nice-select, tr').addClass('grabbing');
    var entity = e.target.parentElement.parentElement;
    entity.classList.add('highlight');
  }

  mouseUpHandler(e) {
    $('.handle, a, input, textarea, .x-button, .nice-select, tr').removeClass('grabbing');
    e.target.parentElement.parentElement.classList.remove('highlight');
  }

  dragOverHandler(e) {
    e.target.classList.add('highlight');
  }

  dragOutHandler(e) {
    e.target.classList.remove('highlight');
  }

  dragEndHandler() {
    $('.handle, a, input, textarea, .x-button, .nice-select, tr').removeClass('grabbing');
    $('tr.highlight').removeClass('highlight');
  }

  dropHandler(e, ui) {
    const { webinar } = this.state;
    let draggedIndex = ui.draggable.attr('id').split('-')[1];
    let dropZoneIndex = e.target.dataset.index;
    $('.highlight').removeClass('highlight');
    const currentOrder = sortBy(Object.keys(webinar.sidebarOptions), (key) => webinar.sidebarOptions[key].order)
    let newOrder = Tools.rearrangeFields(currentOrder, draggedIndex, dropZoneIndex);
    let newSidebarOptions = {...webinar.sidebarOptions};
    Object.entries(newOrder).forEach((array) => {
      let order = array[0];
      let name = array[1];
      newSidebarOptions[name].order = order;
    });
    webinar.sidebarOptions = newSidebarOptions;
    this.setState({
      webinar: webinar
    }, () => {
      this.setState({
        changesToSave: this.checkForChanges()
      });
    });
  }

  updateWebinarObjects(response, entityName) {
    this.setState({
      [entityName === 'host/inviter' ? 'hosts' : 'downloads']: sortBy(response, 'order')
    });
  }

  modifyOfferBeforeCreate(entity) {
    let startTime = entity.startTime;
    if (!isNaN(+startTime)) {
      let startTimeInSeconds = startTime * 60;
      if (entity.anchorPoint === "start") {
        entity.startTime = startTimeInSeconds;
      } else {
        entity.startTime = (this.state.webinarSaved.duration * 60) - startTimeInSeconds;
      }
    }
    return entity;
  }

  addOfferCallback(webinarOffers) {
    this.setState({
      fetching: false,
      offers: webinarOffers,
      offerModalOpen: false
    });
  }

  render() {
    if (this.state.changesToSave) {
      var buttonText = 'Save';
    } else {
      var buttonText = this.state.justSaved ? 'Saved' : 'No Changes';
    }
    return(
      <div id="webinar-details">
        <div className="component">
          <div className="white-box">
            { Admin.renderSpinner(this.state.fetching) }
            { Admin.renderGrayedOut(this.state.fetching) }
            <div className="row">
              { Admin.renderField.bind(this)({ columnWidth: 3, entity: "webinar", property: "name", placeholder: "7 Steps" }) }
              { Admin.renderField.bind(this)({ columnWidth: 3, entity: "webinar", property: "url", placeholder: "7-steps-to-launch" }) }
              { Admin.renderField.bind(this)({ columnWidth: 3, entity: "webinar", property: "title", placeholder: "7 steps to launch your own profitable online course" }) }
              { Admin.renderDropDown.bind(this)({
                columnWidth: 3,
                entity: "webinar",
                property: "webinarType",
                optionLabel: "label",
                optionValue: "value",
                skipChangeCase: false
              }) }
            </div>
            { this.renderAutomatedEventForm() }
            { this.renderLiveEventForm() }
            <hr />
            { this.renderBottomLinks(buttonText) }
          </div>
        </div>
        <Modal isOpen={ this.state.downloadsModalOpen } onRequestClose={ this.closeModal.bind(this) } contentLabel="Modal" style={ SelectModalStyles }>
          <ModalSelect options={ this.state.otherDownloads } property="name" func={ this.selectWebinarObject.bind(this, 'download') } />
        </Modal>
        <Modal isOpen={ this.state.hostsModalOpen } onRequestClose={ this.closeModal.bind(this) } contentLabel="Modal" style={ SelectModalStyles }>
          <ModalSelect options={ this.state.otherHosts } property="name" func={ this.selectWebinarObject.bind(this, 'host') } />
        </Modal>
        <Modal isOpen={ this.state.copyModalOpen } onRequestClose={ () => { this.setState({ copyModalOpen: false }) } } contentLabel="Modal" style={ CopyModalStyles }>
          <WebinarCopy />
        </Modal>
        <Modal isOpen={ this.state.offerModalOpen } onRequestClose={ this.closeModal.bind(this) } contentLabel="Modal" style={ Admin.newEntityModalStyles({ width: 1000 }, 3) }>
          <NewEntity
            entityName="webinarOffer"
            initialEntity={ { title: "", description: "", webinarId: this.state.webinar.id, startTime: "0", ctaButtonType: "green_button", ctaButtonText: "", ctaUrl: "", anchorPoint: "start", duration: "0" } }
            staticState={ { anchorPoints, ctaButtonTypes } }
            duration={ this.state.webinarSaved.duration }
            beforeCreate={ this.modifyOfferBeforeCreate.bind(this) }
            callback={ this.addOfferCallback.bind(this) }
          />
        </Modal>
        <Modal isOpen={ this.state.deleteModalOpen } onRequestClose={ Admin.closeDeleteModal.bind(this) } contentLabel="Modal" style={ ConfirmDeleteModalStyles }>
          <ConfirmDelete entityId={ this.state.webinar.id } entityName="webinar" entityDirectory="webinars" closeModal={ Admin.closeDeleteModal.bind(this) } closeAndFetch={ Admin.closeDeleteModalAndFetch.bind(this) } delete={ Admin.clickDelete.bind(this) } />
        </Modal>
      </div>
    );
  }

  renderAutomatedEventForm() {
    return(
      <div className={this.state.webinar.webinarType === 'automated' ? '' : 'hidden'}>
        { this.renderAutomatedEventFields() }
        <hr/>
        { this.renderPlayPageDetails() }
        <hr/>
        { this.renderWebinarOffers() }
      </div>
    )
  }

  renderWebinarOffers() {
    return(
      <div>
        <h3>Offers</h3>
        <table className="admin-table no-hover downloads-table">
          <thead>
            <tr>
              <th>Title</th>
              <th>Description</th>
              <th>Appears</th>
              <th>Lasts for</th>
            </tr>
          </thead>
          <tbody>
            <tr><td></td></tr>
            { sortBy(this.state.offers, "startTime").map((offer, index) => {
              return(
                <WebinarOfferIndexItem key={ index } offer={ offer } webinarDuration={ this.state.webinar.duration } />
              );
            }) }
          </tbody>
        </table>
        <a className="green-outline-button small" onClick={ this.clickAddOffer.bind(this) }>Add Offer</a>
      </div>
    );
  }

  renderAutomatedEventFields() {
    return(
      <div>
        <div className="row">
          {
            Admin.renderDropDown.bind(this)({
              columnWidth: 3,
              entity: "webinar",
              property: "automationInterval",
              optionLabel: "label",
              optionValue: "value",
              skipChangeCase: true,
              doNotAlphabetize: true,
            })
          }
          {
            Admin.renderField.bind(this)({
              columnWidth: 3,
              entity: "webinar",
              property: "thankYouVideoId",
              columnHeader: "Thank You Video ID",
            })
          }
          {
            Admin.renderField.bind(this)({
              columnWidth: 3,
              entity: "webinar",
              property: "replayVideoId",
              columnHeader: "Automated Video ID",
            })
          }
          {
            Admin.renderField.bind(this)({
              columnWidth: 3,
              entity: "webinar",
              property: "duration",
              columnHeader: "Webinar Duration (in minutes)"
            })
          }
        </div>
        <div className="row">
          {
            Admin.renderImageField.bind(this)({
              columnWidth: 3,
              entity: "webinar",
              property: "automatedHeroImageUrl",
              imageUrlProperty: "automatedHeroImageUrl",
              columnHeader: "Registration Hero",
              createImageRecord: false
            })
          }
          {
            Admin.renderImageField.bind(this)({
              columnWidth: 3,
              entity: "webinar",
              property: "automatedHostImageUrl",
              imageUrlProperty: "automatedHostImageUrl",
              columnHeader: "Host Image",
              createImageRecord: false
            })
          }
          {
            Admin.renderField.bind(this)({
              columnWidth: 6,
              entity: "webinar",
              property: "automatedHostHeader",
              columnHeader: "Host Header Text",
              placeholder: "Hosted by Jess Catorc & Cameron Mattis of Teachable"
            })
          }
        </div>
        <div className="row">
          {
            Admin.renderTextBox.bind(this)({
              columnWidth: 6,
              entity: "webinar",
              property: "automatedHostDescription",
              columnHeader: "Host Header Description",
              placeholder: "We're here to help you do you :)",
              rows: 8,
              fieldInfo: "*Accepts Markdown and HTML Tags"
            })
          }
          {
            Admin.renderTextBox.bind(this)({
              columnWidth: 6,
              entity: "webinar",
              property: "automatedCtaHeader",
              columnHeader: "Cta Header",
              placeholder: "Upgrade Now",
              rows: 8,
              fieldInfo: "*Accepts Markdown and HTML Tags"
            })
          }
        </div>
        <div className="row">
          {
            Admin.renderField.bind(this)({
              columnWidth: 12,
              entity: "webinar",
              property: "automatedPostEventRedirect",
              columnHeader: "Redirect URL (When Event Ends)",
            })
          }
        </div>
      </div>
    );
  }

  renderHostFields() {
    return(
      <div>
        <table className="admin-table no-hover hosts-table">
          <thead>
            <tr>
              <th>Hosts</th>
            </tr>
          </thead>
          <tbody>
            <tr><td></td></tr>
            { this.state.hosts.map((host, index) => {
              return(
                <WebinarObject key={ host.id } index={ index } entity={ host } entityName="host/inviter" webinarId={ this.state.webinar.id } clickXButton={ this.clickXButton.bind(this) } renderHandle={ this.state.hosts.length > 1 } entities={ this.state.hosts } tableClassName="hosts-table" callback={ this.updateWebinarObjects.bind(this) } />
              );
            }) }
          </tbody>
        </table>
        <a className={ 'green-outline-button small' } onClick={ this.clickAddHost.bind(this) }>Add Host</a>
      </div>
    );
  }

  renderPlayPageDetails() {
    return(
      <div>
        <div className="row">
          <div className="col-xs-12">
            <h2>Header</h2>
            <input className={ Admin.errorClass(this.state.errors, []) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.webinar.header || "" } data-field="header" />
            { Admin.renderFieldError(this.state.errors, []) }
          </div>
        </div>
        <div className="row">
          <div className="col-xs-12">
            <h2>Subheader</h2>
            <input className={ Admin.errorClass(this.state.errors, []) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.webinar.subheader || "" } data-field="subheader" />
            { Admin.renderFieldError(this.state.errors, []) }
          </div>
        </div>
        <div className="row">
          <div className="col-xs-12">
            <h2>Description</h2>
            <textarea rows={5} className={ Admin.errorClass(this.state.errors, []) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.webinar.description || "" } data-field="description" />
            { Admin.renderFieldError(this.state.errors, []) }
          </div>
        </div>
        <div className="row">
          <div className="col-xs-12">
            <h2>Bullet Points</h2>
            <textarea rows={ 5 } value={ this.state.webinar.bulletPoints || "" } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } data-field="bulletPoints"></textarea>
            { Admin.renderFieldError(this.state.errors, []) }
          </div>
        </div>
        <div className="row">
          <div className="col-xs-12">
            <h2>FAQ <small>(supports markdown)</small></h2>
            <textarea rows={ 5 } value={ this.state.webinar.faq || "" } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } data-field="faq"></textarea>
            { Admin.renderFieldError(this.state.errors, []) }
          </div>
        </div>
      </div>
    )
  }

  renderLiveEventForm() {
    return(
      <div className={this.state.webinar.webinarType === 'live' ? '' : 'hidden'}>
        <div className="row">
          { Admin.renderField.bind(this)({ columnWidth: (this.state.webinar.recurring ? 6 : 3), entity: "webinar", property: "time", columnHeader: "Event Date/Time", placeholder: "11/5/1605 2:00pm", readOnly: this.state.webinar.recurring }) }
          { Admin.renderField.bind(this)({ columnWidth: 3, entity: "webinar", property: "time2", columnHeader: "Event Date/Time 2", placeholder: "11/5/1605 2:00pm", hidden: this.state.webinar.recurring }) }
          { Admin.renderField.bind(this)({ columnWidth: 2, entity: "webinar", property: "expectedDuration", columnHeader: "Expected Duration (in minutes)", placeholder: "90" }) }
          { Admin.renderDropDown.bind(this)({
            columnWidth: 2,
            entity: "webinar",
            property: "platform",
            optionLabel: "label",
            optionValue: "value",
            skipChangeCase: true
          }) }
        </div>
        { this.renderLiveFields() }
        <hr />
        <div className="row">
          <div className="col-xs-12">
            { this.renderHostFields( )}
          </div>
        </div>
        <hr />
        { this.renderPlayPageDetails() }
        <hr />
        { this.renderChatSidebarOptions() }
        <hr />
        <div className="row">
          <div className="col-xs-6">
            <div className="row">
              { Admin.renderImageField.bind(this)({ columnWidth: 9, entity: "webinar", property: "customModalImage", columnHeader: "Custom Registration Page Modal Image", fieldInfo: "Leave this blank to display the default image with Jess and Cam" }) }
            </div>
            <div className="row">
              { Admin.renderField.bind(this)({ columnWidth: 9, entity: "webinar", property: "couponCode", fieldInfo: "Fill out this field to use a coupon with this webinar" }) }
            </div>
            <div className="row">
              { Admin.renderField.bind(this)({ columnWidth: 9, entity: "webinar", property: "customCtaUrl", columnHeader: "Custom CTA Base URL", fieldInfo: "Fill out this field to link to a page other than SPC" }) }
            </div>
            <div className="row">
              { Admin.renderField.bind(this)({ columnWidth: 9, entity: "webinar", property: "customCtaText", columnHeader: "Custom CTA Text", fieldInfo: "Leave this blank to use \"Upgrade to Pro/Basic/Annual/Monthly\"" }) }
            </div>
          </div>
          <div className="col-xs-6">
            <div className="row">
              { Admin.renderField.bind(this)({ columnWidth: 9, entity: "webinar", property: "customUtmCampaign", columnHeader: "Custom CTA Param: utm_campaign", fieldInfo: `Leave this blank to use "YYYY_MM_DD-${this.state.webinar.url || ""}"` }) }
            </div>
            <div className="row">
              { Admin.renderField.bind(this)({ columnWidth: 9, entity: "webinar", property: "customUtmContent", columnHeader: "Custom CTA Param: utm_content", fieldInfo: "Leave this blank to use \"play_page_btn\" and \"replay_page_btn\"" }) }
            </div>
            <div className="row">
              { Admin.renderField.bind(this)({ columnWidth: 9, entity: "webinar", property: "customMtCampaign", columnHeader: "Custom CTA Param: mt_campaign_group", fieldInfo: "Leave this blank to use \"webinar\"" }) }
            </div>
            <div className="row">
              { Admin.renderBooleanDropDown.bind(this)({ columnWidth: 9, entity: "webinar", property: "regPageWarning", columnHeader: "Registration Page - Limited Space Warning" }) }
            </div>
            <div className="row">
              { Admin.renderBooleanDropDown.bind(this)({ columnWidth: 9, entity: "webinar", property: "displayThankYouFreeAccountCta", columnHeader: "Thank You Page - Free Account CTA" }) }
            </div>
          </div>
        </div>
        <hr />
        <div className="row">
          { Admin.renderField.bind(this)({ columnWidth: 3, entity: "webinar", property: "thankYouVideoId", columnHeader: "Thank You Video Wistia ID" }) }
          { Admin.renderField.bind(this)({ columnWidth: 3, entity: "webinar", property: "replayVideoId", columnHeader: "Replay Video Wistia ID" }) }
          <div className="col-xs-3">
            <h2>Upgrade Time Frame</h2>
            <div className="clearfix">
              <select className={ Admin.errorClass(this.state.errors, Admin.errors.upgradeTimeframe) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.webinar.upgradeTimeframe || "" } data-field="upgradeTimeframe">
                <option value="monthly">Monthly</option>
                <option value="annual">Annual</option>
                <option value="monthly/annual">Monthly and Annual</option>
              </select>
            </div>
            { Admin.renderFieldError(this.state.errors, Admin.errors.upgradeTimeframe) }
          </div>
          <div className="col-xs-3">
            <h2>Upgrade Options</h2>
            <select className={ Admin.errorClass(this.state.errors, Admin.errors.upgradeTimeframe) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.webinar.upgradeOptions || "" } data-field="upgradeOptions">
              <option value="basic/pro">Basic and Pro</option>
              <option value="basic">Basic Only</option>
              <option value="pro">Pro Only</option>
              <option value="business">Business Only</option>
            </select>
          </div>
        </div>
        <div className="row">
          <div className="col-xs-3 select-scroll-6">
            <h2>Basic Replay Bundle</h2>
            <select className={ Admin.errorClass(this.state.errors, []) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.webinar.basicReplayBundle || "" } data-field="basicReplayBundle">
              <option value="">(None)</option>
              { this.state.bundles.map((bundle, index) => {
                return(
                  <option key={ index } value={ bundle.id }>{ bundle.name }</option>
                );
              }) }
            </select>
            <div className="no-dropdown-field-error"></div>
          </div>
          <div className="col-xs-3 select-scroll-6">
            <h2>Basic Play Bundle</h2>
            <select className={ Admin.errorClass(this.state.errors, []) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.webinar.basicPlayBundle || "" } data-field="basicPlayBundle">
              <option value="">(None)</option>
              { this.state.bundles.map((bundle, index) => {
                return(
                  <option key={ index } value={ bundle.id }>{ bundle.name }</option>
                );
              }) }
            </select>
            <div className="no-dropdown-field-error"></div>
          </div>
          <div className="col-xs-3 select-scroll-6">
            <h2>Pro Replay Bundle</h2>
            <select className={ Admin.errorClass(this.state.errors, []) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.webinar.proReplayBundle || "" } data-field="proReplayBundle">
              <option value="">(None)</option>
              { this.state.bundles.map((bundle, index) => {
                return(
                  <option key={ index } value={ bundle.id }>{ bundle.name }</option>
                );
              }) }
            </select>
            <div className="no-dropdown-field-error"></div>
          </div>
          <div className="col-xs-3 select-scroll-6">
            <h2>Pro Play Bundle</h2>
            <select className={ Admin.errorClass(this.state.errors, []) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.webinar.proPlayBundle || "" } data-field="proPlayBundle">
              <option value="">(None)</option>
              { this.state.bundles.map((bundle, index) => {
                return(
                  <option key={ index } value={ bundle.id }>{ bundle.name }</option>
                );
              }) }
            </select>
            <div className="no-dropdown-field-error"></div>
          </div>
        </div>
        <hr />
        <table className="admin-table no-hover downloads-table">
          <thead>
            <tr>
              <th>Downloads</th>
            </tr>
          </thead>
          <tbody>
            <tr><td></td></tr>
            { this.state.downloads.map((download, index) => {
              return(
                <WebinarObject key={ download.id } index={ index } entity={ download } entityName="download" webinarId={ this.state.webinar.id } clickXButton={ this.clickXButton.bind(this) } renderHandle={ this.state.downloads.length > 1 } entities={ this.state.downloads } tableClassName={ "downloads-table" } callback={ this.updateWebinarObjects.bind(this) } />
              );
            }) }
          </tbody>
        </table>
        { this.renderAddDownloadButton() }
      </div>
    );
  }

  renderBottomLinks(buttonText: string) {
    return(
      <div>
        <a className={ "orange-button standard-button-width btn" + Admin.renderDisabledButtonClass(this.state.fetching || !this.state.changesToSave) } onClick={ this.clickSave.bind(this) }>
          { buttonText }
        </a>
        <a className={ "delete-button btn" + Admin.renderDisabledButtonClass(this.state.fetching) } onClick={ Admin.changeState.bind(this, "deleteModalOpen", true) }>
          Delete
        </a>
        <a className={ "light-orange-button copy-button btn" + Admin.renderDisabledButtonClass(this.state.fetching) } onClick={ this.clickCopy.bind(this) }>
          Copy
        </a>
        <div className="page-links-container">
          <a className={ "light-orange-button page-links-button btn" + Admin.renderDisabledButtonClass(this.state.fetching) }>
            Page links
          </a>
          <ul className="page-links-inner-menu">
            <div></div>
            <li><a href={ "/webinars/" + this.state.webinar.url + "/register" } target="_blank" rel="noopener noreferrer">Register</a></li>
            <li><a href={ "/webinars/" + this.state.webinar.url + "/thank-you" + (this.state.webinar.webinarType == "automated" ? `?session_id=${this.state.webinar.upcomingSessionId}`: "") } target="_blank" rel="noopener noreferrer">Thank You</a></li>
            <li><a href={ "/webinars/" + this.state.webinar.url + "/play" + (this.state.webinar.webinarType == "automated" ? `/${this.state.webinar.upcomingSessionId}`: "") } target="_blank" rel="noopener noreferrer">Play</a></li>
            <li><a href={ `/webinars/${this.state.webinar.url}/replay` } target="_blank" rel="noopener noreferrer">Replay</a></li>
          </ul>
        </div>
      </div>
    );
  }

  clickSidebarSwitch(e) {
    let webinar = this.state.webinar;
    let newSidebarOptions = { ...webinar.sidebarOptions };
    newSidebarOptions[e.target.dataset.name].enabled = newSidebarOptions[e.target.dataset.name].enabled == "true" ? "false" : "true";
    webinar.sidebarOptions = newSidebarOptions;
    this.setState({
      webinar: webinar
    }, () => {
      this.setState({
        changesToSave: this.checkForChanges()
      });
    });
  }

  renderChatSidebarOptions() {
    if (this.state.webinar.sidebarOptions) {
      let sidebarOptions = Object.entries(this.state.webinar.sidebarOptions).map((array, index) => {
        let key = array[0];
        let obj = array[1];
        return Object.assign(obj, { name: key });
      });
      let sortedSidebarOptions = sortBy(sidebarOptions, "order");
      return(
        <div className="row">
          <div className="col-xs-12">
            <table className="admin-table no-hover sidebar-table">
              <thead>
                <tr>
                  <th>Chat Sidebar</th>
                </tr>
              </thead>
              <tbody>
                <tr><td></td></tr>
                { sortedSidebarOptions.map((sidebarOption: sidebarOption, index) => {
                  return(
                    <tr key={ index }>
                      <td id={ `index-${sidebarOption.order}` } data-index={ sidebarOption.order }>
                        { this.renderTopDropZone(sidebarOption.order) }
                        <div>
                          { upperFirst(sidebarOption.name) }
                        </div>
                        <div className={ `switch-container small-switch${ ["chat", "replies"].includes(sidebarOption.name) ? " hidden" : "" }` }>
                          <label>
                            <input type="checkbox" checked={ sidebarOption.enabled == "true" } onChange={ this.clickSidebarSwitch.bind(this) } data-name={ sidebarOption.name } />
                            <div></div>
                          </label>
                        </div>
                        <img className="handle" src={ window.Images.handle } onMouseDown={ this.mouseDownHandler } onMouseUp={ this.mouseUpHandler } />
                        { this.renderBottomDropZone(sidebarOption.order) }
                      </td>
                    </tr>
                  );
                }) }
              </tbody>
            </table>
          </div>
        </div>
      );
    } else {
      return null;
    }
  }

  renderLiveFields() {
    if (this.state.webinar.recurring === false) {
      return(
        <div className="row">
          <div className="col-xs-3">
            <h2>Active Start Date/Time</h2>
            <input className={ Admin.errorClass(this.state.errors, Admin.errors.liveStartDate) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.webinar.liveStartDate } data-field="liveStartDate" placeholder="11/5/1605 2:00pm" />
            { Admin.renderFieldError(this.state.errors, Admin.errors.liveStartDate) }
          </div>
          <div className="col-xs-3">
            <h2>Active End Date/Time</h2>
            <input className={ Admin.errorClass(this.state.errors, Admin.errors.liveEndDate) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.webinar.liveEndDate } data-field="liveEndDate" placeholder="11/5/1605 2:00pm" />
            { Admin.renderFieldError(this.state.errors, Admin.errors.liveEndDate) }
          </div>
          <div className="col-xs-6">
            <h2>Expired Redirect</h2>
            <input className={ Admin.errorClass(this.state.errors, Admin.errors.redirect) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.webinar.redirect } data-field="redirect" />
            { Admin.renderFieldError(this.state.errors, Admin.errors.redirect) }
          </div>
        </div>
      );
    } else {
      return null;
    }
  }

  renderAddDownloadButton() {
    if (this.state.downloads.length < 8) {
      return(
        <a className={ 'green-outline-button small' } onClick={ this.clickAddDownload.bind(this) }>Add Download</a>
      );
    } else {
      return null;
    }
  }

  renderTopDropZone(order) {
    return(
      <div className={ "top-drop-zone" + (order == 0 ? '' : ' hidden') } data-index={ "-1" }></div>
    );
  }

  renderBottomDropZone(order) {
    return(
      <div className="drop-zone" data-index={ order }></div>
    );
  }

  componentDidUpdate() {
    $('.match-height-layout').matchHeight();
  }
}

const mapStateToProps = (reducers, props) => {
  return reducers.standardReducer;
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntity, createEntity, updateEntity, deleteEntity }, dispatch);
}

export default connect(mapStateToProps, mapDispatchToProps)(withExitPrompt(WebinarDetails));
