import React from 'react'
import Modal from 'react-modal'
import { connect } from 'react-redux'
import { bindActionCreators } from 'redux'
import isEqual from 'lodash/isEqual'
import cloneDeep from 'lodash/cloneDeep'
import ConfirmDelete from './confirm-delete'
import Admin from './../admin'
import { ConfirmDeleteModalStyles } from './helpers/modal-styles'
import { fetchEntity, updateEntity, deleteEntity } from '../actions/index'
import { withExitPrompt } from './util/with-exit-prompt'

interface WebinarOfferDetailsProps {
  deleteEntity: any;
  duration: number;
  errors: string[];
  fetchEntity: any;
  webinar: any;
  webinarOffer: any;
  webinarOfferSaved: any;
  updateEntity: any;
}

interface WebinarOfferDetailsState extends ComponentState {
  anchorPoints: any;
  changesToSave: boolean;
  ctaButtonTypes: any;
  deleteModalOpen: boolean;
  errors: string[];
  webinarOffer: any;
  webinarOfferSaved: any;
}

class WebinarOfferDetails extends React.Component<WebinarOfferDetailsProps, WebinarOfferDetailsState> {

  constructor(props) {
    super(props);
    this.state = {
      changesToSave: false,
      deleteModalOpen: false,
      fetching: true,
      webinarOffer: {},
      webinarOfferSaved: {},
      errors: [],
      anchorPoints: [
        { text: "at", value: "start" },
        { text: "with", value: "end" }
      ],
      ctaButtonTypes: [
        { text: "Green Button", value: "green_button" },
        { text: "Orange Button", value: "orange_button" },
        { text: "Link", value: "link" }
      ]
    };
  }

  componentDidMount() {
    let pathDirectories = window.location.pathname.split('/');
    this.props.fetchEntity({
      id: pathDirectories[pathDirectories.length - 1],
      directory: "webinar_offers",
      entityName: "webinarOffer"
    }).then(() => {
      let webinarOffer = Object.assign(this.props.webinarOffer, { anchorPoint: 'start', startTime: Math.floor(this.props.webinarOffer.startTime / 60), duration: Math.floor(this.props.webinarOffer.duration / 60) });
      this.setState({
        fetching: false,
        webinarOffer,
        webinarOfferSaved: cloneDeep(webinarOffer)
      }, () => {
        Admin.setUpNiceSelect({ selector: 'select', func: Admin.changeField.bind(this, this.changeFieldArgs()) });
      });
    });
  }

  changeFieldArgs() {
    return {
      thing: "webinarOffer",
      errorsArray: this.state.errors,
      changesFunction: this.checkForChanges.bind(this)
    };
  }

  checkForChanges() {
    return !isEqual(this.state.webinarOffer, this.state.webinarOfferSaved);
  }

  clickSave() {
    this.setState({
      fetching: true
    }, () => {
      let webinarOffer = this.state.webinarOffer;
      let startTime = webinarOffer.startTime;
      let offerDurationInMinutes = webinarOffer.duration;
      let webinarDurationInSeconds = this.props.webinar.duration;
      let webinarDurationInMinutes = Math.floor(webinarDurationInSeconds / 60);

      if (!isNaN(+startTime)) {
        let startTimeInSeconds = startTime * 60;
        if (webinarOffer.anchorPoint === 'start') {
          webinarOffer.startTime = startTimeInSeconds;
        } else {
          webinarOffer.startTime = webinarDurationInSeconds - startTimeInSeconds;
        }
      }

      if (!isNaN(+offerDurationInMinutes)) {
        let offerDurationInSeconds = offerDurationInMinutes * 60;
        webinarOffer.duration = offerDurationInSeconds;
      }

      let pathDirectories = window.location.pathname.split('/');
      this.props.updateEntity({
        id: pathDirectories[pathDirectories.length - 1],
        directory: 'webinar_offers',
        entityName: 'webinarOffer',
        entity: webinarOffer
      }).then(() => {
        let anchorPoint = this.state.webinarOffer.anchorPoint;
        let mergeObj = { anchorPoint }
        let startTimeInMinutes = Math.floor(this.props.webinarOffer.startTime / 60);
        mergeObj['duration'] = Math.floor(this.props.webinarOffer.duration / 60);
        if (anchorPoint === 'end') {
          mergeObj['startTime'] = webinarDurationInMinutes - startTimeInMinutes;
        } else {
          mergeObj['startTime'] = startTimeInMinutes;
        }
        let webinarOffer = Object.assign(this.props.webinarOffer, mergeObj);
        this.setState({
          fetching: false,
          webinarOffer,
          webinarOfferSaved: cloneDeep(webinarOffer),
          changesToSave: false
        });
      }, () => {
        this.setState({
          fetching: false,
          errors: this.props.errors
        });
      });
    });
  }

  confirmDelete() {
    this.props.deleteEntity({
      directory: 'webinar_offers',
      id: this.state.webinarOffer.id,
      callback: () => {
        window.location.href = `/admin/webinars/${this.state.webinarOffer.webinarId}`;
      }
    });
  }

  render() {
    return(
      <div id="webinar-offer-details" className="component details-component">
        <h1>Webinar Offer Details</h1>
        <div className="white-box m-bottom">
          { Admin.renderSpinner(this.state.fetching) }
          { Admin.renderGrayedOut(this.state.fetching) }
          <div className="row">
            { Admin.renderField.bind(this)({ columnWidth: 6, entity: "webinarOffer", property: "title" }) }
            { Admin.renderField.bind(this)({ columnWidth: 6, entity: "webinarOffer", property: "description" }) }
          </div>
          <div className="row">
            { Admin.renderDropDown.bind(this)({
              columnWidth: 2,
              entity: "webinarOffer",
              property: "anchorPoint",
              optionLabel: "text",
              optionValue: "value",
              doNotAlphabetize: true,
              columnHeader: "Appears",
              readOnly: !(this.props.webinar && +this.props.webinar.duration)
            }) }
            { Admin.renderNumberField.bind(this)({ columnWidth: 3, entity: "webinarOffer", property: "startTime", columnHeader: (this.state["webinarOffer"].anchorPoint === "start" ? "Minutes In" : "Minutes Left"), min: 0, max: (this.props.webinar && +this.props.webinar.duration) }) }
            { Admin.renderNumberField.bind(this)({ columnWidth: 3, entity: "webinarOffer", property: "duration", columnHeader: "Duration (in minutes)" }) }
          </div>
          <div className="row">
            { Admin.renderDropDown.bind(this)({
              columnWidth: 4,
              entity: "webinarOffer",
              property: "ctaButtonType",
              columnHeader: "CTA Type",
              optionLabel: "text",
              optionValue: "value"
            }) }
            { Admin.renderField.bind(this)({ columnWidth: 4, entity: "webinarOffer", property: "ctaButtonText", columnHeader: "CTA Text" }) }
            { Admin.renderField.bind(this)({ columnWidth: 4, entity: "webinarOffer", property: "ctaUrl", columnHeader: "CTA Url" }) }
          </div>
          { Admin.renderDetailsButtons.call(this) }
        </div>
        <a className="link" href={ `/admin/webinars/${this.state.webinarOffer.webinarId}` }>Return to Webinar Details</a>
        <Modal isOpen={ this.state.deleteModalOpen } onRequestClose={ Admin.closeDeleteModal.bind(this) } contentLabel='Modal' style={ ConfirmDeleteModalStyles }>
          <ConfirmDelete skipCheck={ true } entityId={ this.state.webinarOffer.id } entityName="webinarOffer" entityDirectory="webinar_offers" closeModal={ Admin.closeDeleteModal.bind(this) } closeAndFetch={ Admin.closeDeleteModalAndFetch.bind(this) } delete={ this.confirmDelete.bind(this) } />
        </Modal>
      </div>
    );
  }

  componentDidUpdate() {
    $('.match-height-layout').matchHeight();
  }
}

const mapStateToProps = (reducers, props) => {
  return reducers.standardReducer;
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntity, updateEntity, deleteEntity }, dispatch);
}

export default connect(mapStateToProps, mapDispatchToProps)(withExitPrompt(WebinarOfferDetails));
