# frozen_string_literal: true

# CloudFlare proxy will add some headers that will tell us more information about the true end user.
# This middleware overrides the normal request settings to make it appear as if we are connected directly.
#
# https://support.cloudflare.com/hc/en-us/articles/200170986-How-does-Cloudflare-handle-HTTP-Request-headers
#
# - treat CF-Connecting-IP as overriding X-Forwarded-For
# - parse CF-Visitor to retrieve the scheme
#
# Additionally, we set the scheme from X-Resty-Forwarded-Proto, but that can be
# removed once OpenResty has been fully removed from the stack.
#
# We override the normal rack env so it seems as transparent as possible.
class CloudflareMiddleware

  def initialize(app)
    @app = app
  end

  def call(env)
    if (user_ip = env["HTTP_CF_CONNECTING_IP"].presence)
      env["HTTP_X_FORWARDED_FOR"] = user_ip
      env["REMOTE_ADDR"] = user_ip
      env["HTTP_CLIENT_IP"] = user_ip
    end

    if (cf_scheme = parse_cf_scheme(env).presence)
      env["HTTP_X_FORWARDED_PROTO"] = cf_scheme
      env["HTTP_X_FORWARDED_PORT"] = (cf_scheme == "https" ? "443" : "80")
    end

    @app.call(env)
  end

  private

  def parse_cf_scheme(env)
    return nil if env["HTTP_CF_VISITOR"].blank?
    visitor_struct = JSON.parse(env["HTTP_CF_VISITOR"])
    visitor_struct["scheme"]
  rescue JSON::ParserError
    nil
  end

end

