#!/bin/sh
ENV_FILE="$(pwd)/.env"

usage() {
    cat <<EOF
Usage Commands:
{
    start                 # start cms stack (requires TEACHABLE_SECRET_KEY)
    stop                  # stop the cms stack
    import_db             # imports production database into local set up
    build_image           # builds docker image to use for k8s app
    update_secret         # updates kube secret used by app
}
EOF
}

create_secret() {
    SECRET_EXISTS=$(kubectl get secrets | grep -c "cms-env")

    if [ $SECRET_EXISTS -eq 0 ]; then
        kubectl create secret \
                generic \
                cms-env \
                --from-file=$ENV_FILE
    else
        echo "Secret exists"

    fi
}

update_secret() {
    echo "Updating secret ..."
    kubectl delete secret cms-env
    create_secret
    echo "Updated."
}

set_paths() {
    # puts in path of the repo
    SRC_CODE_PATH=$(pwd)
    SRC_CODE_PATH_EXISTS=$(cat k8s/local/cms*deployment.yaml | grep -c "SRC_CODE_PATH")

    if [ $SRC_CODE_PATH_EXISTS -gt 0 ]; then
        sed -i '' "s|SRC_CODE_PATH|$SRC_CODE_PATH|g" k8s/local/cms*deployment.yaml
    fi
}

start_local() {
    # decrypt secrets
    script/environment-vars.sh decrypt
    # set paths if true
    set_paths
    # create kube secret
    create_secret
    # deploy
    kubectl apply -f k8s/local
}

stop_local() {
    kubectl delete -f k8s/local
}

import_db() {
    DB_CONTAINER_PORT=$(kubectl get service postgres -o 'jsonpath={.spec.ports[].nodePort}')

    ENV_FILE=$ENV_FILE \
    TARGET_DATABASE_URL=teachable-marketing_development \
    DB_PORT=$DB_CONTAINER_PORT \
    script/importdb.sh
}

build_image() {
    docker build . -f Dockerfile.local -t cms-local
}

case $1 in
    start)
        start_local
        ;;

    stop)
        stop_local
        ;;

    import_db)
        import_db
        ;;

    build_image)
        build_image
        ;;

    update_secret)
        update_secret
        ;;

    *)
        usage
        ;;
esac
