require "rails_helper"

RSpec.describe "Tracking link redirects", type: :request do

  let(:user) { create(:user) }

  before(:each) do
    # Keep SeoRedirect from getting in the way
    host! "example.com"
    stub_request(:post, "#{ENV["EVENTABLE_URL"]}/add/cms")
  end

  context "visiting an active link" do

    before(:each) do
      @link = FactoryBot.create(:tracking_link, :published)
      get "/r/#{@link.slug}", headers: {
        "HTTP_REFERER": "https://teachable.com/"
      }
    end

    it "redirects to the tracking links destination url" do
      expect(response.status).to eq(301)
      expect(response.location).to start_with(@link.destination_url)
    end

    it "appends the link id as a query parameter" do
      expect(response_location_query).to include("link_id" => @link.id.to_s)
    end

    it "sets an anonymous id in a js accessible cookie" do
      expect(response_anonymous_cookie_value).not_to be_nil
    end

    it "does not override the anonymous cookie if already set" do
      # cookie populated in visit to link configured in before block
      existing_anonymous_id_cookie = cookies[TrackingLinks::ANONYMOUS_COOKIE]

      get "/r/#{@link.slug}"

      expect(response_anonymous_cookie_value).to eq(existing_anonymous_id_cookie)
    end

    it "logs the redirect event and appends the id" do

      expect(response_location_query).to have_key("event_id")

      redirect_event = TrackingLinkRedirectEvent.find(response_location_query["event_id"])

      expect(redirect_event.anonymous_id).to eq(response_anonymous_cookie_value)
      expect(redirect_event.request_url).to eq(request.url)
      expect(redirect_event.redirected_at).not_to be_nil
      expect(redirect_event.referrer).to eq(request.referer)
      expect(redirect_event.visitor_user_agent).to eq(request.user_agent)
      expect(redirect_event.visitor_ip).to eq(request.ip)
    end

    it "Uses the cloudflare ip if present" do
      # _link = FactoryBot.create(:tracking_link, :published)

      headers = {
        "ACCEPT" => "application/json",
        "REMOTE_ADDR" => "127.0.0.1",
        "HTTP_CF_CONNECTING_IP" => "127.0.0.2"
      }
      get "/r/#{@link.slug}", headers: headers

      expect(response).to have_http_status(:redirect)

      redirect_event = TrackingLinkRedirectEvent.find(response_location_query["event_id"])
      expect(redirect_event.visitor_ip.to_s).to eq("127.0.0.2")
    end
  end

  context "visiting an active link while logged in" do

    let(:current_user) { FactoryBot.create(:user) }

    before do
      token = AAA::Test.generate_token(id: current_user.id)
      get after_login_path, params: { token: token, no_redirect: "true" }
    end

    it "records the users id" do
      link = FactoryBot.create(:tracking_link, :published)

      get tracking_link_redirect_path(slug: link.slug)

      redirect_event = TrackingLinkRedirectEvent.find(response_location_query["event_id"])

      expect(redirect_event.cms_user_id).to eq(current_user.id)
    end

  end

  context "that forwards all query params" do
    it "appends all query params to the destination url" do
      link = TrackingLink.create(forward_all_query_params: true, slug: SecureRandom.hex, destination_url: "https://example.com", status: "published", creator: user, publisher: user, published_at: Time.now)

      get "/r/#{link.slug}?foo=bar"

      expect(response).to have_http_status(301)
      expect(response_location_query).to include("foo" => "bar")
    end

    it "additional query params override forwarded query params" do
      link = TrackingLink.create(forward_all_query_params: true,
        additional_query_params: {
          "conflicting" => "takes_precedence",
        },
        slug: SecureRandom.hex,
        destination_url: "https://example.com",
        status: "published", creator: user,
        publisher: user,
        published_at: Time.now)

      get "/r/#{link.slug}?foo=bar&conflicting=overridden"

      expect(response).to have_http_status(301)
      expect(response_location_query).to include("foo" => "bar", "conflicting" => "takes_precedence")

    end

  end

  context "that forwards a whitelist of query params" do
    it "appends only whitelisted query parameters to the destination url" do
      link = TrackingLink.create(forward_query_params_whitelist: ["good_param"], slug: SecureRandom.hex, destination_url: "https://example.com", status: :published, creator: user, publisher: user, published_at: Time.now)

      get "/r/#{link.slug}?good_param=bar&bad_param=baz"

      expect(response).to have_http_status(301)
      expect(response_location_query).to include("good_param" => "bar")

    end
  end

  it "inactive links return 404" do
    link = FactoryBot.create(:tracking_link)

    get "/r/#{link.slug}"

    expect(response).to have_http_status(:not_found)
  end

  it "visiting a slug with no link return 404" do
    get "/r/nonexistentslug"

    expect(response).to have_http_status(:not_found)
  end

  def response_anonymous_cookie_value
    response.cookies[TrackingLinks::ANONYMOUS_COOKIE]
  end

  def response_location_query
    query_string = URI.parse(@response.location).query
    Rack::Utils.parse_query(query_string)
  end
end
