# frozen_string_literal: true

require "rails_helper"

RSpec.describe TrackingLinks::Updater do
  
  let(:user) { create(:user) }

  describe ".call" do
    it "will return a new instance of a given tracking link if the current record is already published" do
      tracking_link = create(:tracking_link, :published, creator: user, forward_all_query_params: true)
      updater = create(:user)
      new_tracking_link = TrackingLinks::Updater.call(tracking_link: tracking_link, update_attributes: { forward_all_query_params: false }, updater: updater)
      expect(new_tracking_link.slug).to eq tracking_link.slug
      expect(new_tracking_link.status).to eq "published"
      expect(tracking_link.hidden_at).to be_kind_of Time
      expect(tracking_link.hider).to eq updater
      expect(tracking_link.id).not_to eq new_tracking_link.id
    end

    it "will return an existing instance of a given tracking link if the current record has never been published" do
      tracking_link = create(:tracking_link, creator: user, forward_all_query_params: true)
      updater = create(:user)
      new_tracking_link = TrackingLinks::Updater.call(tracking_link: tracking_link, update_attributes: { forward_all_query_params: false }, updater: updater)
      expect(new_tracking_link.forward_all_query_params).to eq false
      expect(tracking_link.id).to eq new_tracking_link.id
    end

    it "will return an existing instance of a given tracking link if the current record is published for the first time" do
      tracking_link = create(:tracking_link, creator: user, forward_all_query_params: true)
      updater = create(:user)
      new_tracking_link = TrackingLinks::Updater.call(tracking_link: tracking_link, update_attributes: { status: :published }, updater: updater)
      expect(new_tracking_link.status).to eq "published"
      expect(new_tracking_link.published_at).to be_kind_of Time
      expect(new_tracking_link.publisher).to eq updater
      expect(tracking_link.id).to eq new_tracking_link.id
    end

    it "will return an existing instance of a given tracking link if the current record is published and status set to hidden" do
      tracking_link = create(:tracking_link, :published, creator: user, forward_all_query_params: true)
      updater = create(:user)
      new_tracking_link = TrackingLinks::Updater.call(tracking_link: tracking_link, update_attributes: { status: :hidden }, updater: updater)
      expect(new_tracking_link.status).to eq "hidden"
      expect(tracking_link.id).to eq new_tracking_link.id
    end

    it "will return the original instance of a tracking link record with appropriate errors if the copy messes up" do
      tracking_link = create(:tracking_link, :published, creator: user, forward_all_query_params: true)
      updater = create(:user)
      bad_url = "https://#{SecureRandom.hex(500)}.com"
      new_tracking_link = TrackingLinks::Updater.call(tracking_link: tracking_link, update_attributes: { forward_all_query_params: false, destination_url: bad_url }, updater: updater)
      expect(new_tracking_link.errors.full_messages).to eq ["Destination url is too long (maximum is 1000 characters)"]
    end

    it "will return a new instance of a given tracking link, with the status set to unpublished, if the original record is hidden" do
      tracking_link = create(:tracking_link, :hidden, creator: user, forward_all_query_params: true)
      updater = create(:user)
      new_tracking_link = TrackingLinks::Updater.call(tracking_link: tracking_link, update_attributes: { forward_all_query_params: false }, updater: updater)
      expect(new_tracking_link.slug).to eq tracking_link.slug
      expect(new_tracking_link.status).to eq "draft"
      expect(new_tracking_link.hidden_at).to eq nil
      expect(new_tracking_link.hider).to eq nil
      expect(new_tracking_link.published_at).to eq nil
      expect(new_tracking_link.publisher).to eq nil
      expect(tracking_link.id).not_to eq new_tracking_link.id
    end
  end

end
